module optical
  !# Module to setup the optical properties of the model
  !#
  !# @note
  !# ![](http://brams.cptec.inpe.br/wp-content/uploads/2015/11/logo-brams.navigation.png "")
  !#
  !# **Brief**: The module was take from BRAMS' model original and modified Carma radiation
  !#  (Community Aerosol and Radiation Model for Atmosphere) and setup the atmosphere optical properties.
  !# Jacobson, M. Z., R. P. Turco, E. J. Jensen, and O. B. Toon (1994), Modeling coagulation among
  !# particles of different composition and size,Atmos Environ.,28 (7), 1327–1338,
  !# doi:10.1016/1352-2310(94)90280-1.
  !# ---
  !# **Documentation**: <http://brams.cptec.inpe.br/documentation/>
  !# ---
  !# **Author**: Rodrigues, L.F. **&#9993;**<mailto:luiz.rodrigues@inpe.br>
  !# ---
  !# **Date**: 2018Aug
  !#
  !# @endnote
  !#
  !# @changes
  !#
  !# +
  !# @endchanges
  !# @bug
  !# No active bugs reported now
  !# @endbug
  !#
  !# @todo
  !#  &#9744; <br/>
  !# @endtodo
  !#
  !# @warning
  !# Now is under CC-GPL License, please see
  !# &copy; <https://creativecommons.org/licenses/GPL/2.0/legalcode.pt>
  !# @endwarning
  !#
  !#--- ----------------------------------------------------------------------------------------
  implicit none

  private

  !parameters
  double precision,parameter :: one = 1.d0
  !#  define double precision 1.0 to be multiplied by literal constants
  !#  that are embedded within parenthes
  double precision,parameter :: small_pc = 1.d-25
  !# define small particle number concentration [ # / x_units / y_units / z_units ]
  double precision,parameter :: almost_zero = 1.d-7
  !#  define smallest possible number such that one + almost_zero > one
  double precision,parameter :: almost_one = one - almost_zero
  !#  define value slightly < 1.0 to test for total evaporation [ dimensionless ]
  double precision,parameter :: epsilon=almost_zero
  !# roundoff error precision
  real,parameter :: g=980.6
  !# Gravity acel.
  real,parameter :: scday=86400.0
  !# Seconds by day
  integer,parameter :: lunoprt = 10
  !#  logical unit number for output print file
  integer,parameter :: nbin = 30
  !#  define # particle radius bins
  integer,parameter :: ngroup = 1
  !#  define # particle groups
  integer,parameter :: nelem = 1
  !#  define # particle elements
  integer,parameter :: nrad=nbin
  !# maximum number of aerosol radius bins;
  integer,parameter :: ngas = 1
  !# define # gases
  integer,parameter:: ntotal_aer = 2

  !  define values of flag used for particle element
  !  type specification:
  integer,parameter :: i_involatile = 0
  !# is cn (involatile) number concentration [#/cm^3]
  integer,parameter :: i_volatile = 1
  !# is water droplet (volatile) number concentration [#/cm^3]
  integer,parameter :: i_coremass = 2
  !# is core mass concentration [g/cm^3re]
  integer,parameter :: i_volcore = 3
  !# is core mass concentration [g/cm^3] of a volatile core
  integer,parameter :: i_core2mom = 4
  !# is core second moment (mass^2) [g^2/cm^3]

  integer,parameter,dimension(nelem) :: itype=(/i_involatile/)
  !# particle type specification array     {setupbins}
  integer,parameter,dimension(ngroup) ::  nelemg=(/1/)
  !#  number of elements in each group (elements in a group can include
  !#  particle number concentration, mass concentrations of cores, and second
  !#  moments of core mass distributions)
  !# define # grid pts in x, y, z directions
  integer,parameter :: nx =  1
  integer,parameter :: ny =  1
  integer,parameter :: nz =  1
  integer,parameter :: nxyz=nx*ny*nz
  !
  integer,parameter :: nz_below = 0
  !# layers in rad xfer model domain underlying aerosol model domain

  real,parameter :: pcore = 10.
  !# kml define the mass percentil of the bc core

  real,parameter,dimension(nelem) :: rhocore=(/1.85/)
  real,parameter,dimension(nelem) :: rhoshell=(/1.31/)
  real,parameter,dimension(nelem) :: rhoelem=(/100.*rhocore(1)*rhoshell(1)/ &
                                     ((100-pcore)*rhocore(1)+pcore*rhoshell(1))/)
  real,parameter :: pi = 3.14159265358979d+0
  !#  define circle constant [ unitless ]
  real,parameter,dimension(ngroup) :: rmin=(/1.e-6/)
  !#  minimum radius for each group (used below to calculate <rmassmin>) [cm]
  real,parameter,dimension(ngroup) :: rmrat=(/1.902/)
  !#  ratio of particle mass between successive bins (one for each group)
  double precision,parameter :: fix_coref = one * 0.01
  !#  define core fraction (for core mass and second moment) used
  !#  when particle number concentrations are limited to small_pc

  integer, parameter :: nsitesaot  = 12
  integer, parameter :: nmodelsaot = 1
  integer,parameter :: nsol = 32
  !#  define # solar wavelength bins
  integer,parameter :: nir = 18
  !#  define # infrared wavelength bins
  integer,parameter :: nwave = nsol + nir
  !#  define total # wavelength bins
  integer,parameter :: nsolp  = 83
  !# number of solar probability intervals;
  integer,parameter :: nirp   = 71
  !# number of infrared probability intervals;
  integer,parameter :: ntotal = nsolp + nirp
  !# total number of probability intervals;
  integer,parameter :: ngauss = 3
  !# total number of gauss quadrature points;
  integer,parameter :: iprocopio = 1
  integer,parameter :: ir_aerad =1
  !# long wave radiation is on

  !!!!! parameter arrays from globrad !!!!!

  real,parameter,dimension(nirp)  :: &
        contnm=       (/0.      ,    0.     ,     0.     ,     0.     ,      0.     ,     0.      ,&
                        0.      ,    0.     ,     0.     ,     0.     ,      0.     ,     0.      ,&
                        0.      ,    0.     ,     0.     ,     0.     ,      0.     ,     0.      ,&
                        0.      ,    0.     ,     0.     ,     0.     ,      0.     ,     0.      ,&
                        0.      ,    0.     ,     0.     ,     0.     ,      0.     ,     0.      ,&
                        0.      ,    0.     ,     0.     ,     0.     ,      0.     ,     0.      ,&
                        2.e-7   ,    2.e-7  ,     2.e-7  ,     2.e-7  ,      2.e-7  ,     2.e-7   ,&
                        2.e-7   ,    2.e-7  ,     2.e-7  ,     2.e-7  ,      2.e-7  ,     2.e-7   ,&
                        4.266e-7,    4.266e-7,    4.266e-7,    4.266e-7,     4.266e-7,    4.266e-7,&
                        4.266e-7,    4.266e-7,    4.266e-7,    9.65e-7 ,     9.65e-7 ,    9.65e-7 ,&
                        2.423e-6,    2.423e-6,    2.423e-6,    2.423e-6,     5.765e-6,    5.765e-6,&
                        5.765e-6,    5.765e-6,    1.482e-5,    1.482e-5,     1.482e-5/)
  !# bergstrom's water vapor continuum fix
  real,parameter,dimension(ntotal) :: &
        ta=   (/1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000, &
                1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000, &
                1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000, &
                1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000, &
                1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000, &
                1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000, &
                1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000, &
                1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000, &
                1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000, &
                1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000, &
                1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000, &
                1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000, &
                1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000, &
                1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000, &
                1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000   ,  1.500000, &
                0.700000   ,  0.700000   ,  0.700000   ,  0.700000   ,  0.700000   ,  0.700000, &
                0.700000   ,  0.700000   ,  0.700000   ,  0.700000   ,  0.700000   ,  0.700000, &
                0.700000   ,  0.700000   ,  0.700000   ,  0.700000   ,  0.700000   ,  0.700000, &
                0.700000   ,  0.700000   ,  0.700000   ,  0.700000   ,  0.700000   ,  0.240000, &
                0.240000   ,  0.240000   ,  0.240000   ,  0.240000   ,  0.240000   ,  0.240000, &
                0.240000   ,  0.240000   ,  0.240000   ,  0.240000   ,  0.240000   ,  0.240000, &
                0.240000   ,  0.240000   ,  0.240000   ,  0.240000   ,  0.240000   ,  0.270000, &
                0.270000   ,  0.270000   ,  0.270000   ,  0.270000   ,  0.270000   ,  0.270000, &
                0.270000   ,  0.270000   ,  0.270000   ,  0.270000   ,  0.270000   ,  0.270000, &
                0.270000   ,  0.270000   ,  0.270000   ,  0.270000   ,  0.270000   ,  0.270000, &
                0.270000   ,  0.270000   ,  0.270000   ,  0.270000/)
  real,parameter,dimension(ntotal) :: &
        tb=   (/0.500000   ,   0.500000   ,   0.500000   ,   0.500000 ,   0.500000   ,   0.500000, &
                0.500000   ,   0.500000   ,   0.500000   ,   0.500000 ,   0.500000   ,   0.500000, &
                0.500000   ,   0.500000   ,   0.500000   ,   0.500000 ,   0.780000   ,   0.780000, &
                0.780000   ,   0.780000   ,   0.780000   ,   0.780000 ,   0.780000   ,   0.780000, &
                0.780000   ,   0.780000   ,   0.780000   ,   0.780000 ,   0.780000   ,   0.780000, &
                0.780000   ,   0.780000   ,   0.780000   ,   0.780000 ,   0.780000   ,   0.780000, &
                1.130000   ,   1.130000   ,   1.130000   ,   1.130000 ,   1.130000   ,   1.130000, &
                1.130000   ,   1.130000   ,   1.130000   ,   1.130000 ,   1.130000   ,   1.130000, &
                1.130000   ,   1.130000   ,   1.130000   ,   1.130000 ,   2.000000   ,   2.000000, &
                2.000000   ,   2.000000   ,   2.000000   ,   2.000000 ,   2.000000   ,   2.000000, &
                2.000000   ,   2.000000   ,   2.000000   ,   2.000000 ,   2.000000   ,   2.000000, &
                2.000000   ,   2.000000   ,   2.000000   ,   2.000000 ,   2.000000   ,   2.000000, &
                2.000000   ,   2.000000   ,   2.000000   ,   2.000000 ,   2.000000   ,  -0.120000, &
               -0.120000   ,  -0.120000   ,  -0.120000   ,  -0.120000 ,  -0.120000   ,  -0.120000, &
               -0.120000   ,  -0.120000   ,  -0.120000   ,  -0.120000 ,  -0.120000   ,  -0.120000, &
               -0.120000   ,  -0.120000   ,  -0.120000   ,  -0.120000 ,  -0.120000   ,  -0.120000, &
               -0.120000   ,  -0.120000   ,  -0.120000   ,  -0.120000 ,  -0.120000   ,  -0.120000, &
               -0.120000   ,  -0.120000   ,  -0.120000   ,  -0.120000 ,  -0.120000   ,  -0.120000, &
               -0.120000   ,  -0.120000   ,  -0.120000   ,  -0.120000 ,  -0.120000   ,  -0.008000, &
               -0.008000   ,  -0.008000   ,  -0.008000   ,  -0.008000 ,  -0.008000   ,  -0.008000, &
               -0.008000   ,  -0.008000   ,  -0.008000   ,  -0.008000 ,  -0.008000   ,  -0.008000, &
               -0.008000   ,  -0.008000   ,  -0.008000   ,  -0.008000 ,  -0.008000   ,  -0.011000, &
               -0.011000   ,  -0.011000   ,  -0.011000   ,  -0.011000 ,  -0.011000   ,  -0.011000, &
               -0.011000   ,  -0.011000   ,  -0.011000   ,  -0.011000 ,  -0.011000   ,  -0.011000, &
               -0.011000   ,  -0.011000   ,  -0.011000   ,  -0.011000 ,  -0.011000   ,  -0.011000, &
               -0.011000   ,  -0.011000   ,  -0.011000   ,  -0.011000/)
  real,parameter,dimension(ntotal) :: &
        wa=   (/5.580e-7   ,  5.580e-7   ,  5.580e-7   ,  5.580e-7   ,  5.580e-7   ,  5.580e-7, &
                5.580e-7   ,  5.580e-7   ,  5.580e-7   ,  5.580e-7   ,  5.580e-7   ,  5.580e-7, &
                5.580e-7   ,  5.580e-7   ,  5.580e-7   ,  5.580e-7   ,  2.180e-5   ,  2.180e-5, &
                2.180e-5   ,  2.180e-5   ,  2.180e-5   ,  2.180e-5   ,  2.180e-5   ,  2.180e-5, &
                2.180e-5   ,  2.180e-5   ,  2.180e-5   ,  2.180e-5   ,  2.180e-5   ,  2.180e-5, &
                2.180e-5   ,  2.180e-5   ,  2.180e-5   ,  2.180e-5   ,  2.180e-5   ,  2.180e-5, &
                8.550e-4   ,  8.550e-4   ,  8.550e-4   ,  8.550e-4   ,  8.550e-4   ,  8.550e-4, &
                8.550e-4   ,  8.550e-4   ,  8.550e-4   ,  8.550e-4   ,  8.550e-4   ,  8.550e-4, &
                8.550e-4   ,  8.550e-4   ,  8.550e-4   ,  8.550e-4   ,  1.940e-1   ,  1.940e-1, &
                1.940e-1   ,  1.940e-1   ,  1.940e-1   ,  1.940e-1   ,  1.940e-1   ,  1.940e-1, &
                1.940e-1   ,  1.940e-1   ,  1.940e-1   ,  1.940e-1   ,  1.940e-1   ,  1.940e-1, &
                1.940e-1   ,  1.940e-1   ,  1.940e-1   ,  1.940e-1   ,  1.940e-1   ,  1.940e-1, &
                1.940e-1   ,  1.940e-1   ,  1.940e-1   ,  1.940e-1   ,  1.940e-1   ,  0.110000, &
                0.110000   ,  0.110000   ,  0.110000   ,  0.110000   ,  0.110000   ,  0.110000, &
                0.110000   ,  0.110000   ,  0.110000   ,  0.110000   ,  0.110000   ,  0.110000, &
                0.110000   ,  0.110000   ,  0.110000   ,  0.110000   ,  0.110000   ,  0.110000, &
                0.110000   ,  0.110000   ,  0.110000   ,  0.110000   ,  0.110000   ,  0.110000, &
                0.110000   ,  0.110000   ,  0.110000   ,  0.110000   ,  0.110000   ,  0.110000, &
                0.110000   ,  0.110000   ,  0.110000   ,  0.110000   ,  0.110000   ,  0.430000, &
                0.430000   ,  0.430000   ,  0.430000   ,  0.430000   ,  0.430000   ,  0.430000, &
                0.430000   ,  0.430000   ,  0.430000   ,  0.430000   ,  0.430000   ,  0.430000, &
                0.430000   ,  0.430000   ,  0.430000   ,  0.430000   ,  0.430000   ,  0.740000, &
                0.740000   ,  0.740000   ,  0.740000   ,  0.740000   ,  0.740000   ,  0.740000, &
                0.740000   ,  0.740000   ,  0.740000   ,  0.740000   ,  0.740000   ,  0.740000, &
                0.740000   ,  0.740000   ,  0.740000   ,  0.740000   ,  0.740000   ,  0.740000, &
                0.740000   ,  0.740000   ,  0.740000   ,  0.740000/)
  real,parameter,dimension(ntotal) :: &
        wb=   (/-1.25e-7   ,   -1.25e-7   ,   -1.25e-7   ,   -1.25e-7 ,   -1.25e-7   ,   -1.25e-7, &
                -1.25e-7   ,   -1.25e-7   ,   -1.25e-7   ,   -1.25e-7 ,   -1.25e-7   ,   -1.25e-7, &
                -1.25e-7   ,   -1.25e-7   ,   -1.25e-7   ,   -1.25e-7 ,   -2.25e-5   ,   -2.25e-5, &
                -2.25e-5   ,   -2.25e-5   ,   -2.25e-5   ,   -2.25e-5 ,   -2.25e-5   ,   -2.25e-5, &
                -2.25e-5   ,   -2.25e-5   ,   -2.25e-5   ,   -2.25e-5 ,   -2.25e-5   ,   -2.25e-5, &
                -2.25e-5   ,   -2.25e-5   ,   -2.25e-5   ,   -2.25e-5 ,   -2.25e-5   ,   -2.25e-5, &
                -1.28e-3   ,   -1.28e-3   ,   -1.28e-3   ,   -1.28e-3 ,   -1.28e-3   ,   -1.28e-3, &
                -1.28e-3   ,   -1.28e-3   ,   -1.28e-3   ,   -1.28e-3 ,   -1.28e-3   ,   -1.28e-3, &
                -1.28e-3   ,   -1.28e-3   ,   -1.28e-3   ,   -1.28e-3 ,   -8.04e-3   ,   -8.04e-3, &
                -8.04e-3   ,   -8.04e-3   ,   -8.04e-3   ,   -8.04e-3 ,   -8.04e-3   ,   -8.04e-3, &
                -8.04e-3   ,   -8.04e-3   ,   -8.04e-3   ,   -8.04e-3 ,   -8.04e-3   ,   -8.04e-3, &
                -8.04e-3   ,   -8.04e-3   ,   -8.04e-3   ,   -8.04e-3 ,   -8.04e-3   ,   -8.04e-3, &
                -8.04e-3   ,   -8.04e-3   ,   -8.04e-3   ,   -8.04e-3 ,   -8.04e-3   ,  -0.018000, &
               -0.018000   ,  -0.018000   ,  -0.018000   ,  -0.018000 ,  -0.018000   ,  -0.018000, &
               -0.018000   ,  -0.018000   ,  -0.018000   ,  -0.018000 ,  -0.018000   ,  -0.018000, &
               -0.018000   ,  -0.018000   ,  -0.018000   ,  -0.018000 ,  -0.018000   ,  -0.018000, &
               -0.018000   ,  -0.018000   ,  -0.018000   ,  -0.018000 ,  -0.018000   ,  -0.018000, &
               -0.018000   ,  -0.018000   ,  -0.018000   ,  -0.018000 ,  -0.018000   ,  -0.018000, &
               -0.018000   ,  -0.018000   ,  -0.018000   ,  -0.018000 ,  -0.018000   ,   0.003000, &
                0.003000   ,   0.003000   ,   0.003000   ,   0.003000 ,   0.003000   ,   0.003000, &
                0.003000   ,   0.003000   ,   0.003000   ,   0.003000 ,   0.003000   ,   0.003000, &
                0.003000   ,   0.003000   ,   0.003000   ,   0.003000 ,   0.003000   ,   0.014000, &
                0.014000   ,   0.014000   ,   0.014000   ,   0.014000 ,   0.014000   ,   0.014000, &
                0.014000   ,   0.014000   ,   0.014000   ,   0.014000 ,   0.014000   ,   0.014000, &
                0.014000   ,   0.014000   ,   0.014000   ,   0.014000 ,   0.014000   ,   0.014000, &
                0.014000   ,   0.014000   ,   0.014000   ,   0.014000/)
  real,parameter,dimension(ntotal) :: &
        ga=   (/0.841000   ,   0.841000   ,   0.841000   ,   0.841000 ,   0.841000   ,   0.841000, &
                0.841000   ,   0.841000   ,   0.841000   ,   0.841000 ,   0.841000   ,   0.841000, &
                0.841000   ,   0.841000   ,   0.841000   ,   0.841000 ,   0.821000   ,   0.821000, &
                0.821000   ,   0.821000   ,   0.821000   ,   0.821000 ,   0.821000   ,   0.821000, &
                0.821000   ,   0.821000   ,   0.821000   ,   0.821000 ,   0.821000   ,   0.821000, &
                0.821000   ,   0.821000   ,   0.821000   ,   0.821000 ,   0.821000   ,   0.821000, &
                0.786000   ,   0.786000   ,   0.786000   ,   0.786000 ,   0.786000   ,   0.786000, &
                0.786000   ,   0.786000   ,   0.786000   ,   0.786000 ,   0.786000   ,   0.786000, &
                0.786000   ,   0.786000   ,   0.786000   ,   0.786000 ,   0.820000   ,   0.820000, &
                0.820000   ,   0.820000   ,   0.820000   ,   0.820000 ,   0.820000   ,   0.820000, &
                0.820000   ,   0.820000   ,   0.820000   ,   0.820000 ,   0.820000   ,   0.820000, &
                0.820000   ,   0.820000   ,   0.820000   ,   0.820000 ,   0.820000   ,   0.820000, &
                0.820000   ,   0.820000   ,   0.820000   ,   0.820000 ,   0.820000   ,   0.840000, &
                0.840000   ,   0.840000   ,   0.840000   ,   0.840000 ,   0.840000   ,   0.840000, &
                0.840000   ,   0.840000   ,   0.840000   ,   0.840000 ,   0.840000   ,   0.840000, &
                0.840000   ,   0.840000   ,   0.840000   ,   0.840000 ,   0.840000   ,   0.840000, &
                0.840000   ,   0.840000   ,   0.840000   ,   0.840000 ,   0.840000   ,   0.840000, &
                0.840000   ,   0.840000   ,   0.840000   ,   0.840000 ,   0.840000   ,   0.840000, &
                0.840000   ,   0.840000   ,   0.840000   ,   0.840000 ,   0.840000   ,   0.250000, &
                0.250000   ,   0.250000   ,   0.250000   ,   0.250000 ,   0.250000   ,   0.250000, &
                0.250000   ,   0.250000   ,   0.250000   ,   0.250000 ,   0.250000   ,   0.250000, &
                0.250000   ,   0.250000   ,   0.250000   ,   0.250000 ,   0.250000   ,   0.500000, &
                0.500000   ,   0.500000   ,   0.500000   ,   0.500000 ,   0.500000   ,   0.500000, &
                0.500000   ,   0.500000   ,   0.500000   ,   0.500000 ,   0.500000   ,   0.500000, &
                0.500000   ,   0.500000   ,   0.500000   ,   0.500000 ,   0.500000   ,   0.500000, &
                0.500000   ,   0.500000   ,   0.500000   ,   0.500000/)
  real,parameter,dimension(ntotal) :: &
        gb=   (/0.002080   ,   0.002080   ,   0.002080   ,   0.002080 ,   0.002080   ,   0.002080, &
                0.002080   ,   0.002080   ,   0.002080   ,   0.002080 ,   0.002080   ,   0.002080, &
                0.002080   ,   0.002080   ,   0.002080   ,   0.002080 ,   0.003060   ,   0.003060, &
                0.003060   ,   0.003060   ,   0.003060   ,   0.003060 ,   0.003060   ,   0.003060, &
                0.003060   ,   0.003060   ,   0.003060   ,   0.003060 ,   0.003060   ,   0.003060, &
                0.003060   ,   0.003060   ,   0.003060   ,   0.003060 ,   0.003060   ,   0.003060, &
                0.005320   ,   0.005320   ,   0.005320   ,   0.005320 ,   0.005320   ,   0.005320, &
                0.005320   ,   0.005320   ,   0.005320   ,   0.005320 ,   0.005320   ,   0.005320, &
                0.005320   ,   0.005320   ,   0.005320   ,   0.005320 ,   0.005590   ,   0.005590, &
                0.005590   ,   0.005590   ,   0.005590   ,   0.005590 ,   0.005590   ,   0.005590, &
                0.005590   ,   0.005590   ,   0.005590   ,   0.005590 ,   0.005590   ,   0.005590, &
                0.005590   ,   0.005590   ,   0.005590   ,   0.005590 ,   0.005590   ,   0.005590, &
                0.005590   ,   0.005590   ,   0.005590   ,   0.005590 ,   0.005590   ,   0.003000, &
                0.003000   ,   0.003000   ,   0.003000   ,   0.003000 ,   0.003000   ,   0.003000, &
                0.003000   ,   0.003000   ,   0.003000   ,   0.003000 ,   0.003000   ,   0.003000, &
                0.003000   ,   0.003000   ,   0.003000   ,   0.003000 ,   0.003000   ,   0.003000, &
                0.003000   ,   0.003000   ,   0.003000   ,   0.003000 ,   0.003000   ,   0.003000, &
                0.003000   ,   0.003000   ,   0.003000   ,   0.003000 ,   0.003000   ,   0.003000, &
                0.003000   ,   0.003000   ,   0.003000   ,   0.003000 ,   0.003000   ,  -0.080000, &
               -0.080000   ,  -0.080000   ,  -0.080000   ,  -0.080000 ,  -0.080000   ,  -0.080000, &
               -0.080000   ,  -0.080000   ,  -0.080000   ,  -0.080000 ,  -0.080000   ,  -0.080000, &
               -0.080000   ,  -0.080000   ,  -0.080000   ,  -0.080000 ,  -0.080000   ,  -0.070000, &
               -0.070000   ,  -0.070000   ,  -0.070000   ,  -0.070000 ,  -0.070000   ,  -0.070000, &
               -0.070000   ,  -0.070000   ,  -0.070000   ,  -0.070000 ,  -0.070000   ,  -0.070000, &
               -0.070000   ,  -0.070000   ,  -0.070000   ,  -0.070000 ,  -0.070000   ,  -0.070000, &
               -0.070000   ,  -0.070000   ,  -0.070000   ,  -0.070000/)
  real,parameter,dimension(ntotal) :: &
        tia=  (/30.6  ,  30.6 ,  30.6   ,  30.6   ,  30.6   ,  30.6 ,&
                30.6  ,  30.6 ,  30.6   ,  30.6   ,  30.6   ,  30.6 ,&
                30.6  ,  30.6 ,  30.6   ,  30.6   ,  30.6   ,  30.6 ,&
                30.6  ,  30.6 ,  30.6   ,  30.6   ,  30.6   ,  30.6 ,&
                30.6  ,  30.6 ,  30.6   ,  30.6   ,  30.6   ,  30.6 ,&
                30.6  ,  30.6 ,  30.6   ,  30.6   ,  30.6   ,  30.6 ,&
                30.6  ,  30.6 ,  30.6   ,  30.6   ,  30.6   ,  30.6 ,&
                30.6  ,  30.6 ,  30.6   ,  30.6   ,  30.6   ,  30.6 ,&
                30.6  ,  30.6 ,  30.6   ,  30.6   ,  30.6   ,  30.6 ,&
                30.6  ,  30.6 ,  30.6   ,  30.6   ,  30.6   ,  30.6 ,&
                30.6  ,  30.6 ,  30.6   ,  30.6   ,  30.6   ,  30.6 ,&
                30.6  ,  30.6 ,  30.6   ,  30.6   ,  30.6   ,  30.6 ,&
                30.6  ,  30.6 ,  30.6   ,  30.6   ,  30.6   ,  63.1 ,&
                63.1  ,  63.1 ,  63.1   ,  63.1   ,  63.1   ,  63.1 ,&
                63.1  ,  63.1 ,  63.1   ,  63.1   ,  63.1   ,  63.1 ,&
                63.1  ,  63.1 ,  63.1   ,  63.1   ,  63.1   ,  63.1 ,&
                63.1  ,  63.1 ,  63.1   ,  63.1   ,  63.1   ,  63.1 ,&
                63.1  ,  63.1 ,  63.1   ,  63.1   ,  63.1   ,  63.1 ,&
                63.1  ,  63.1 ,  63.1   ,  63.1   ,  63.1   ,  63.1 ,&
                63.1  ,  63.1 ,  63.1   ,  63.1   ,  63.1   ,  63.1 ,&
                63.1  ,  63.1 ,  63.1   ,  63.1   ,  63.1   ,  63.1 ,&
                63.1  ,  63.1 ,  63.1   ,  63.1   ,  63.1   ,  63.1 ,&
                63.1  ,  63.1 ,  63.1   ,  63.1   ,  63.1   ,  63.1 ,&
                63.1  ,  63.1 ,  63.1   ,  63.1   ,  63.1   ,  63.1 ,&
                63.1  ,  63.1 ,  63.1   ,  63.1   ,  63.1   ,  63.1 ,&
                63.1  ,  63.1 ,  63.1   ,  63.1/)
  real,parameter,dimension(ntotal) :: &
        tib=  (/254.8 ,  254.8   ,  254.8   ,  254.8   ,  254.8   ,  254.8 , &
                254.8 ,  254.8   ,  254.8   ,  254.8   ,  254.8   ,  254.8 , &
                254.8 ,  254.8   ,  254.8   ,  254.8   ,  254.8   ,  254.8 , &
                254.8 ,  254.8   ,  254.8   ,  254.8   ,  254.8   ,  254.8 , &
                254.8 ,  254.8   ,  254.8   ,  254.8   ,  254.8   ,  254.8 , &
                254.8 ,  254.8   ,  254.8   ,  254.8   ,  254.8   ,  254.8 , &
                254.8 ,  254.8   ,  254.8   ,  254.8   ,  254.8   ,  254.8 , &
                254.8 ,  254.8   ,  254.8   ,  254.8   ,  254.8   ,  254.8 , &
                254.8 ,  254.8   ,  254.8   ,  254.8   ,  254.8   ,  254.8 , &
                254.8 ,  254.8   ,  254.8   ,  254.8   ,  254.8   ,  254.8 , &
                254.8 ,  254.8   ,  254.8   ,  254.8   ,  254.8   ,  254.8 , &
                254.8 ,  254.8   ,  254.8   ,  254.8   ,  254.8   ,  254.8 , &
                254.8 ,  254.8   ,  254.8   ,  254.8   ,  254.8   ,  265.0 , &
                265.0 ,  265.0   ,  265.0   ,  265.0   ,  265.0   ,  265.0 , &
                265.0 ,  265.0   ,  265.0   ,  265.0   ,  265.0   ,  265.0 , &
                265.0 ,  265.0   ,  265.0   ,  265.0   ,  265.0   ,  265.0 , &
                265.0 ,  265.0   ,  265.0   ,  265.0   ,  265.0   ,  265.0 , &
                265.0 ,  265.0   ,  265.0   ,  265.0   ,  265.0   ,  265.0 , &
                265.0 ,  265.0   ,  265.0   ,  265.0   ,  265.0   ,  265.0 , &
                265.0 ,  265.0   ,  265.0   ,  265.0   ,  265.0   ,  265.0 , &
                265.0 ,  265.0   ,  265.0   ,  265.0   ,  265.0   ,  265.0 , &
                265.0 ,  265.0   ,  265.0   ,  265.0   ,  265.0   ,  265.0 , &
                265.0 ,  265.0   ,  265.0   ,  265.0   ,  265.0   ,  265.0 , &
                265.0 ,  265.0   ,  265.0   ,  265.0   ,  265.0   ,  265.0 , &
                265.0 ,  265.0   ,  265.0   ,  265.0   ,  265.0   ,  265.0 , &
                265.0 ,  265.0   ,  265.0   ,  265.0/)
  real,parameter,dimension(ntotal) :: &
        wia=  (/0.000000   ,   0.000000   ,   0.000000   ,   0.000000 ,   0.000000   ,   0.000000 , &
                0.000000   ,   0.000000   ,   0.000000   ,   0.000000 ,   0.000000   ,   0.000000 , &
                0.000000   ,   0.000000   ,   0.000000   ,   0.000000 ,   0.000000   ,   0.000000 , &
                0.000000   ,   0.000000   ,   0.000000   ,   0.000000 ,   0.000000   ,   0.000000 , &
                0.000000   ,   0.000000   ,   0.000000   ,   0.000000 ,   0.000000   ,   0.000000 , &
                0.000000   ,   0.001676   ,   0.004115   ,   0.004115 ,   0.004115   ,   0.004115 , &
                0.154500   ,   0.154500   ,   0.154500   ,   0.154500 ,   0.278800   ,   0.278800 , &
                0.278800   ,   0.248300   ,   0.248300   ,   0.248300 ,   0.248300   ,   0.248300 , &
                0.248300   ,   0.248300   ,   0.248300   ,   0.196500 ,   0.497000   ,   0.497000 , &
                0.497000   ,   0.497000   ,   0.497000   ,   0.497000 ,   0.497000   ,   0.497000 , &
                0.497000   ,   0.497000   ,   0.497000   ,   0.497000 ,   0.497000   ,   0.497000 , &
                0.497000   ,   0.497000   ,   0.497000   ,   0.497000 ,   0.497000   ,   0.497000 , &
                0.462400   ,   0.462400   ,   0.462400   ,   0.462400 ,   0.495400   ,   0.000000 , &
                0.000000   ,   0.000000   ,   0.000000   ,   0.000000 ,   0.000000   ,   0.000000 , &
                0.000000   ,   0.000000   ,   0.000000   ,   0.000000 ,   0.000000   ,   0.000000 , &
                0.000000   ,   0.000000   ,   0.000000   ,   0.000000 ,   0.000000   ,   0.000000 , &
                0.000000   ,   0.000000   ,   0.000000   ,   0.000000 ,   0.000000   ,   0.000000 , &
                0.000000   ,   0.000000   ,   0.000000   ,   0.000000 ,   0.000000   ,   0.000000 , &
                0.000000   ,   0.000000   ,   0.000000   ,   0.000000 ,   0.000000   ,   0.000000 , &
                0.000000   ,   0.000000   ,   0.000000   ,   0.000000 ,   0.000000   ,   0.000000 , &
                0.000000   ,   0.000000   ,   0.000000   ,   0.000000 ,   0.000000   ,   0.000000 , &
                0.000000   ,   0.000000   ,   0.000000   ,   0.000000 ,   0.000000   ,   0.000000 , &
                0.000000   ,   0.000000   ,   0.000000   ,   0.000000 ,   0.000000   ,   0.000000 , &
                0.000000   ,   0.000000   ,   0.000000   ,   0.000000 ,   0.000000   ,   0.000000 , &
                0.000000   ,   0.000000   ,   0.000000   ,   0.000000 ,   0.000000   ,   0.000000 , &
                0.000000   ,   0.000000   ,   0.000000   ,   0.000000/)
  real,parameter,dimension(ntotal) :: &
        wib=  (/0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.093820   ,  0.226700   ,  0.226700   ,  0.226700   ,  0.226700 , &
                0.221100   ,  0.221100   ,  0.221100   ,  0.221100   ,  0.194500   ,  0.194500 , &
                0.194500   ,  0.200900   ,  0.200900   ,  0.200900   ,  0.200900   ,  0.200900 , &
                0.200900   ,  0.200900   ,  0.200900   ,  0.213000   ,  0.013410   ,  0.013410 , &
                0.013410   ,  0.013410   ,  0.013410   ,  0.013410   ,  0.013410   ,  0.013410 , &
                0.013410   ,  0.013410   ,  0.013410   ,  0.013410   ,  0.013410   ,  0.013410 , &
                0.013410   ,  0.013410   ,  0.013410   ,  0.013410   ,  0.013410   ,  0.013410 , &
                0.000740   ,  0.000740   ,  0.000740   ,  0.000740   ,  0.032410   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000/)
  real,parameter,dimension(ntotal) :: &
        gia=  (/0.837100   ,  0.837100   ,  0.837100   ,  0.837100   ,  0.837100   ,  0.845300 , &
                0.852100   ,  0.852100   ,  0.852600   ,  0.853900   ,  0.853900   ,  0.854100 , &
                0.854900   ,  0.854600   ,  0.854600   ,  0.854600   ,  0.855800   ,  0.855800 , &
                0.855800   ,  0.855800   ,  0.855400   ,  0.855400   ,  0.855400   ,  0.857100 , &
                0.857100   ,  0.857100   ,  0.857100   ,  0.858800   ,  0.858800   ,  0.858800 , &
                0.858800   ,  0.861100   ,  0.863100   ,  0.863100   ,  0.863100   ,  0.863100 , &
                0.901800   ,  0.901800   ,  0.901800   ,  0.901800   ,  0.930500   ,  0.930500 , &
                0.930500   ,  0.924900   ,  0.924900   ,  0.924900   ,  0.924900   ,  0.924900 , &
                0.924900   ,  0.924900   ,  0.924900   ,  0.923200   ,  0.993800   ,  0.993800 , &
                0.993800   ,  0.993800   ,  0.993800   ,  0.993800   ,  0.993800   ,  0.993800 , &
                0.993800   ,  0.993800   ,  0.993800   ,  0.993800   ,  0.993800   ,  0.993800 , &
                0.993800   ,  0.993800   ,  0.993800   ,  0.993800   ,  0.993800   ,  0.993800 , &
                0.959400   ,  0.959400   ,  0.959400   ,  0.959400   ,  0.980400   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000/)
  real,parameter,dimension(ntotal) :: &
        gib=  (/0.016830   ,  0.016830   ,  0.016830   ,  0.016730   ,  0.016730   ,  0.016510 , &
                0.016290   ,  0.016290   ,  0.016040   ,  0.016160   ,  0.016160   ,  0.016090 , &
                0.016140   ,  0.016000   ,  0.016000   ,  0.016000   ,  0.015940   ,  0.015940 , &
                0.015940   ,  0.015940   ,  0.015940   ,  0.015940   ,  0.015940   ,  0.015920 , &
                0.015920   ,  0.015920   ,  0.015920   ,  0.016010   ,  0.016010   ,  0.016010 , &
                0.016010   ,  0.016110   ,  0.016070   ,  0.016070   ,  0.016070   ,  0.016070 , &
                0.019120   ,  0.019120   ,  0.019120   ,  0.019120   ,  0.020560   ,  0.020560 , &
                0.020560   ,  0.019760   ,  0.019760   ,  0.019760   ,  0.019760   ,  0.019760 , &
                0.019760   ,  0.019760   ,  0.019760   ,  0.016970   ,  0.002930   ,  0.002930 , &
                0.002930   ,  0.002930   ,  0.002930   ,  0.002930   ,  0.002930   ,  0.002930 , &
                0.002930   ,  0.002930   ,  0.002930   ,  0.002930   ,  0.002930   ,  0.002930 , &
                0.002930   ,  0.002930   ,  0.002930   ,  0.002930   ,  0.002930   ,  0.002930 , &
                0.002750   ,  0.002750   ,  0.002750   ,  0.002750   ,  0.010300   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000/)
  real,parameter,dimension(ntotal) :: &
        alpha=(/0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.081e-3 , &
                0.081e-3   ,  0.081e-3   ,  0.081e-3   ,  0.000268   ,  0.000268   ,  0.000268 , &
                0.000268   ,  0.000652   ,  0.000715   ,  0.000715   ,  0.000715   ,  0.000715 , &
                0.033500   ,  0.033500   ,  0.033500   ,  0.033500   ,  0.071900   ,  0.071900 , &
                0.071900   ,  0.061200   ,  0.061200   ,  0.061200   ,  0.061200   ,  0.061200 , &
                0.061200   ,  0.061200   ,  0.061200   ,  0.044900   ,  0.207000   ,  0.207000 , &
                0.207000   ,  0.207000   ,  0.207000   ,  0.207000   ,  0.207000   ,  0.207000 , &
                0.207000   ,  0.207000   ,  0.207000   ,  0.207000   ,  0.207000   ,  0.207000 , &
                0.207000   ,  0.207000   ,  0.207000   ,  0.207000   ,  0.207000   ,  0.207000 , &
                0.001840   ,  0.001840   ,  0.001840   ,  0.001840   ,  0.273000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000/)
  real,parameter,dimension(ntotal) :: &
        gama= (/0.032710   ,  0.032710   ,  0.032710   ,  0.032530   ,  0.032530   ,  0.032300 , &
                0.032920   ,  0.032920   ,  0.032470   ,  0.032600   ,  0.032600   ,  0.032550 , &
                0.032770   ,  0.033190   ,  0.033190   ,  0.033190   ,  0.033160   ,  0.033160 , &
                0.033160   ,  0.033160   ,  0.033160   ,  0.033160   ,  0.033160   ,  0.033720 , &
                0.033720   ,  0.033720   ,  0.033720   ,  0.034410   ,  0.034410   ,  0.034410 , &
                0.034410   ,  0.035370   ,  0.036530   ,  0.036530   ,  0.036530   ,  0.036530 , &
                0.049990   ,  0.049990   ,  0.049990   ,  0.049990   ,  0.063600   ,  0.063600 , &
                0.063600   ,  0.060450   ,  0.060450   ,  0.060450   ,  0.060450   ,  0.060450 , &
                0.060450   ,  0.060450   ,  0.060450   ,  0.050150   ,  0.044600   ,  0.044600 , &
                0.044600   ,  0.044600   ,  0.044600   ,  0.044600   ,  0.044600   ,  0.044600 , &
                0.044600   ,  0.044600   ,  0.044600   ,  0.044600   ,  0.044600   ,  0.044600 , &
                0.044600   ,  0.044600   ,  0.044600   ,  0.044600   ,  0.044600   ,  0.044600 , &
                0.029370   ,  0.029370   ,  0.029370   ,  0.029370   ,  0.146200   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000   ,  0.000000 , &
                0.000000   ,  0.000000   ,  0.000000   ,  0.000000/)
  real,parameter,dimension(nwave+1) :: &
        wave= (/0.256,  0.280,  0.296,  0.319,  0.335,  0.365,  0.420,  0.440,&
                0.470,  0.482,  0.500,  0.550,  0.598,  0.660,  0.670,  0.690,&
                0.762,  0.719,  0.813,  0.862,  0.926,  1.005,  1.111,  1.333,&
                1.562,  1.770,  2.051,  2.210,  2.584,  3.284,  3.809,  4.292,&
                4.546,  4.878,  5.128,  5.405,  5.714,  6.061,  6.452,  6.897,&
                7.407,  8.333,  9.009, 10.309, 12.500, 13.889, 16.667, 20.000,&
               26.316, 35.714, 62.50/)

  real,parameter,dimension(ngauss) :: gangle= (/0.2123405382, 0.5905331356, 0.9114120405/)
  !# gauss angles for gaussian integration moments (use first moment values) n=3
  real,parameter,dimension(ngauss) :: gratio= (/0.4679139346, 0.3607615730, 0.1713244924/)
  !# gauss ratio for gaussian integration moments (use first moment values) n=3
  real,parameter,dimension(ngauss) :: gweight=(/0.0698269799, 0.2292411064, 0.2009319137/)
  !# gauss weights for gaussian integration moments (use first moment values) n=3
  real ,parameter,dimension(6) :: o3mixp=(/5.3e-08,5.5e-08,6.5e-08,2.2e-07,1.4e-06,1.14e-05/)
  !# ozone mass mixing ratio (g/g) at pressure levels
  !# defined by pj vector. current values are taken from
  !# u.s. standard atmosphere mid-latitude profile
  real ,parameter,dimension(6) :: pj=(/1.0,0.75,0.50,0.25,0.10,0.01/)
  real ,parameter,dimension(ntotal) :: &
        weight=       (/1.00     , 1.00     , 1.00     , 1.00     , 1.00     , 1.00     , 1.00     , 1.00     ,&
                        1.00     , 1.00     , 1.00     , 1.00     , 1.00     , 1.00     , 1.00     , 1.00     ,&
                        0.8471   , 0.1316   , 0.0158   , 0.0055   , 0.7825   , 0.19240  , 0.0251   , 0.8091   ,&
                        0.1689   , 0.0221   , 1.0000   , 0.41070  , 0.4885   , 0.0751   , 0.0258   , 1.0000   ,&
                        0.6385   , 0.29660  , 0.0484   , 0.0164   , 0.2964   , 0.3854   , 0.2583   , 0.05990  ,&
                        0.9486   , 0.0455   , 0.0059   , 0.2962   , 0.3234   , 0.25880  , 0.1216   , 0.5866   ,&
                        0.3524   , 0.0449   , 0.0160   , 1.00000  , 2.2823e-2, 3.8272e-3, 3.1557e-3, 2.7938e-3,&
                        2.3677e-1, 3.9705e-2, 3.2738e-2, 2.8984e-2, 1.8182e-1, 3.0489e-2, 2.5139e-2, 2.2256e-2,&
                        1.7783e-1, 2.9820e-2, 2.4587e-2, 2.1768e-2, 8.0862e-2, 1.3560e-2, 1.1180e-2, 9.8984e-3,&
                        0.3807   , 0.3416   , 0.2256   , 0.0522   , 1.0000   , 0.3908   , 0.1383   , 0.1007   ,&
                        0.0967   , 0.0984   , 0.1751,                                                          &
                        !data for infrared&
                        0.3710   , 0.4400   , 0.1890   , 0.3070   , 0.3980   , 0.2050   , 0.0900   , 0.4030   ,&
                        0.4150   , 0.1820   , 0.3770   , 0.4330   , 0.1900   , 0.3100   , 0.4580   , 0.2320   ,&
                        0.3160   , 0.3980   , 0.1850   , 0.1010   , 0.2030   , 0.4010   , 0.2720   , 0.1240   ,&
                        0.3820   , 0.4240   , 0.1940   , 0.4120   , 0.1800   , 0.4080   , 1.0000   , 0.2220   ,&
                        0.3400   , 0.1400   , 0.2980   , 1.0000   , 0.0117   , 0.0311   , 0.0615   , 0.0458   ,&
                        0.0176   , 0.0468   , 0.0927   , 0.0689   , 0.0487   , 0.1292   , 0.2558   , 0.1903   ,&
                        0.0242   , 0.0633   , 0.0665   , 0.0589   , 0.1541   , 0.1620   , 0.0740   , 0.1936   ,&
                        0.2035   , 0.1570   , 0.4100   , 0.4330   , 0.0810   , 0.2080   , 0.4090   , 0.3020   ,&
                        0.0990   , 0.2200   , 0.4040   , 0.2770   , 0.3670   , 0.5870   , 0.0460/)

  integer,parameter,dimension(ntotal) :: &
        nprob=   (/01,02,03,04,05,06,07,08,09,10,11,12,13,14,15,16,&
                 17,17,17,17,18,18,18,19,19,19,20,21,21,21,21,22,&
                 23,23,23,23,24,24,24,24,25,25,25,26,26,26,26,27,&
                 27,27,27,28,29,29,29,29,29,29,29,29,29,29,29,29,&
                 29,29,29,29,29,29,29,29,30,30,30,30,31,32,32,32,&
                 32,32,32,33,33,33,34,34,34,34,35,35,35,36,36,36,&
                 37,37,37,38,38,38,38,39,39,39,39,40,40,40,41,41,&
                 41,42,43,43,43,43,44,45,45,45,45,45,45,45,45,45,&
                 45,45,45,46,46,46,46,46,46,46,46,46,47,47,47,48,&
                 48,48,48,49,49,49,49,50,50,50/)
  !# nprob is the number of probability intervals in each wavelength
  !# interval. note that wave bins 11 and 12 are reversed in order.
  !# this is done for historical reasons.  cross sections, weights,
  !# refractive indicies etc. for bins 11 and 12 must be reversed also.

  real,parameter,dimension(nsol) :: &
        solfx=(/4.1712e0, 2.5074e0, 1.2024e1, 1.7296e1, 1.2299e1, 5.6975e1,&
                6.6e1   , 4.5725e1, 4.2147e1, 2.9169e1, 6.4328e1, 9.225e1 ,&
                9.7647e1, 5.467e1 , 2.3085e1, 2.9665e1, 5.9268e1, 5.0747e1,&
                5.7410e1, 4.3280e1, 7.4598e1, 5.2732e1, 8.6900e1, 1.2126e2,&
                2.5731e1, 6.0107e1, 1.8400e1, 9.5952e0, 3.5664e1, 1.2764e1,&
                4.0354e0, 4.5364e0/)
  !# solar fluxes ( w/m**2)
  real,parameter,dimension(ntotal) :: &
        pso2=   (/0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.75 , 0.75 , 0.75 , 0.75 , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  ,                                   &
                  ! long &
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0 /)
  real,parameter,dimension(ntotal) :: &
        pso3=   (/0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  ,                                   &
                  ! long
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0/)
   real,parameter,dimension(ntotal) :: &
        psh2o=  (/0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.54 , 0.54 , 0.54 , 0.54 ,&
                  0.54 , 0.54 , 0.0  , 0.54 , 0.54 , 0.54 , 0.54 , 0.0  ,&
                  0.52 , 0.52 , 0.52 , 0.52 , 0.44 , 0.44 , 0.44 , 0.44 ,&
                  0.0  , 0.0  , 0.0  , 0.62 , 0.62 , 0.62 , 0.62 , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.60 , 0.60 , 0.60 , 0.60 ,&
                  0.60 , 0.60 , 0.60 , 0.60 , 0.60 , 0.60 , 0.60 , 0.60 ,&
                  0.60 , 0.60 , 0.60 , 0.60 , 0.60 , 0.60 , 0.60 , 0.60 ,&
                  0.60 , 0.60 , 0.60 , 0.60 , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  ,                                   &
                  ! long&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0/)
    real,parameter,dimension(ntotal) :: &
        psco2=  (/0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.82 ,&
                  0.82 , 0.82 , 0.82 , 0.0  , 0.88 , 0.88 , 0.88 , 0.88 ,&
                  0.88 , 0.88 , 0.88 , 0.88 , 0.88 , 0.88 , 0.88 , 0.88 ,&
                  0.88 , 0.88 , 0.88 , 0.88 , 0.88 , 0.88 , 0.88 , 0.88 ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.93 , 0.93 , 0.93 ,&
                  0.93 , 0.93 , 0.93 ,                                   &
                  ! long
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  ,&
                  0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0  , 0.0/)
  real,parameter,dimension(nsolp) :: &
        xah2o=(/0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,&
                0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,&
                0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,&
                0.0    ,   0.0    ,   0.0    ,   0.1765 ,   9.2460 ,   0.0000 ,&
                0.1765 ,   9.2460 ,   0.0000 ,   0.0000 ,   0.2939 ,   0.5311 ,&
              113.40   ,   0.0000 ,   0.0000 ,   0.3055 ,   5.2180 , 113.00   ,&
                0.00   ,   0.3355 ,   5.5090 , 124.90   ,   0.0    ,   0.0    ,&
                0.0    ,   0.0    ,   0.3420 ,   7.1190 ,  95.740  ,   0.0    ,&
                0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,&
                0.0    ,   0.0    ,   0.3012 ,   0.3012 ,   0.3012 ,   0.3012 ,&
                5.021  ,   5.021  ,   5.021  ,   5.021  ,  63.17   ,  63.17   ,&
               63.17   ,  63.17   , 699.1    , 699.1    , 699.1    , 699.1    ,&
                0.0    ,   6.3321 ,   5.336  , 123.4    ,   0.0    ,   0.0    ,&
                0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0/)
  real,parameter,dimension(nsolp) :: &
        xaco2=(/0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,&
                0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,&
                0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,&
                0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,&
                0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,&
                0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,&
                0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0035 ,&
                0.1849 ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,&
                0.0032 ,   0.0705 ,   1.2980 ,   0.0    ,   0.0    ,   0.0077 ,&
                0.2027 ,   3.8160 ,   0.0000 ,   0.0077 ,   0.2027 ,   3.8160 ,&
                0.0000 ,   0.0077 ,   0.2027 ,   3.8160 ,   0.0000 ,   0.0077 ,&
                0.2027 ,   3.8160 ,   0.0000 ,   0.0077 ,   0.2027 ,   3.8160 ,&
                0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,&
                0.0053 ,   0.0705 ,   0.6732 ,   6.2880 ,  83.900/)
  real,parameter,dimension(nsolp) :: &
        xao2= (/0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,&
                0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,&
                0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,&
                0.0001 ,   0.0022 ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,&
                0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,&
                0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,&
                0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,&
                0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,&
                0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,&
                0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,&
                0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,&
                0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,&
                0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0    ,&
                0.0    ,   0.0    ,   0.0    ,   0.0    ,   0.0/)
  real,parameter,dimension(nsolp) :: &
       xao3=(/260.0     , 100.9     , 11.93    , 0.7370 ,   0.0872 ,   0.0     ,&
                0.0     ,   0.0     ,  0.0     , 0.0    ,   0.01831,   0.069172,&
                0.1180  ,   0.016389,  0.010926, 0.0    ,   0.0    ,   0.0     ,&
                0.0     ,   0.0     ,  0.0     , 0.0    ,   0.0    ,   0.0     ,&
                0.0     ,   0.0     ,  0.0     , 0.0    ,   0.0    ,   0.0     ,&
                0.0     ,   0.0     ,  0.0     , 0.0    ,   0.0    ,   0.0     ,&
                0.0     ,   0.0     ,  0.0     , 0.0    ,   0.0    ,   0.0     ,&
                0.0     ,   0.0     ,  0.0     , 0.0    ,   0.0    ,   0.0     ,&
                0.0     ,   0.0     ,  0.0     , 0.0    ,   0.0    ,   0.0     ,&
                0.0     ,   0.0     ,  0.0     , 0.0    ,   0.0    ,   0.0     ,&
                0.0     ,   0.0     ,  0.0     , 0.0    ,   0.0    ,   0.0     ,&
                0.0     ,   0.0     ,  0.0     , 0.0    ,   0.0    ,   0.0     ,&
                0.0     ,   0.0     ,  0.0     , 0.0    ,   0.0    ,   0.0     ,&
                0.0     ,   0.0     ,  0.0     , 0.0    ,   0.0/)




  !variables
  integer :: ienconc(ngroup)
  !# particle number conc. element for group
  integer :: igelem(nelem)
  !# groups to which elements belong
  integer :: ncore(ngroup)
  !# number of core elements (itype = 2) in group
  integer :: nlayer
  real    :: rhop3(nxyz,nbin,ngroup)
  real    :: rhopcore3(nxyz,nbin,ngroup)
  real    :: rmassmin(ngroup)
  !# mass of particle in first bin [g]
  real    :: rmass(nbin,ngroup)
  !# mass bins [g]
  real    :: rmasscore(nbin,ngroup)
  !# mass bin core [g]
  real    :: rmassup(nbin,ngroup)
  !# upper bin boundary mass [g]
  real    :: rmasscoreup(nbin,ngroup)
  !# upper bin boundary core mass [g]
  real    :: dm(nbin,ngroup)
  !# width of bins in mass space [g]
  real    :: vol(nbin,ngroup)
  !# particle volume [cm^3]
  real    :: rcore(nbin,ngroup)
  !# core radius bins [cm]
  real    :: rcoreup(nbin,ngroup)
  !# upper bin boundary core radius [cm]
  real    :: rup(nbin,ngroup)
  !# upper bin boundary radius [cm]
  real    :: dr(nbin,ngroup)
  !# width of bins in radius space [cm]
  real    :: rlow(nbin,ngroup)
  !# lower bin boundary radius [cm]
  real    :: diffmass(nbin,ngroup,nbin,ngroup)
  !# difference between <rmass> values
  real    :: radius(nbin,ngroup)
  !# radius bins [cm]
  integer :: nz_rad

  character(500) :: aotmappath
  integer, parameter :: im = selected_int_kind(6)
  !# 4 byte integer
  integer, parameter :: rb = selected_real_kind(12)
  !# 8 byte real
  integer(kind=im), parameter :: nbndsw = 14
  !# From RRTMG number of sw bands
  !# rrtmg_sw 14 spectral intervals (microns):
  !# 1  3.846 -  3.077 = 3.4615
  !# 2  3.077 -  2.500  = 2.7885
  !# 3  2.500 -  2.150  = 2.3250
  !# 4  2.150 -  1.942  = 2.0460
  !# 5  1.942 -  1.626  = 1.7840
  !# 6  1.626 -  1.299  = 1.4595
  !# 7  1.299 -  1.242  = 1.2705
  !# 8  1.242 -  0.7782 = 1.0101
  !# 9  0.7782-  0.6250 =
  !# 10 0.6250-  0.4415 =
  !# 11 0.4415-  0.3448 =
  !# 12 0.3448-  0.2632 =
  !# 13 0.2632-  0.2000 =
  !# 14 12.195 -  3.846 =
  real :: casee(nmodelsaot,nbndsw,nsitesaot)
  real :: casew(nmodelsaot,nbndsw,nsitesaot)
  real :: caseg(nmodelsaot,nbndsw,nsitesaot)

  real :: caser(nmodelsaot, nsitesaot)
  real :: cased(nmodelsaot, nsitesaot)
  real :: casew_tuv(nwave,nsitesaot)

  real :: xsecta(nrad,ngroup)
  real :: rdqext(nrad,ngroup,nwave)
  real :: qscat(nrad,ngroup,nwave)
  real :: qbrqs(nrad,ngroup,nwave)
  real :: treal(2,nwave)
  !# real refractive index for liquid water
  real :: ttmag(2,nwave)
  !# imaginary refractive index for liquid water
  real :: ptop
  real :: ako3(4,6)
  real :: akco2(6,6)
  real :: akh2o(54,6)


  integer :: ir

  integer :: imie

  !allocatables - comunication among routines
  integer,allocatable :: lla(:),lls(:)
  !
  real, allocatable :: uw0(:,:)
  real, allocatable :: ug0(:,:)
  real, allocatable :: tt(:,:)
  real, allocatable :: qv_aerad(:,:)
  real, allocatable :: rdh2o(:,:)
  real, allocatable :: pc(:,:,:,:)
  real, allocatable :: tauaer(:,:,:)
  real, allocatable :: taul(:,:,:)
  real, allocatable :: w0(:,:,:)
  real, allocatable :: g0(:,:,:)
  real, allocatable :: opd(:,:,:)
  real, allocatable :: uopd(:,:,:)
  real, allocatable :: y3(:,:,:,:)
  real, allocatable :: pc_aerad(:,:,:,:)
  real, allocatable :: pah2o(:,:,:)
  real, allocatable :: paray(:,:,:)
  real, allocatable :: taugas(:,:,:)

  type aotMap_t
    real, pointer, dimension(:,:) :: aotMap
  end type aotMap_t
  type(aotmap_t), allocatable, dimension(:) :: opt_aotmap, &
                opt_aotMapm

  type aod_t
    real, pointer, dimension(:,:,:) :: aod
    !# Aod
    real, pointer, dimension(:,:,:) :: ssa
    !# single scattering albedo
    real, pointer, dimension(:,:,:) :: asp
    !# asymmetry parameter
  end type aod_t
  type(aod_t), allocatable, dimension(:,:,:) :: opt_Aod, opt_Aodm
  !# Aod in each cell (ngrid,nbands)

  logical :: aodFirstTime
  !# initialization control
  character(len=*),parameter :: mapAotFile = './tables/rad_carma/infMapAOT.vfm'
  !# File with AOT data
  integer, allocatable, dimension(:,:) :: currSite

  public aodDriver,aodFirstTime,setOptMemory,opt_Aod,opt_Aodm

  contains

  subroutine setOptMemory(ngrids,imean,nmzp,nmxp,nmyp,nSpecies,spc_name)
    !# Allocate and set aotMapvariables in memory
    !#
    !# @note
    !# ![](http://brams.cptec.inpe.br/wp-content/uploads/2015/11/logo-brams.navigation.png "")
    !#
    !# **Brief**: Sets the variable aotmap in memory. Must be called in ModeOneproc
    !#
    !# **Documentation**: <http://brams.cptec.inpe.br/documentation/>
    !#
    !# **Author**: Rodrigues, L.F. **&#9993;**<mailto:luiz.rodrigues@inpe.br>
    !#
    !# **Date**: 2018Aug
    !# @endnote
    !#
    !# @changes
    !#
    !# +
    !# @endchanges
    !# @bug
    !# No active bugs reported now
    !# @endbug
    !#
    !# @todo
    !#  &#9744; <br/>
    !# @endtodo
    !#
    !# @warning
    !# Now is under CC-GPL License, please see
    !# &copy; <https://creativecommons.org/licenses/GPL/2.0/legalcode.pt>
    !# @endwarning
    !#
    !#--- ----------------------------------------------------------------------------------------
    !
    integer, intent(in) :: ngrids
    !# Total of grids
    integer, intent(in) :: imean
    !# Mean type of variables
    integer, intent(in) :: nSpecies
    !# Number of aerosol species
    character(len=*), intent(in) :: spc_name(:)
    !Name of all species
    integer, dimension(ngrids) :: nmzp
    !# Levels in each grid
    integer, dimension(ngrids) :: nmxp
    !# Lon pointes in each grid
    integer, dimension(ngrids) :: nmyp
    !#Lat points in each grid

    integer :: ierr
    integer :: ng
    integer :: band
    integer :: iSpc
    character(len=*),parameter :: h='(**setOptMemory**)'

    !print *,'i. nWave=',nWave
    !print *,'Allocating memory for optical, grids=',ngrids,', bands=',nbndsw; call flush(6)
    allocate(opt_aotMap(ngrids), STAT=ierr)
    if (ierr/=0) call fatal_error(h//"Error allocating aotMap in optical")
    allocate(opt_aotMapm(ngrids), STAT=ierr)
    if (ierr/=0) call fatal_error(h//"Error allocating aotMapm in optical")
    allocate(opt_aod(ngrids,nSpecies,nbndsw), STAT=ierr)
    if (ierr/=0) call fatal_error(h//"Error llocating aod in optical")
    allocate(opt_aodm(ngrids,nSpecies,nbndsw), STAT=ierr)
    if (ierr/=0) call fatal_error(h//"Error allocating aodm in optical")
    do ng=1,ngrids
      call opt_nullify_aotMap(opt_aotMap,ng)
      call opt_nullify_aotMap(opt_aotMapm,ng)
      call opt_alloc_aotMap  (opt_aotMap(ng),nmxp(ng), nmyp(ng))
      call opt_alloc_aotMap  (opt_aotMapm(ng),nmxp(ng), nmyp(ng))
      call opt_filltab_aotMap(opt_aotMap(ng), opt_aotMapm(ng), ng, imean,nmxp(ng), nmyp(ng) )
      !
      do iSpc=1,nSpecies
        do band=1,nbndsw
          call opt_nullify_aod(opt_aod,ng,iSpc,band)
          call opt_nullify_aod(opt_aodm,ng,iSpc,band)
          call opt_alloc_aod  (opt_aod(ng,iSpc,band),nmzp(ng),nmxp(ng), nmyp(ng))
          call opt_alloc_aod  (opt_aodm(ng,iSpc,band),nmzp(ng),nmxp(ng), nmyp(ng))
          call opt_filltab_aod(opt_aod(ng,iSpc,band), opt_aodm(ng,iSpc,band), ng, &
	          iSpc, band, imean,nmzp(ng), nmxp(ng), nmyp(ng),spc_name )
	enddo
      enddo

    enddo

    aodFirstTime=.true.

  end subroutine setOptMemory

  subroutine opt_alloc_aod(c_aod,nz,nx,ny)
    !# Allocate  aod variable
    !#
    !# @note
    !# ![](http://brams.cptec.inpe.br/wp-content/uploads/2015/11/logo-brams.navigation.png "")
    !#
    !# **Brief**: Allocate and initilize to zero the variable aod
    !#
    !# **Documentation**: <http://brams.cptec.inpe.br/documentation/>
    !#
    !# **Author**: Rodrigues, L.F. **&#9993;**<mailto:luiz.rodrigues@inpe.br>
    !#
    !# **Date**: 2018Aug
    !# @endnote
    !#
    !# @changes
    !#
    !# +
    !# @endchanges
    !# @bug
    !# No active bugs reported now
    !# @endbug
    !#
    !# @todo
    !#  &#9744; <br/>
    !# @endtodo
    !#
    !# @warning
    !# Now is under CC-GPL License, please see
    !# &copy; <https://creativecommons.org/licenses/GPL/2.0/legalcode.pt>
    !# @endwarning
    !#
    !#--- ----------------------------------------------------------------------------------------
    !
    type(aod_t) :: c_aod
    !# Variable to be allocated Map of AOT
    integer, intent(in) :: nz
    !# Number of levels
    integer, intent(in) :: nx
    !# Number of lats
    integer, intent(in) :: ny
    !# Number of lons

  	allocate(c_aod%aod(nz,nx,ny))
    allocate(c_aod%ssa(nz,nx,ny))
    allocate(c_aod%asp(nz,nx,ny))
  	c_aod%aod(:,:,:) = 0.
    c_aod%ssa(:,:,:) = 0.
    c_aod%asp(:,:,:) = 0.

  end subroutine opt_alloc_aod

  subroutine opt_nullify_aod(aod_c, ng, iSpc, band)
    !# Nullify  aod variable
    !#
    !# @note
    !# ![](http://brams.cptec.inpe.br/wp-content/uploads/2015/11/logo-brams.navigation.png "")
    !#
    !# **Brief**: Allocate and initiliza to zero the variable aod
    !#
    !# **Documentation**: <http://brams.cptec.inpe.br/documentation/>
    !#
    !# **Author**: Rodrigues, L.F. **&#9993;**<mailto:luiz.rodrigues@inpe.br>
    !#
    !# **Date**: 2018Aug
    !# @endnote
    !#
    !# @changes
    !#
    !# +
    !# @endchanges
    !# @bug
    !# No active bugs reported now
    !# @endbug
    !#
    !# @todo
    !#  &#9744; <br/>
    !# @endtodo
    !#
    !# @warning
    !# Now is under CC-GPL License, please see
    !# &copy; <https://creativecommons.org/licenses/GPL/2.0/legalcode.pt>
    !# @endwarning
    !#
    !#--- ----------------------------------------------------------------------------------------
    !
    implicit none

    type(aod_t), intent(inout) :: aod_c(:,:,:)
    !# Type to be nullified
    integer, intent(in)          :: ng
    !# Grid
    integer, intent(in)          :: band
    !# band number
    integer, intent(in)        :: iSpc
    !# Specie number

    if (associated(aod_c(ng,iSpc,band)%aod))  nullify (aod_c(ng,iSpc,band)%aod)
    if (associated(aod_c(ng,iSpc,band)%asp))  nullify (aod_c(ng,iSpc,band)%asp)
    if (associated(aod_c(ng,iSPc,band)%ssa))  nullify (aod_c(ng,iSpc,band)%ssa)

  end subroutine opt_nullify_aod

  subroutine opt_filltab_aod(imap, imapm, ng, iSpc, band, imean, n1, n2, n3, spc_name)
    !# Fill the  aod variable in var tables of model
    !#
    !# @note
    !# ![](http://brams.cptec.inpe.br/wp-content/uploads/2015/11/logo-brams.navigation.png "")
    !#
    !# **Brief**: Fill the  aod variable in var tables of model
    !#
    !# **Documentation**: <http://brams.cptec.inpe.br/documentation/>
    !#
    !# **Author**: Rodrigues, L.F. **&#9993;**<mailto:luiz.rodrigues@inpe.br>
    !#
    !# **Date**: 2018Aug
    !# @endnote
    !#
    !# @changes
    !#
    !# +
    !# @endchanges
    !# @bug
    !# No active bugs reported now
    !# @endbug
    !#
    !# @todo
    !#  &#9744; <br/>
    !# @endtodo
    !#
    !# @warning
    !# Now is under CC-GPL License, please see
    !# &copy; <https://creativecommons.org/licenses/GPL/2.0/legalcode.pt>
    !# @endwarning
    !#
    !#--- ----------------------------------------------------------------------------------------
    !
    use var_tables, only: InsertVTab
    include "i8.h"
    integer, intent(in) :: ng
    !# grid
    integer, intent(in) :: n1
    !# number of levels
    integer, intent(in) :: n2
    !# Lon number of points
    integer, intent(in) :: n3
    !# Lat number of points
    integer, intent(in) :: band
    !# number of band
    integer, intent(in) :: iSpc
    !# number of the specie
    integer, intent(in) :: imean
    !# Mean of variable
    character(len=*),intent(in) :: spc_name(:)
    !Name of all species
    type(aod_t) :: imap
    !# Var to be inserted
    type(aod_t) :: imapm
    !# Var to be inserted (m)

    character(len=2), dimension(14), parameter :: bn= &
                                        (/'01','02','03','04','05','06','07', &
                                          '08','09','10','11','12','13','14'/)
    integer(kind=i8)       :: npts
    !# Total of points 1D

    if(associated(imap%aod))then
        npts = n1*n2*n3
        !print *,'ftopt vname=','aod_'//trim(spc_name(iSpc))//'_b'//bn(band),npts
        call InsertVTab(imap%aod, imapm%aod, ng, npts, imean, &
            'aod_'//trim(spc_name(iSpc))//'_b'//bn(band)//' :3:mpti')
    end if
    if(associated(imap%asp))then
        npts = n1*n2*n3
        call InsertVTab(imap%asp, imapm%asp, ng, npts, imean, &
            'asp_'//trim(spc_name(iSpc))//'_b'//bn(band)//' :3:mpti')
    end if
    if(associated(imap%ssa))then
        npts = n1*n2*n3
        call InsertVTab(imap%ssa, imapm%ssa, ng, npts, imean, &
            'ssa_'//trim(spc_name(iSpc))//'_b'//bn(band)//' :3:mpti')
    end if

  end subroutine opt_filltab_aod


  subroutine opt_alloc_aotMap(c_aotMap,nx,ny)
    !# Allocate  aotMap variable
    !#
    !# @note
    !# ![](http://brams.cptec.inpe.br/wp-content/uploads/2015/11/logo-brams.navigation.png "")
    !#
    !# **Brief**: Allocate and initiliza to zero the variable aotmap
    !#
    !# **Documentation**: <http://brams.cptec.inpe.br/documentation/>
    !#
    !# **Author**: Rodrigues, L.F. **&#9993;**<mailto:luiz.rodrigues@inpe.br>
    !#
    !# **Date**: 2018Aug
    !# @endnote
    !#
    !# @changes
    !#
    !# +
    !# @endchanges
    !# @bug
    !# No active bugs reported now
    !# @endbug
    !#
    !# @todo
    !#  &#9744; <br/>
    !# @endtodo
    !#
    !# @warning
    !# Now is under CC-GPL License, please see
    !# &copy; <https://creativecommons.org/licenses/GPL/2.0/legalcode.pt>
    !# @endwarning
    !#
    !#--- ----------------------------------------------------------------------------------------
    !
    type(aotMap_t) :: c_aotMap
    !# Variable to be allocated Map of AOT
    integer, intent(in) :: nx
    !# Number of lats
    integer, intent(in) :: ny
    !# Number of lons

  	allocate(c_aotMap%aotMap(nx,ny))
  	c_aotMap%aotMap(:,:) = 0.

  end subroutine opt_alloc_aotMap

  subroutine opt_nullify_aotMap(aot, ng)
    !# Nullify  aotMap variable
    !#
    !# @note
    !# ![](http://brams.cptec.inpe.br/wp-content/uploads/2015/11/logo-brams.navigation.png "")
    !#
    !# **Brief**: Allocate and initiliza to zero the variable aotmap
    !#
    !# **Documentation**: <http://brams.cptec.inpe.br/documentation/>
    !#
    !# **Author**: Rodrigues, L.F. **&#9993;**<mailto:luiz.rodrigues@inpe.br>
    !#
    !# **Date**: 2018Aug
    !# @endnote
    !#
    !# @changes
    !#
    !# +
    !# @endchanges
    !# @bug
    !# No active bugs reported now
    !# @endbug
    !#
    !# @todo
    !#  &#9744; <br/>
    !# @endtodo
    !#
    !# @warning
    !# Now is under CC-GPL License, please see
    !# &copy; <https://creativecommons.org/licenses/GPL/2.0/legalcode.pt>
    !# @endwarning
    !#
    !#--- ----------------------------------------------------------------------------------------
    !
    implicit none

    type(aotMap_t), intent(inout) :: aot(:)
    !# Type to be nullified
    integer, intent(in)          :: ng
    !# Grid
    if (associated(aot(ng)%aotMap))  nullify (aot(ng)%aotMap)

  end subroutine opt_nullify_aotMap

  subroutine opt_dealloc_aotMap()
    !# Dellocate  aotMap variable
    !#
    !# @note
    !# ![](http://brams.cptec.inpe.br/wp-content/uploads/2015/11/logo-brams.navigation.png "")
    !#
    !# **Brief**: Dellocate and the variable aotmap
    !#
    !# **Documentation**: <http://brams.cptec.inpe.br/documentation/>
    !#
    !# **Author**: Rodrigues, L.F. **&#9993;**<mailto:luiz.rodrigues@inpe.br>
    !#
    !# **Date**: 2018Aug
    !# @endnote
    !#
    !# @changes
    !#
    !# +
    !# @endchanges
    !# @bug
    !# No active bugs reported now
    !# @endbug
    !#
    !# @todo
    !#  &#9744; <br/>
    !# @endtodo
    !#
    !# @warning
    !# Now is under CC-GPL License, please see
    !# &copy; <https://creativecommons.org/licenses/GPL/2.0/legalcode.pt>
    !# @endwarning
    !#
    !#--- ----------------------------------------------------------------------------------------
    !
    use mem_grid, only: &
       ngrids,         &!(in)
       nnxp,           &!(in)
       nnyp,           &!(in)
       nnzp             !(in)


    integer :: idx

    if(ALLOCATED(opt_aotMap))then
      do idx = 1, ngrids
        deallocate(opt_aotMap(idx)%aotMap)
      end do
      deallocate(opt_aotMap)
    end if

    if(ALLOCATED(opt_aotMapm))then
      do idx = 1, ngrids
        deallocate(opt_aotMapm(idx)%aotMap)
      end do
      deallocate(opt_aotMapm)
    end if

  end subroutine opt_dealloc_aotMap

  subroutine opt_filltab_aotMap(imap, imapm, ng, imean, n1, n2)
    !# Fill the  aotMap variable in var tables of model
    !#
    !# @note
    !# ![](http://brams.cptec.inpe.br/wp-content/uploads/2015/11/logo-brams.navigation.png "")
    !#
    !# **Brief**: Fill the  aotMap variable in var tables of model
    !#
    !# **Documentation**: <http://brams.cptec.inpe.br/documentation/>
    !#
    !# **Author**: Rodrigues, L.F. **&#9993;**<mailto:luiz.rodrigues@inpe.br>
    !#
    !# **Date**: 2018Aug
    !# @endnote
    !#
    !# @changes
    !#
    !# +
    !# @endchanges
    !# @bug
    !# No active bugs reported now
    !# @endbug
    !#
    !# @todo
    !#  &#9744; <br/>
    !# @endtodo
    !#
    !# @warning
    !# Now is under CC-GPL License, please see
    !# &copy; <https://creativecommons.org/licenses/GPL/2.0/legalcode.pt>
    !# @endwarning
    !#
    !#--- ----------------------------------------------------------------------------------------
    !
    use var_tables, only: InsertVTab
    include "i8.h"
    integer, intent(in) :: ng
    !# grid
    integer, intent(in) :: n1
    !# Lon number of points
    integer, intent(in) :: n2
    !# Lat number of points
    integer, intent(in) :: imean
    !# Mean of variable
    type(aotMap_t) :: imap
    !# Var to be inserted
    type(aotMap_t) :: imapm
    !# Var to be inserted (m)

    integer(kind=i8)       :: npts
    !# Total of points 1D

   	if(associated(imap%aotMap))then
		    npts = n1*n2
		    call InsertVTab(imap%aotMap, imapm%aotMap, ng, npts, imean, &
			      'AOTMAP :2:hist:anal:mpti')
	  end if

  end subroutine opt_filltab_aotMap

  subroutine opt_read_aotmap()
    !# Read  aot Map from input file
    !#
    !# @note
    !# ![](http://brams.cptec.inpe.br/wp-content/uploads/2015/11/logo-brams.navigation.png "")
    !#
    !# **Brief**: Read aot map from file. The file contais the site type according with:
    !# 00 undef
    !# 01 Abracos_Hill.lev15
    !# 02 CUIABA-MIRANDA.lev15
    !# 03 Rio_Branco.lev15
    !# 04 Alta_Floresta.lev15
    !# 05 Campo_Grande_SONDA.lev15
    !# 06 CEILAP-BA.lev15
    !# 07 Cordoba-CETT.lev15
    !# 08 SANTA_CRUZ.lev15
    !# 09 Sao_Paulo.lev15
    !# For the following we set inside model:
    !# 10 belterra.
    !# 11 1 oceanico
    !# 12 continental
    !#
    !# **Documentation**: <http://brams.cptec.inpe.br/documentation/>
    !#
    !# **Author**: Rodrigues, L.F. **&#9993;**<mailto:luiz.rodrigues@inpe.br>
    !#
    !# **Date**: 2018Aug
    !# @endnote
    !#
    !# @changes
    !#
    !# +
    !# @endchanges
    !# @bug
    !# No active bugs reported now
    !# @endbug
    !#
    !# @todo
    !#  &#9744; <br/>
    !# @endtodo
    !#
    !# @warning
    !# Now is under CC-GPL License, please see
    !# &copy; <https://creativecommons.org/licenses/GPL/2.0/legalcode.pt>
    !# @endwarning
    !#
    !#--- ----------------------------------------------------------------------------------------
    !
    use mem_grid
    use node_mod, only: &
          nodei0, &
          nodej0, & ! intent(in)
          nodemxp, &
          nodemyp, & ! intent(in)
          nmachs,  & ! intent(in)
          mynum,  &  ! intent(in)
          mchnum, &  ! intent(in)
          master_num, & ! intent(in)
          ia, &
          iz, &
          ja, &
          jz, &
          mxp, &
          myp, &
          mzp, &
          nxbeg, &
          nxend, &
          nybeg, &
          nyend
    use parlib, only: &
          parf_bcast ! subroutine
    use readbcst, only: &
         gatherdata

    integer :: i
    integer :: j
    integer :: nlon
    integer :: nlat
    integer :: ii
    integer :: jj
    integer :: effsite
    integer :: nsites
    integer :: ifm
    integer :: i1
    integer :: i2
    integer :: ic
    integer :: j1
    integer :: j2
    integer :: jc
    integer :: qi1
    integer :: qi2
    integer :: qj1
    integer :: qj2
    real    :: latni
    real    :: latnf
    real    :: lonni
    real    :: lonnf
    real    :: latstep
    real    :: lonstep
    real    :: dlonr
    real    :: dlatr
    real    :: undef
    real    :: comm(10)
    integer, allocatable, dimension(:,:) :: infmap
    real, allocatable, dimension(:,:)    :: infmapreal
    real, allocatable, dimension(:)      :: moda
    real, allocatable, dimension(:,:)    :: prlon
    real, allocatable, dimension(:,:)    :: prlat
    real :: globalglon(nnxp(1), nnyp(1)) !apenas para uma grade teste
    real :: globalglat(nnxp(1), nnyp(1)) !apenas para uma grade teste
    real :: globalaot(nnxp(1), nnyp(1))
    character(len=16)  :: varn

    if (mchnum==master_num) then !Only master do the read
      open(unit=17, file=trim(mapaotfile), status='old')
      !print *,'Reading the header of AOTMap vfm file: ', trim(mapaotfile); call flush(6)
      read(17,*) lonni, latni, lonnf, latnf
      read(17,*) nlon, nlat, lonstep, latstep
      read(17,*) nsites, undef
      !Prepare array to be broadcast for all processors
      comm(1) =lonni
      comm(2) =latni
      comm(3) =lonnf
      comm(4) =latnf
      comm(5) =lonstep
      comm(6) =latstep
      comm(7) =undef
      comm(8) =real(nlon)
      comm(9) =real(nlat)
      comm(10)=real(nsites)
    end if
    !print *, 'broadcasting AOT Map data'; call flush(6)
    call parf_bcast(comm, int(size(comm,1),i8), master_num)
    !Getting information from array
    lonni    =comm(1)
    latni    =comm(2)
    lonnf    =comm(3)
    latnf    =comm(4)
    lonstep  =comm(5)
    latstep  =comm(6)
    undef    =comm(7)
    nlon     =int(comm(8) )
    nlat     =int(comm(9) )
    nsites   =int(comm(10))
    !print *, 'Allocating information'; call flush(6)
    allocate(infmap(nlon, nlat),infmapreal(nlon, nlat))
    if (mchnum==master_num) then !Only master read
      !print *,'Reading the data inside AOTMap file'; call flush(6)
      call viirec(17, infmap, nlon*nlat, 'lin')
      !Make it real
      infmapreal=real(infmap)
      close(17)
    end if
    !print *, 'broadcasting AOT data'; call flush(6)
    call parf_bcast(infmapreal, int(size(infmap,1),i8), int(size(infmap,2),i8), master_num)
    !Make it int again
    infmap=int(infmapreal)
    !print *,'Processing local data'; call flush(6)
    allocate(prlat(nlon, nlat), prlon(nlon,nlat), moda(nsites))
    call apiprlatlon(nlon, nlat, prlat, prlon, latstep, lonstep, latni, lonni)
    do ifm =1, ngrids
      opt_aotMap(ifm)%aotmap(:,:) = 0.
      call newgrid(ifm)
      varn = 'glon'
      call gatherdata(2, varn, ifm, nnxp(ifm), nnyp(ifm), &
            nmachs, mchnum, mynum, master_num,             &
             grid_g(ifm)%glon, globalglon)
      varn = 'glat'
      call gatherdata(2, varn, ifm, nnxp(ifm), nnyp(ifm), &
            nmachs, mchnum, mynum, master_num,             &
            grid_g(ifm)%glat, globalglat)
      do i =1, nnxp(ifm)
        do j=1, nnyp(ifm)
          ! evitando pontos fora do dominio da grade de umidade de solo
          if ( globalglat(i,j)<latni .or. globalglat(i,j)>latnf .or. &
              globalglon(i,j)<lonni .or. globalglon(i,j)>lonnf) cycle
          call interpolacao(globalglon(i,j), globalglat(i,j), nlon, nlat, &
                              prlat, prlon, i1, i2, ic, j1, j2, jc)
          if(ic.ge.0 .and. jc .ge. 0)then
            dlonr = 0.5*(globalglon(nodemxp(mynum,ifm),j) - globalglon(1,j))/&
                                                         float(nnxp(ifm)-1)
            dlatr = 0.5*(globalglat(i,nodemyp(mynum,ifm)) - globalglat(i,1))/&
                                                         float(nnyp(ifm)-1)
            qi1=int(dlonr/lonstep+0.5)
            qi2=int(dlonr/lonstep+0.5)
            qj1=int(dlatr/latstep+0.5)
            qj2=int(dlatr/latstep+0.5)
            moda(:) = 0
            do jj =max(1,jc-qj1),min(nlat,jc+qj2)
              do ii = max(1,ic-qi1),min(nlon,ic+qi2)
                if( infmap(ii,jj) .ne. undef)then
                  moda(infmap(ii,jj)) = moda(infmap(ii,jj)) + 1
                end if
              end do
            end do
            effsite = 1
            do ii = 1, nsites
              do jj = 1, nsites-1
                if(moda(ii) .gt. moda(jj))then
                  effsite = ii
                end if
              end do
            end do
            globalaot(i,j)= effsite !to remove 0 values
          end if
        end do
      end do
      !print *, "Making buffer for grid",ifm; call flush(6)
      call mk_2_buff(globalaot,opt_aotMap(ifm)%aotmap, &
                              nnxp(ifm), nnyp(ifm), &
                              nodemxp(mynum,ifm), nodemyp(mynum,ifm), &
                              nxbeg(mynum,ifm),nxend(mynum,ifm),nybeg(mynum,ifm),nyend(mynum,ifm))
    end do
    !print *, 'End AOT Reading'; call flush(6)
  end subroutine opt_read_aotMap

  subroutine aodDriver(m1,m2,m3,ia,iz,ja,jz,ngrids)
    !# Driver to compute AOD data to RRTGM
    !#
    !# @note
    !# ![](http://brams.cptec.inpe.br/wp-content/uploads/2015/11/logo-brams.navigation.png "")
    !#
    !# **Brief**: Compute AOD for all points. If first Time compute the site information in all
    !# grid according aot map and/or path area. The totalAOD is return back to RRTMG and fill
    !# the type opt(ng,nbands)%aod(k,i,j) in var_tables to plot in pos-process, etc.
    !#
    !# **Documentation**: <http://brams.cptec.inpe.br/documentation/>
    !#
    !# **Author**: Rodrigues, L.F. **&#9993;**<mailto:luiz.rodrigues@inpe.br>
    !#
    !# **Date**: 2018Aug
    !# @endnote
    !#
    !# @changes
    !#
    !# +
    !# @endchanges
    !# @bug
    !# No active bugs reported now
    !# @endbug
    !#
    !# @todo
    !#  &#9744; <br/>
    !# @endtodo
    !#
    !# @warning
    !# Now is under CC-GPL License, please see
    !# &copy; <https://creativecommons.org/licenses/GPL/2.0/legalcode.pt>
    !# @endwarning
    !#
    !#--- ----------------------------------------------------------------------------------------
    !
    use mem_leaf, only: &
                    leaf_g
    use aer1_list, only: &
                    mode_alloc, &
                    nspecies,    &! Number of aerosol species
                    nmodes,      &! Number of aerosol modes
                    spc_alloc,   &! Species table allocation
                    part_radius, &! Radius of particles
                    part_dens     ! Density of particles (rho)
    use mem_aer1, only: &
                    aer1_g !Aerosol data
    use mem_grid, only: &
                    grid_g, &
                    dzt, &
                    dtlt, &
                    time
    use mem_basic, only: &
                    basic_g
    use mem_radiate, only:        &
                    radfrq

    integer, intent(in) :: m1
    !# number of vertical levels
    integer, intent(in) :: m2
    !#
    integer, intent(in) :: m3
    !#
    integer, intent(in) :: ia
    !#
    integer, intent(in) :: iz
    !#
    integer, intent(in) :: ja
    !#
    integer, intent(in) :: jz
    !#
    integer, intent(in) :: ngrids
    !# Current grid
    real :: aer(m2,m3,m1,nSpecies,nModes)

    integer, parameter :: ngrid=1
    integer :: i
    integer :: j
    integer :: k
    integer :: iCol
    integer :: iMod
    integer :: iSpc
    integer :: band
    real :: kgAir
    real :: layTh

    !--- radiation calculation is updated only every radfrq seconds
    IF ( .not. (mod(time+.001, radfrq) < dtlt .or. time<0.001)) return

    if(aodFirstTime) then !Only in the first call because the site have the same type
      allocate(currSite(m2,m3))
      call setupraddata() !Fill the arrays with optical characteristics
      !print *, 'Reading AOT Map'; call flush(6)
      call opt_read_aotmap()
      !print *, 'Init the sites types'; call flush(6)
      call initCurrSite(ia,iz,jz,jz,m2,m3,opt_aotMap(ngrid)%aotmap, &
                        currSite,leaf_g(ngrid)%patch_area(:,:,1)) !Init the site type
      aodFirstTime=.false.
    endif
    !print *, 'filling aer'; call flush(6)
    do i=ia,iz
      do j=ja,jz
        do k=1,m1
          if(k<m1) then
            !Determinando a espessura da camada em [m]
            layTh=grid_g(ngrid)%rtgt(i,j)/dzt(k)
          endif
          do iSpc=1,nSpecies
            do iMod=1,nModes
              if(mode_alloc(iMod,iSpc) /= 1 ) cycle
              !Determinando a massa de ar por m2 na camada do ponto de grade
              kgAir=basic_g(ngrid)%dn0(k,i,j)*layTh*1.0e-9
              !A Massa de aerosol é a massa em todo o ponto de grade em kg
              aer(i,j,k,iSpc,iMod)=aer1_g(iMod,iSpc,ngrid)%sc_p(k,i,j)*kgAir
              !aer(i,j,k,iSpc,iMod)=100*kgAir !For test only
            enddo
          enddo
        enddo
      enddo
    enddo

    !print *,'COmputing AOD'; call flush(6)
    call computeAod(ngrid,m1,m2,m3,ia,iz,ja,jz,nbndsw,nModes,nSpecies,currSite,&
                    part_radius,part_dens,mode_alloc,aer)
    !print *,'AOD Computed'; call flush(6)

  end subroutine aodDriver

  real function aod(aerMass,extCoef,rho,radius)
    !# Calculate aod
    !#
    !# @note
    !# ![](http://brams.cptec.inpe.br/wp-content/uploads/2015/11/logo-brams.navigation.png "")
    !#
    !# **Brief**: this function calculate the aod for a specific lambda. Lacis, A. A., and
    !# M. I. Mishchenko, 1994: Climate forcing, climate sensitivity, and climate response:
    !# A radiative modeling perspective on atmospheric aerosols. Aerosol Forcing of Climate,
    !# R. J. Charlson and J. Heintzenberg, Eds., John Wiley and Sons
    !# \[\tau_b=\frac{3 \text{Q}_b\text{M}}{4\rho r_e}\]
    !# Where
    !# \[{\tau}_b \text{ = AOD for specific b band []}\]
    !# \[\text{Q}_b \text{ = Extinction Coeficient for specific b band []}\]
    !# \[\text{M} \text{ = Mass of aerosol []}\]
    !# \[\rho \text{ = density os aerosol particle []}\]
    !# \[r_e \text{= efective radius of the aerosol particle []}\]
    !#
    !# **Documentation**: <http://brams.cptec.inpe.br/documentation/>
    !#
    !# **Author**: Rodrigues, L.F. **&#9993;**<mailto:luiz.rodrigues@inpe.br>
    !#
    !# **Date**: 2018Aug
    !# @endnote
    !#
    !# @changes
    !#
    !# +
    !# @endchanges
    !# @bug
    !# No active bugs reported now
    !# @endbug
    !#
    !# @todo
    !#  &#9744; <br/>
    !# @endtodo
    !#
    !# @warning
    !# Now is under CC-GPL License, please see
    !# &copy; <https://creativecommons.org/licenses/GPL/2.0/legalcode.pt>
    !# @endwarning
    !#
    !#--- ----------------------------------------------------------------------------------------
    !
    real, intent(in) :: aerMass
    !# Mass of aerosol [g/cm3]
    real, intent(in) :: extCoef
    !# Extinction coefficient
    real, intent(in) :: rho
    !# Particle density [g/cm3]
    real, intent(in) :: radius
    !# particle radius [um]

    aod=(3*extCoef*aerMass)/(4*rho*radius)

  end function aod


  subroutine computeAod(ngrid,m1,m2,m3,ia,iz,ja,jz,nbands,nModes,nSpecies, &
                        currSite,part_radius,part_dens,mode_alloc,aer)
    !# Compute aod for all columns, aerosols and lambdas
    !#
    !# @note
    !# ![](http://brams.cptec.inpe.br/wp-content/uploads/2015/11/logo-brams.navigation.png "")
    !#
    !# **Brief**: this function calculate the aod for all models domain. Use Lacis, A. A., and
    !# M. I. Mishchenko, 1994: Climate forcing, climate sensitivity, and climate response:
    !# A radiative modeling perspective on atmospheric aerosols. Aerosol Forcing of Climate,
    !# R. J. Charlson and J. Heintzenberg, Eds., John Wiley and Sons. And
    !# Tropospheric Aerosol Optical Thickness from the GOCART Model and Comparisons
    !# with Satellite and Sun Photometer Measurements, Chin et al. 2002. and
    !# Regional representativity of AERONET observation sites during the biomass burning
    !# season in South America determined by correlation studies with MODIS Aerosol Optical Depth.
    !# Hoelzemann, J. J. and al. JOURNAL OF GEOPHYSICAL RESEARCH, VOL. 114, D13301,
    !# doi:10.1029/2008JD010369, 2009
    !# \[totalAod_{i,k,b}=\sum_{aer=1}^{n}text{aod}(b)\]
    !# Where
    !# \[totalAod_{i,k,b} \text{ = aod for specific band []}\]
    !# \[\aod(b) \text{ = aod function for band b []}\]
    !#
    !# **Documentation**: <http://brams.cptec.inpe.br/documentation/>
    !#
    !# **Author**: Rodrigues, L.F. **&#9993;**<mailto:luiz.rodrigues@inpe.br>
    !#
    !# **Date**: 2018Aug
    !# @endnote
    !#
    !# @changes
    !#
    !# +
    !# @endchanges
    !# @bug
    !# No active bugs reported now
    !# @endbug
    !#
    !# @todo
    !#  &#9744; <br/>
    !# @endtodo
    !#
    !# @warning
    !# Now is under CC-GPL License, please see
    !# &copy; <https://creativecommons.org/licenses/GPL/2.0/legalcode.pt>
    !# @endwarning
    !#
    !#--- ----------------------------------------------------------------------------------------
    !
    use node_mod, only:  &
       mynum

    integer, intent(in) :: ngrid
    !#
    integer, intent(in) :: m1
    !# total of vertical levels
    integer, intent(in) :: m2
    !# total of lon points
    integer, intent(in) :: m3
    !# total of lat pointers
    integer, intent(in) :: ia
    !#
    integer, intent(in) :: iz
    !#
    integer, intent(in) :: ja
    !#
    integer, intent(in) :: jz
    !#
    integer, intent(in) :: nbands
    !# total of wavelengh bands
    integer, intent(in) :: nModes
    !# Number of modes of aerosol from aer1_list
    integer, intent(in) :: nSpecies
    !# Number os species of aerosol from aer1_list
    integer,dimension(m2,m3),intent(in) :: currSite
    !# Used to determine which table (casee) must be used for Ext. Coef.
    !# The array must be initialized in initCurrSite (just once)
    real,dimension(nmodes,nspecies),intent(in) :: part_radius !radius
    !# radius of each bin aerosol particles from aer1_list
    real,dimension(nmodes,nspecies), intent(in) :: part_dens !rho
    !# density of each particle in aerosols
    integer,dimension(nmodes,nspecies),intent(in) :: mode_alloc
    !# Determine if aerosol is allocated, transported and more
    !# spaction(specie,[1=source,2=drydep,3=wetdep,4=fdda, 5=offline emission, 6=transport])
    real, intent(in) :: aer(m2,m3,m1,nSpecies,nModes)
    !# Aerosol mass

    integer :: band
    integer :: iSpc
    integer :: iMod
    integer :: k,i,j
    integer,parameter :: idAot=1 !For now just one per site
    real :: extCoef

    !Loop in all columns
    do i=ia,iz
      do j=ja,jz
        !loop in k (leves)
        do k=1,m1
          !Loop in bands
          do band=1,nbands
            !Loop in all aerosol species
            do iSpc=1,nSpecies
              opt_Aod(ngrid,iSpc,band)%aod(k,i,j)=0.0
              !Setting the exctintion coeficient
              extCoef=casee(idAot,band,currsite(i,j))
              opt_Aod(ngrid,iSpc,band)%ssa(k,i,j)=caseW(idAot,band,currSite(i,j))
              opt_Aod(ngrid,iSpc,band)%asp(k,i,j)=caseG(idAot,band,currSite(i,j))
              !write (*,fmt='(4(I2.2,1X),2(E18.8,1X))') i,j,k,band,opt_Aod(ngrid,band)%ssa(k,i,j),opt_Aod(ngrid,band)%asp(k,i,j); call flush(6)
              !Loop in all aerosol species mode (bin)
              do iMod=1,nModes
                if(mode_alloc(iMod,iSpc) /= 1 ) cycle
                opt_Aod(ngrid,iSpc,band)%aod(k,i,j)=opt_Aod(ngrid,iSpc,band)%aod(k,i,j) &
                  +aod(aer(i,j,k,iSpc,iMod),extCoef,part_dens(iMod,iSpc),part_radius(iMod,iSpc))
              enddo
            enddo
          enddo
        enddo
      enddo
    enddo

  end subroutine computeAod

  subroutine initCurrSite(ia,iz,ja,jz,m2,m3,aotMap,currSite,patch_area)
    !# Initialize crossreference pointer to Qext tables
    !#
    !# @note
    !# ![](http://brams.cptec.inpe.br/wp-content/uploads/2015/11/logo-brams.navigation.png "")
    !#
    !# **Brief**: this subroutine prepare the pointer to Qext table
    !# Tropospheric Aerosol Optical Thickness from the GOCART Model and Comparisons
    !# with Satellite and Sun Photometer Measurements, Chin et al. 2002. and
    !# Regional representativity of AERONET observation sites during the biomass burning
    !# season in South America determined by correlation studies with MODIS Aerosol Optical Depth.
    !# Hoelzemann, J. J. and al. JOURNAL OF GEOPHYSICAL RESEARCH, VOL. 114, D13301,
    !# doi:10.1029/2008JD010369, 2009
    !#
    !# **Documentation**: <http://brams.cptec.inpe.br/documentation/>
    !#
    !# **Author**: Rodrigues, L.F. **&#9993;**<mailto:luiz.rodrigues@inpe.br>
    !#
    !# **Date**: 2018Aug
    !# @endnote
    !#
    !# @changes
    !#
    !# +
    !# @endchanges
    !# @bug
    !# No active bugs reported now
    !# @endbug
    !#
    !# @todo
    !#  &#9744; <br/>
    !# @endtodo
    !#
    !# @warning
    !# Now is under CC-GPL License, please see
    !# &copy; <https://creativecommons.org/licenses/GPL/2.0/legalcode.pt>
    !# @endwarning
    !#
    !#--- ----------------------------------------------------------------------------------------
    !
    integer, intent(in) :: ia
    !#
    integer, intent(in) :: iz
    !#
    integer, intent(in) :: ja
    !#
    integer, intent(in) :: jz
    !#
    integer, intent(in) :: m2
    !#
    integer, intent(in) :: m3
    !
    real,intent(in) :: patch_area(m2,m3)
    !# Path of soil characteritics
    real, intent(in) :: aotMap(m2,m3)
    !# Aot map
    integer,dimension(m2,m3),intent(out) :: currSite
    !# Used to determine which table used for Ext. Coef.

    integer :: i,j

    !Loop in all Cols
    do i=1,m2
      do j=1,m3
        if(int(aotMap(i,j)) > 0.0 .and. int(aotMap(i,j)) < 11) then !If not mapped inside list
	 	   !Is one of the mapped
           currSite(i,j)=aotMap(i,j)  
        else 
		  !If not mapped inside list
          if (patch_area(i,j) .gt. .009) then !Is ocean
            currSite(i,j)=11
          else !Is continental
            currSite(i,j)=12
          endif
        endif
      enddo
    enddo

  end subroutine initCurrSite


  ! subroutine setupbins()
  !   !# Sets up the particle size bins
  !   !#
  !   !# @note
  !   !# ![](http://brams.cptec.inpe.br/wp-content/uploads/2015/11/logo-brams.navigation.png "")
  !   !#
  !   !# **Brief**: this routine evaluates the derived mapping arrays and sets up
  !   !#  the particle size bins.
  !   !#
  !   !# **Documentation**: <http://brams.cptec.inpe.br/documentation/>
  !   !#
  !   !# **Author**: Rodrigues, L.F. **&#9993;**<mailto:luiz.rodrigues@inpe.br>
  !   !#
  !   !# **Date**: 2018Aug
  !   !# @endnote
  !   !#
  !   !# @changes
  !   !#
  !   !# +
  !   !# @endchanges
  !   !# @bug
  !   !# No active bugs reported now
  !   !# @endbug
  !   !#
  !   !# @todo
  !   !#  &#9744; <br/>
  !   !# @endtodo
  !   !#
  !   !# @warning
  !   !# Now is under CC-GPL License, please see
  !   !# &copy; <https://creativecommons.org/licenses/GPL/2.0/legalcode.pt>
  !   !# @endwarning
  !   !#
  !   !#--- ----------------------------------------------------------------------------------------
  !   !
  !
  !   !local
  !   integer :: igrp
  !   integer :: ielem
  !   integer :: j
  !   integer :: ie
  !   integer :: ig
  !   integer :: ibin
  !   real    :: cpi
  !   real    :: vrfact
  !
  !   integer :: ixyz
  !
  !   call setupraddata()
  !
  !   !  determine which elements are particle number concentrations
  !   !  <ienconc(igroup)> is the element corresponding to particle number
  !   !  concentration in group <igroup>
  !   !
  !   igrp = 0
  !   do ielem = 1, nelem
  !      if (itype(ielem)==i_involatile .or. itype(ielem)==i_volatile) then
  !     igrp          = igrp + 1
  !         ienconc(igrp) = ielem
  !      end if
  !   end do
  !   !
  !   !  determine which group each element belongs to
  !   !  i.e., <igelem(ielem)> is the group to which element <ielem> belongs
  !   !
  !   igrp = 0
  !   do ielem = 1, nelem
  !      if (itype(ielem)==i_involatile .or. itype(ielem)==i_volatile) then
  !         igrp = igrp + 1
  !      end if
  !      igelem(ielem) = igrp
  !   end do
  !   !
  !   ! particle mass densities (nxyz*nbin for each group) -- the user might want
  !   ! to modify this (this code segment does not appear in setupaer subroutine
  !   ! because <igelem> is not defined until this subroutine).
  !   !
  !   do ie = 1,nelem
  !      ig = igelem(ie)
  !     do ibin = 1,nbin
  !       do ixyz = 1,nxyz
  !        rhop3(ixyz,ibin,ig) = rhoelem(ie)
  !        rhopcore3(ixyz,ibin,ig) = rhocore(ie)
  !       end do
  !     end do
  !   end do
  !   !
  !   !
  !   !  set up the particle bins.
  !   !  for each particle group, the mass of a particle in
  !   !  bin j is <rmrat> times that in bin j-1
  !   !
  !   !  rmass(nbin,ngroup) =  bin center mass [g]
  !   !  radius(nbin,ngroup)   =  bin mean (volume-weighted) radius [cm]
  !   !  vol(nbin,ngroup) =  bin center volume [cm^3]
  !   !  dr(nbin,ngroup)  =  bin width in radius space [cm]
  !   !  dv(nbin,ngroup)  =  bin width in volume space [cm^3]
  !   !  dm(nbin,ngroup)  =  bin width in mass space [g]
  !   !
  !   cpi = 4./3.*pi
  !
  !   do igrp = 1, ngroup
  !
  !      rmassmin(igrp) = cpi*rhop3(1,1,igrp)*rmin(igrp)**3
  !
  !      vrfact = ( (3./2./pi/(rmrat(igrp)+1.))**(one/3.) )*    &
  !       ( rmrat(igrp)**(one/3.) - 1. )
  !
  !      do j = 1, nbin
  !     !print *,'lfrdbg->igrp,j,rmassmin(igrp),rmrat(igrp):', &
  !     !     igrp,j,rmassmin(igrp),rmrat(igrp)
  !     !call flush(6)
  !     rmass(j,igrp)     = rmassmin(igrp) * rmrat(igrp)**(j-1)
  !     rmasscore(j,igrp) = pcore/100. * rmass(j,igrp)
  !
  !     rmassup(j,igrp)     = 2.*rmrat(igrp)/(rmrat(igrp)+1.)*rmass(j,igrp)
  !     rmasscoreup(j,igrp) = pcore/100. * rmassup(j,igrp)
  !
  !     dm(j,igrp)  = 2.*(rmrat(igrp)-1.)/(rmrat(igrp)+1.)*rmass(j,igrp)
  !     vol(j,igrp) = rmass(j,igrp) / rhop3(1,1,igrp)
  !
  !     radius(j,igrp)   = ( rmass(j,igrp)/rhop3(1,1,igrp)/cpi )**(one/3.)
  !     xsecta(j,igrp) = pi * radius(j,igrp)**2.
  !     !print *,'lfrdbg->cpi,one,rmasscore(j,igrp),rhopcore3(1,1,igrp):', &
  !     !     cpi,one,rmasscore(j,igrp),rhopcore3(1,1,igrp)
  !     !call flush(6)
  !     rcore(j,igrp)   = ( rmasscore(j,igrp)/rhopcore3(1,1,igrp)/cpi )**(one/3.)
  !     rup(j,igrp)   = ( rmassup(j,igrp)/rhop3(1,1,igrp)/cpi )**(one/3.)
  !     rcoreup(j,igrp) = ( rmasscoreup(j,igrp)/rhopcore3(1,1,igrp)/cpi )**(one/3.)
  !     !print *,'lfrdbg->vrfact,rmass(j,igrp),rhop3(1,1,igrp):', &
  !     !       vrfact,rmass(j,igrp),rhop3(1,1,igrp)
  !
  !     dr(j,igrp)   = vrfact*(rmass(j,igrp)/rhop3(1,1,igrp))**(one/3.)
  !     rlow(j,igrp) = rup(j,igrp) - dr(j,igrp)
  !      end do
  !   end do
  !
  ! end subroutine setupbins


  ! subroutine initaer(m1,pmr,dn0r,ia,iz,ja,jz,nzpmax,totm,n_aer,aotmap,maxz,xland_aerad)
  !   !# Setup the aerosol variables
  !   !#
  !   !# @note
  !   !# ![](http://brams.cptec.inpe.br/wp-content/uploads/2015/11/logo-brams.navigation.png "")
  !   !#
  !   !# **Brief**: allocate and setup the variables for aerosol in atmosphere
  !   !#
  !   !# **Documentation**: <http://brams.cptec.inpe.br/documentation/>
  !   !#
  !   !# **Author**: Rodrigues, L.F. **&#9993;**<mailto:luiz.rodrigues@inpe.br>
  !   !#
  !   !# **Date**: 2018Aug
  !   !# @endnote
  !   !#
  !   !# @changes
  !   !#
  !   !# +
  !   !# @endchanges
  !   !# @bug
  !   !# No active bugs reported now
  !   !# @endbug
  !   !#
  !   !# @todo
  !   !#  &#9744; <br/>
  !   !# @endtodo
  !   !#
  !   !# @warning
  !   !# Now is under CC-GPL License, please see
  !   !# &copy; <https://creativecommons.org/licenses/GPL/2.0/legalcode.pt>
  !   !# @endwarning
  !   !#
  !   !#--- ----------------------------------------------------------------------------------------
  !
  !   integer, intent(in) :: m1,ia,iz,ja,jz,nzpmax,maxz, &
  !   !--(dmk-ccatt-ini)-------------------------------------------------------------------
  !   n_aer
  !   !--(dmk-ccatt-fim)-------------------------------------------------------------------
  !   real, intent(in)    :: pmr(:,:)  ! ((iz-ia+1)*(jz-ja+1),nzpmax)
  !   real, intent(in)    :: dn0r(:,:) ! ((iz-ia+1)*(jz-ja+1),nzpmax)
  !   real, intent(in) :: xland_aerad((iz-ia+1)*(jz-ja+1))
  !   !--(dmk-ccatt-ini)-------------------------------------------------------------------
  !   real, intent(out)   :: totm((iz-ia+1)*(jz-ja+1),m1)
  !   !--(dmk-ccatt-fim)-------------------------------------------------------------------
  !
  !   !local
  !   !--(dmk-ccatt-old)-------------------------------------------------------------------
  !   !    real,dimension((iz-ia+1)*(jz-ja+1),m1) :: totm
  !   !--(dmk-ccatt-fim)-------------------------------------------------------------------
  !
  !   real,dimension((iz-ia+1)*(jz-ja+1),m1) :: r0
  !   real,dimension((iz-ia+1)*(jz-ja+1),m1) :: rsig
  !   real,dimension((iz-ia+1)*(jz-ja+1),m1) :: pdens
  !   integer :: ie,ix,iy
  !   integer :: ielem
  !   integer :: ig
  !   integer :: ip,iend,ij
  !   integer :: j
  !   integer :: k
  !   integer :: kr
  !   integer :: nzz
  !   real    :: arg1
  !   real    :: arg2
  !   real    :: sum((iz-ia+1)*(jz-ja+1))
  !   real    :: totn((iz-ia+1)*(jz-ja+1))
  !   integer :: currsite
  !   integer,dimension((iz-ia+1)*(jz-ja+1)):: idaot
  !   real, intent(in)    :: aotmap(:)          !((iz-ia+1)*(jz-ja+1))
  !   !--(dmk-ccatt-ini)-------------------------------------------------------------------
  !   !    real :: aer_dens(ntotal_aer)
  !   !--(dmk-ccatt-fim)-------------------------------------------------------------------
  !
  !   iend=(iz-ia+1)*(jz-ja+1)
  !   nlayer=maxz
  !   nz_rad=nlayer-1+nz_below
  !
  !   imie=0
  !
  !   allocate(pc(iend,m1,nbin,nelem))
  !   allocate(tauaer(iend,ntotal,nlayer))
  !   allocate(taul(iend,ntotal,nlayer))
  !   allocate(w0(iend,ntotal,nlayer))
  !   allocate(g0(iend,ntotal,nlayer))
  !   allocate(opd(iend,ntotal,nlayer))
  !   allocate(uopd(iend,ntotal,nlayer))
  !   allocate(y3(iend,ntotal,ngauss,nlayer))
  !   allocate(tt(iend,nlayer))
  !   allocate(qv_aerad(iend,m1))
  !   allocate(pc_aerad(iend,nz_rad,nbin,ngroup))
  !   allocate(rdh2o(iend,nlayer))
  !   allocate(lla(iend))
  !   allocate(lls(iend))
  !   allocate(pah2o(iend,ntotal,nlayer))
  !   allocate(paray(iend,ntotal,nlayer))
  !   allocate(taugas(iend,ntotal,nlayer))
  !
  !   nzz = m1 - 1
  !   !
  !   !transfere valores da grade do rams para carma
  !   do k = 1,nzz
  !     kr = k + 1     ! nivel k da grade do carma orresponde ao nivel k + 1 do rams
  !     do ij=1,iend
  !       !    totm = total mass particle concentration (g/cm3)
  !       totm(ij,k)  = pmr(ij,kr) *  dn0r(ij,kr)
  !     end do
  !   end do
  !   !  initialize particle number densities
  !   !  core mass is assumed to be 100% of particle mass
  !   do ielem = 1,nelem
  !     ig = igelem(ielem)
  !     ip = ienconc(ig)
  !     do j = 1,nbin
  !       do k = 1,nzz
  !         do ij = 1,iend
  !           if( ielem .eq. ip )then
  !             ! particle number concentration [#/cm^3]
  !             pc(ij,k,j,ielem) = small_pc
  !           else if( itype(ielem) .eq. i_coremass )then
  !             ! core mass concentration [g/cm^3]
  !             pc(ij,k,j,ielem) = pc(ij,k,j,ip)*rmass(j,ig) * &
  !             fix_coref
  !           else if( itype(ielem) .eq. i_core2mom )then
  !             !  second moment of core mass distribution [ (g/cm^3)^2 ]
  !             pc(ij,k,j,ielem) = pc(ij,k,j,ip)*(rmass(j,ig)*fix_coref)**2
  !           end if
  !         end do
  !       end do
  !     end do
  !   end do
  !   !
  !   !  initial particle distribution: log-normal size distribution
  !   !  for first particle group (which has only one particle element)
  !   !  in a single column
  !   ig = 1
  !   ie = ienconc(ig)
  !
  !   !--(dmk-ccatt-ini)-------------------------------------------------------------------
  !   !-kml-srf-nrosario-270920011
  !   !    if(n_aer == 1) then ! bburn
  !   !    r0(:,:)   = 2.10e-5
  !   rsig(:,:) = 1.55
  !   !  aer_dens(n_aer)= 1.39
  !   !    elseif(n_aer == 2) then ! urban
  !   !    r0(:,:)   = 2.00e-5
  !   !    rsig(:,:) = 1.50
  !   !  aer_dens(n_aer)=1.8
  !   !    elseif(n_aer == 3) then ! continental background
  !   !    r0(:,:)   = 1.80e-5
  !   !    rsig(:,:) = 2.
  !   !  aer_dens(n_aer)=2.2
  !   !    else
  !   !     stop 'n_aer rad carma >3'
  !   !    endif
  !   !    rhop3(1,1,1)=aer_dens(n_aer)
  !
  !   !-kml-srf-nrosario-270920011
  !   !--(dmk-ccatt-fim)-------------------------------------------------------------------
  !
  !   do k = 1,nzz
  !      !
  !      !  log-normal parameters:
  !      !
  !      !    r0   = number mode radius
  !      !    rsig = geometric standard deviation
  !      !    totm = total mass particle concentration (g/cm3) (proveniente do rams)
  !      !
  !      do ij=1,iend
  !       !--(dmk-ccatt-old)-------------------------------------------------------------------
  !       !    r0(ij,k)  = 1.95e-5
  !       !    rsig(ij,k) = 1.62
  !       !--(dmk-ccatt-fim)-------------------------------------------------------------------
  !       idaot(ij) = 1
  !       if(aotmap(ij) .ne. 0)then
  !         currsite = aotmap(ij)
  !       else
  !         if(xland_aerad(ij) .gt. .009)then
  !           currsite = 11
  !         else
  !           currsite = 12
  !         end if
  !       end if
  !       r0(:,:)  = 1.e-4 * caser(idaot(ij),currsite)
  !       pdens(:,:)  = cased(idaot(ij),currsite)
  !       totn(ij) = (6. * totm(ij,k)/(pdens(ij,k)*pi*r0(ij,k)**3))* &
  !               exp((-9./2)*log(rsig(ij,k))**2)
  !     end do
  !     !  adjust prefactor to yield particle number concentration <ntot>
  !     !
  !     sum = 0.
  !     do j = 1,nbin
  !       do ij=1,iend
  !         arg1 = dr(j,ig) / ( sqrt(2.*pi) * radius(j,ig) * log(rsig(ij,k)) )
  !         arg2 = -log( radius(j,ig) / r0(ij,k) )**2 / &
  !              ( 2.*log(rsig(ij,k))**2 )
  !         sum(ij)  = sum(ij) + arg1 * exp( arg2 )
  !       end do
  !     end do
  !     do ij=1,iend
  !       totn(ij) = totn(ij) / sum(ij)
  !     end do
  !     do j = 1,nbin
  !       do ij=1,iend
  !         arg1 = totn(ij) * dr(j,ig) / ( sqrt(2.*pi) * radius(j,ig) * &
  !            log(rsig(ij,k)) )
  !         arg2 = -log( radius(j,ig) / r0(ij,k) )**2 / &
  !            ( 2.*log(rsig(ij,k))**2 )
  !         pc(ij,k,j,ie) = max( arg1 * exp( arg2 ), real(small_pc) )
  !       end do
  !     end do
  !   end do
  !
  ! end subroutine initaer

!   subroutine oppr(ia,iz,ja,jz,m1,m2,m3,aot11,n_aer,ntotal_aer,aotmap, &
!                 daer,wol,gol,taucld,wcld,gcld, &
!                 photojmethod,chemistry, &
!                 ngrid,ccatt,aerosol,xland_aerad,iwl_aerad,iwp_aerad, &
!                 lwl_aerad,lwp_aerad,rain_aerad,gc,p_top,p_aerad,t_aerad)
!     !# Calculates optical properties
!     !#
!     !# @note
!     !# ![](http://brams.cptec.inpe.br/wp-content/uploads/2015/11/logo-brams.navigation.png "")
!     !#
!     !# **Brief**: calculates optical properties such as single scattering albedo,
!     !# asymmetry parameter, etc.
!     !#
!     !# **Documentation**: <http://brams.cptec.inpe.br/documentation/>
!     !#
!     !# **Author**: Rodrigues, L.F. **&#9993;**<mailto:luiz.rodrigues@inpe.br>
!     !#
!     !# **Date**: 2018Aug
!     !#
!     !# the top layer is a dummy. it contains a defined gas amount.
!     !# different models will require different treatment of this.
!     !# calculate total optical depth including gases. then
!     !# given the aerosol optical depths and cloud optical depths,
!     !# calculate final optical properties. we use a delta
!     !# two stream approach to find w0, single scattering albedo,
!     !# g0, asymmmetry parameter, taul, layer optical depth,
!     !# opd, cumulative optical depth to base of layer.
!     !#
!     !# Assume that p is same on all sigma levels. if psurface
!     !# varies a lot, then we may need to calculate tauo3,
!     !# tauco2, tauo2 for each point.
!     !#
!     !# @endnote
!     !#
!     !# @changes
!     !#
!     !# +
!     !# @endchanges
!     !# @bug
!     !# No active bugs reported now
!     !# @endbug
!     !#
!     !# @todo
!     !#  &#9744; <br/>
!     !# @endtodo
!     !#
!     !# @warning
!     !# Now is under CC-GPL License, please see
!     !# &copy; <https://creativecommons.org/licenses/GPL/2.0/legalcode.pt>
!     !# @endwarning
!     !#
!     !#--- ----------------------------------------------------------------------------------------
!     implicit none
!
!     integer,intent(in) :: ia
!     integer,intent(in) :: iz
!     integer,intent(in) :: ja
!     integer,intent(in) :: jz
!     integer,intent(in) :: m1
!     integer,intent(in) :: m2
!     integer,intent(in) :: m3
! !--(dmk-ccatt-ini)----------------------------------------------------------
!     integer,intent(in) :: n_aer
!     integer,intent(in) :: ntotal_aer
!     integer,intent(in) :: ccatt,aerosol
!     integer,intent(in) :: ngrid
!     integer,intent(in) :: chemistry
! !--(dmk-ccatt-fim)----------------------------------------------------------
!     character(len=*),intent(in) :: photojmethod
!     real, intent(in) :: xland_aerad((iz-ia+1)*(jz-ja+1))
!     real, intent(in) :: iwl_aerad((iz-ia+1)*(jz-ja+1),m1)
!     real, intent(in) :: lwl_aerad((iz-ia+1)*(jz-ja+1),m1)
!     real, intent(in) :: iwp_aerad((iz-ia+1)*(jz-ja+1),m1)
!     real, intent(in) :: lwp_aerad((iz-ia+1)*(jz-ja+1),m1)
!     real, intent(in) :: rain_aerad((iz-ia+1)*(jz-ja+1))
!     real, intent(in) :: gc((iz-ia+1)*(jz-ja+1),m1,ngas)
!     real, intent(in) :: p_top((iz-ia+1)*(jz-ja+1))
!     real, intent(in) :: p_aerad((iz-ia+1)*(jz-ja+1),m1)
!     real, intent(in) :: t_aerad((iz-ia+1)*(jz-ja+1),m1)
!
!     real,intent(out) :: daer(4,m1,m2,m3)
!     real,intent(out) :: wol((iz-ia+1)*(jz-ja+1),ntotal,nlayer)
!     !# single scattering albedo *** delta scaled ***
!     real,intent(out) :: gol((iz-ia+1)*(jz-ja+1),ntotal,nlayer)
!     !# asymmetry parameter *** delta scaled ***
!     real,intent(out) :: taucld((iz-ia+1)*(jz-ja+1),ntotal,nlayer)
!     real,intent(out) :: wcld((iz-ia+1)*(jz-ja+1),ntotal,nlayer)
!     real,intent(out) :: gcld((iz-ia+1)*(jz-ja+1),ntotal,nlayer)
!
!
!     real :: aot11(:) !((iz-ia+1)*(jz-ja+1))         !kml2
!     real :: aotmap(:) !((iz-ia+1)*(jz-ja+1))        !kml2
!
!     real,dimension((iz-ia+1)*(jz-ja+1),ntotal,nlayer) :: taua, taus, g01
!     real,dimension((iz-ia+1)*(jz-ja+1),ntotal,nlayer) :: dutaua, dutaua2
!
!     integer :: i,i1,j1,kk,k
!     integer :: ig
!     integer :: iradgas
!     integer :: j
!     integer :: l
!     real    :: cco((iz-ia+1)*(jz-ja+1))
!     real    :: den
!     real    :: denom
!     real    :: fo
!     real    :: pcorr
!     real    :: qcorr
!     real    :: ttas
!     real    :: tauh2o((iz-ia+1)*(jz-ja+1),ntotal,nlayer)
!     real    :: utaul((iz-ia+1)*(jz-ja+1),ntotal,nlayer)
!   !  real    :: uw0((iz-ia+1)*(jz-ja+1),ntotal,nlayer)
!     real    :: wot((iz-ia+1)*(jz-ja+1),ntotal)
!     real    :: got((iz-ia+1)*(jz-ja+1),ntotal)
!     integer,dimension((iz-ia+1)*(jz-ja+1)):: idaot
!
! !--(dmk-ccatt-ini)----------------------------------------------------------
!     !lfr
!     integer,dimension(4) :: nwl,nwu,npl,npu
!     real,dimension(4) :: wl
!     real :: cm
!     real,dimension(4,(iz-ia+1)*(jz-ja+1),m1) :: daer_
! !--(dmk-ccatt-fim)----------------------------------------------------------
!
!     integer :: ij,iend,jjj,in
!
! !kmlnew
!     real,dimension((iz-ia+1)*(jz-ja+1),ntotal,nlayer) :: wolc,woice,worain,gl,gice,grain
!     real,dimension((iz-ia+1)*(jz-ja+1),ntotal,nlayer) :: denc
!     real,dimension((iz-ia+1)*(jz-ja+1),ntotal,nlayer) :: taucldlw,taucldice,taurain
!     real,dimension((iz-ia+1)*(jz-ja+1),nlayer) :: corr,reffi
!
!     real,dimension((iz-ia+1)*(jz-ja+1),nwave) :: rdqextnew,wonew,gonew
!     real x_teste
!     integer :: currsite
!
!     real,dimension((iz-ia+1)*(jz-ja+1),ntotal,nlayer,ntotal_aer) :: tauaer_x
!     real,dimension((iz-ia+1)*(jz-ja+1),ntotal,nlayer,ntotal_aer) :: wol_x
!     real,dimension((iz-ia+1)*(jz-ja+1),ntotal,nlayer,ntotal_aer) :: gol_x
!
!     if(.not. allocated(uw0)) then
!       allocate(uw0(ntotal,nlayer))
!       allocate(ug0(ntotal,nlayer))
!     endif
!
!     corr=0.0
!     denc=0.0
!     reffi=0.0
!     taucldlw=0.0
!     taucldice=0.0
!     taurain=0.0
!     wolc=0.0
!     woice=0.0
!     worain=0.0
!     gl=0.0
!     gice=0.0
!     grain=0.0
!     taucld=0.0
!     wcld=0.0
!     gcld=0.0
!     rdqextnew=0.0
!     wonew=0.0
!     gonew=0.0
!
!     !kmlnew
!     iend=(iz-ia+1)*(jz-ja+1)
!     currsite = 1
!     if (iprocopio == 1 .and. imie == 1) then
!       !aot11=0.1          !tmp kml2
!       do ij=1,iend
!         !idaot(ij) = max(min(int(10*((anint(10.*aot11(ij))/10.)+0.1)/2.),9),1)
!         !at moment, just one optical model per site!
!         idaot(ij) = 1
!         if(aotmap(ij) .ne. 0)then
!           currsite = aotmap(ij)
!         else
!           if(xland_aerad(ij) .gt. .009)then
!             currsite = 11
!           else
!             currsite = 12
!           end if
!         end if
!         do  l = 1,nwave
!           rdqextnew(ij,l) = casee(idaot(ij),l, currsite)
!           wonew(ij,l)   = casew(idaot(ij),l, currsite)
!           gonew(ij,l)   = caseg(idaot(ij),l, currsite)
!           !if(l.eq.11) print*,'ext,wo,go=',rdqextnew(ij,l),wonew(ij,l),gonew(ij,l),ij
!         end do
!       end do
!       taua=0.0
!       do j=1,nlayer
!         do ig = 1,ngroup
!           do  i = 1,nrad
!             do  l = 1,ntotal
!               do ij=1,iend
!                 taua(ij,l,j)=max(real(epsilon), taua(ij,l,j)+rdqextnew(ij,nprob(l))*xsecta(i,ig)* &
!                    pc_aerad(ij,j,i,ig) )
!                 wol(ij,l,j)    = wonew(ij,nprob(l))
!                 gol(ij,l,j)    = gonew(ij,nprob(l))
!               end do
!             end do
!           end do
!         end do
!       end do
!       !--(dmk-ccatt-ini)----------------------------------------------------------
!       ! kml- saving properties by aerosol type
!       tauaer_x(1:iend,1:ntotal,1:nlayer,n_aer) =taua  (1:iend,1:ntotal,1:nlayer)
!       wol_x   (1:iend,1:ntotal,1:nlayer,n_aer) =wol   (1:iend,1:ntotal,1:nlayer)
!       gol_x   (1:iend,1:ntotal,1:nlayer,n_aer) =gol   (1:iend,1:ntotal,1:nlayer)
!       ! kml- saving properties by aerosol type
!       !--(dmk-ccatt-fim)----------------------------------------------------------
!     else !(iprocopio /= 1 .or. imie /= 1))
!       taua=0.0
!       taus=0.0
!       g01=0.0
!       do j=1,nlayer
!         do ig = 1,ngroup
!           do  i = 1,nrad
!             do  l = 1,nwave
!               do ij=1,iend
!                 taua(ij,l,j)=taua(ij,l,j)+rdqext(i,ig,l)*xsecta(i,ig)* &
!                   pc_aerad(ij,j,i,ig)
!                 taus(ij,l,j)=taus(ij,l,j)+qscat(i,ig,l)*xsecta(i,ig)* &
!                   pc_aerad(ij,j,i,ig)
!                 g01(ij,l,j) =g01(ij,l,j) +qbrqs(i,ig,l)*xsecta(i,ig)* &
!                   pc_aerad(ij,j,i,ig)
!               end do
!             end do
!           end do
!         end do
!         do l= 1,ntotal
!           do ij=1,iend
!             tauaer(ij,l,j) = max(taua(ij,nprob(l),j),real(epsilon))
!             wol(ij,l,j)   = taus(ij,nprob(l),j)/tauaer(ij,l,j)
!             ttas=1.0
!             if( wol(ij,l,j) /= 0. ) ttas = taus(ij,nprob(l),j)
!             gol(ij,l,j)  = g01(ij,nprob(l),j)/ttas
!           end do
!         end do
!       end do
!       imie = 1
!     end if
!
!     ! sol. paliativa para manter reprodutibilidade no oper5km e ccatt-4.3:
!     if (ccatt==1 .and. aerosol>0) then
!       !! codigo usado no ccatt-brams 4.3.3
!       !!
!       !! desativado: reprodutibilidade na temperatura e precipitacao da ver operacional, mas
!       !!             perda de reprodutibilidade na aot550 da ver 4.3.3
!       !!
!       !! ativado: reprodutibilidade na aot550 da versao 4.3.3, mas perda de reprodutibilidade
!       !!          na temperatura e precipitacao da ver operacional
!       if(n_aer < ntotal_aer) then
!         return ! 3 = ntotal of aerosols.
!       else
!         ! kml- saving properties by aerosol type
!         tauaer=0.0 ;wol=0.0 ; gol=0.0
!         do i=1,ntotal_aer
!           tauaer(:,:,:)=tauaer(:,:,:)+ tauaer_x(:,:,:,i)
!         enddo
!         do i=1,ntotal_aer
!           wol(:,:,:)=wol(:,:,:)+wol_x(:,:,:,i)*tauaer_x(:,:,:,i)
!           gol(:,:,:)=gol(:,:,:)+gol_x(:,:,:,i)*tauaer_x(:,:,:,i)
!         enddo
!         wol(:,:,:)=wol(:,:,:)/tauaer(:,:,:)
!         gol(:,:,:)=gol(:,:,:)/tauaer(:,:,:)
!         ! kml- saving properties by aerosol type
!       endif
!     endif
!
!     do j=1,nlayer
!       do ij=1,iend
!         if (xland_aerad(ij).ge..009) then
!           reffi(ij,j) =  7.0 * 1.e+3 * lwl_aerad(ij,j) + 5.5
!         else
!           reffi(ij,j) =  9.5 * 1.e+3 * lwl_aerad(ij,j) + 4.0
!         end if
!         !--(dmk-ccatt-ini)----------------------------------------------------------
!         reffi(ij,j)=min(reffi(ij,j),37.) !ner avoiding adapt(plank) problem. max effetive radius (reffi) for liquid cloud drop 37 micron
!         !--(dmk-ccatt-fim)----------------------------------------------------------
!         corr(ij,j) = 1.047 - 0.913e-4 * (tt(ij,j)-273.16) + 0.203e-3 * &
!                    (tt(ij,j)-273.16) **2 - 0.106e-4 * (tt(ij,j)-273.16) **3
!         corr(ij,j) = max(corr(ij,j),real((epsilon)))
!       end do
!     end do
!     do j=1,nlayer
!       do l= 1,ntotal
!         do ij=1,iend
!           if( j .eq. 1) taurain(ij,l,j) = 0.00018 * rain_aerad(ij) * 2000.0
!           taucldlw(ij,l,j)= 1.e+3 * lwp_aerad(ij,j) *(ta(l)/reffi(ij,j)+tb(l)/reffi(ij,j)**2)
!           worain(ij,l,j) = 1. - 0.45
!           wolc(ij,l,j) = (1. - wa(l)) + wb(l) * reffi(ij,j)
!           gl(ij,l,j)   = ga(l) + gb(l) * reffi(ij,j)
!           grain(ij,l,j) = 0.95
!           denc(ij,l,j)      = 1. / (tia(l) + tib(l) * 1.0e+3 * iwl_aerad(ij,j))
!           taucldice(ij,l,j) = corr(ij,j) * 1.0e+3 * iwp_aerad(ij,j) * denc(ij,l,j)
!           !srf-evitando bug - 0**0
!           if(wib(l) < 1.e-5) then
!             x_teste = 0.
!           else
!             x_teste = (1.0e+3 * iwl_aerad(ij,j))**wib(l)
!           endif
!           woice(ij,l,j) =(1.0 - wia(l)*x_teste) * &
!                  ( 1.0 - gama(l) * (corr(ij,j) - 1)/ corr(ij,j) )
!           !srf-evitando bug - 0**0
!           if(gib(l) < 1.e-5) then
!             !    x_teste = 0.
!             x_teste = 1.
!           else
!             x_teste = (1.0e+3 * iwl_aerad(ij,j))**gib(l)
!           endif
!           gice(ij,l,j) = (gia(l)*x_teste) * &
!                  ( 1.0 + alpha(l) * (corr(ij,j) - 1)/ corr(ij,j) )
!           if (l>=91 .and. l<=113) then
!             taucldlw(ij,l,j)= 1.0e+3 * lwp_aerad(ij,j) * ta(l) * exp(tb(l)* reffi(ij,j))
!           end if
!           if (l>=114 .and. l<=154) then
!             taucldlw(ij,l,j)=  1.0e+3 * lwp_aerad(ij,j) * ( ta(l) + tb(l)* reffi(ij,j))
!             gl(ij,l,j) = 1. - ga(l) * exp( gb(l) * reffi(ij,j))
!           end if
!           taucld(ij,l,j)= taucldlw(ij,l,j) + taucldice(ij,l,j) + taurain(ij,l,j)
!           if ( taucld(ij,l,j).gt.epsilon) then
!             wcld(ij,l,j) =  (wolc(ij,l,j) *  taucldlw(ij,l,j)  + &
!                              woice(ij,l,j) * taucldice(ij,l,j) + &
!                              worain(ij,l,j) * taurain(ij,l,j)) &
!                              / taucld(ij,l,j)
!             gcld(ij,l,j) = ( wolc(ij,l,j) *  taucldlw(ij,l,j)*   gl(ij,l,j) + &
!             woice(ij,l,j) * taucldice(ij,l,j)* gice(ij,l,j) + &
!             worain(ij,l,j) * taurain(ij,l,j) * grain(ij,l,j)) &
!             / (wcld(ij,l,j) * taucld(ij,l,j))
!           else
!             wcld(ij,l,j) = 1.0
!             gcld(ij,l,j) = 0.0
!           endif
!         end do
!       end do
!     end do
!     iradgas = 1 !iradgas = 0: no gas in radiative xfer
!     do  j = 1,nlayer
!       kk = max( 1, j-1 )
!       !
!       !   bergstrom water vapor continuum fix:
!       !
!       !    <qcorr> is layer average water vapor mixing ratio
!       !    <pcorr> is layer average pressure
!       !
!       !   for layer 0, calculate mixing ratio [g/g] from vapor column [g/cm^2]
!       !   and average pressure [dyne/cm^2]
!       !
!       if( j == 1 )then
!         do ij=1,iend
!           qcorr = rdh2o(ij,1) * g / p_top(ij)
!           pcorr = p_aerad(ij,1) / 2.
!           cco(ij) = exp(1800./t_aerad(ij,kk))*(qcorr*pcorr/2.87 + pcorr/4610.)
!         end do
!       else
!         do ij=1,iend
!           qcorr = qv_aerad(ij,kk)
!           pcorr = p_aerad(ij,kk)
!           cco(ij) = exp(1800./t_aerad(ij,kk))*(qcorr*pcorr/2.87 + pcorr/4610.)
!         end do
!       end if
!       do  l   = 1,ntotal
!         do ij=1,iend
!           if (l>=lls(ij) .and. l<=lla(ij)) then
!             tauh2o(ij,l,j) = rdh2o(ij,j)*pah2o(ij,l,j)
!             !   bergstrom water vapor continuum fix (next two statements)
!             if( l > nsolp+30 .and. l <= nsolp+36 ) then
!               tauh2o(ij,l,j) = rdh2o(ij,j)*pah2o(ij,l,j)*cco(ij)
!             else
!               tauh2o(ij,l,j) = tauh2o(ij,l,j)
!             end if
!             if (l > nsolp+36) tauh2o(ij,l,j) = tauh2o(ij,l,j) + &
!               cco(ij)*rdh2o(ij,j)*contnm(l-nsolp)
!             taul(ij,l,j)   = tauh2o(ij,l,j)+taugas(ij,l,j)+ &
!               paray(ij,l,j)+tauaer(ij,l,j)+taucld(ij,l,j)
!             if (iradgas == 0) taul(ij,l,j) = tauaer(ij,l,j)
!             if( taul(ij,l,j) < epsilon ) taul(ij,l,j) = epsilon
!           end if
!         end do
!       end do
!       do  l   = 1,ntotal
!         do ij=1,iend
!           if (l>=lls(ij) .and. l<=lla(ij)) then
!             utaul(ij,l,j)  = taul(ij,l,j)
!             wot(ij,l)    = (paray(ij,l,j)+tauaer(ij,l,j)*wol(ij,l,j)+  &
!             taucld(ij,l,j)*wcld(ij,l,j))/taul(ij,l,j)
!             if (iradgas == 0) wot(ij,l) = wol(ij,l,j)
!             wot(ij,l)       = min(1.-real(epsilon),wot(ij,l))
!             !     uw0(ij,l,j)    = wot(ij)
!             denom     = (paray(ij,l,j)+taucld(ij,l,j)*wcld(ij,l,j)+ &
!             tauaer(ij,l,j)*wol(ij,l,j))
!             !if( denom <= epsilon ) denom = epsilon
!             if( denom > epsilon ) then
!               got(ij,l) = ( wcld(ij,l,j)*gcld(ij,l,j)*taucld(ij,l,j) + &
!                   gol(ij,l,j)* wol(ij,l,j)*tauaer(ij,l,j) ) / denom
!               got(ij,l)=max(real(epsilon), got(ij,l))
!             else
!               got(ij,l) = 0.
!             end if
!             if (iradgas == 0) got(ij,l) = gol(ij,l,j)
!           end if
!         end do
!       end do
!       do  l   = 1,ntotal
!         do ij=1,iend
!           if (l>=lls(ij) .and. l<=lla(ij)) then
!             !ug0(l,j)  = got(ij)
!             fo        = got(ij,l)**2
!             den       = 1.-wot(ij,l)*fo
!             taul(ij,l,j)   = taul(ij,l,j) * den
!             w0(ij,l,j)      = (1.-fo)*wot(ij,l)/den
!             g0(ij,l,j)      = got(ij,l)/(1.+got(ij,l))
!             opd(ij,l,j)    = 0.0
!             opd(ij,l,j)    = opd(ij,l,kk)+taul(ij,l,j)
!             uopd(ij,l,j)   = 0.0
!             uopd(ij,l,j)   = uopd(ij,l,kk)+utaul(ij,l,j)
!           end if
!         end do
!       end do
!     end do
!     if(ir_aerad == 1) then
!       do   j =   1,nlayer
!         do  i =   1,ngauss
!           do  l =   1,ntotal
!             do ij=1,iend
!               if(l>=lls(ij) .and. l<=lla(ij)) &
!                 y3(ij,l,i,j) = exp(-taul(ij,l,j)/gangle(i))
!             end do
!           end do
!         end do
!       end do
!     end if
!     !--(dmk-ccatt-ini)----------------------------------------------------------
!     if (trim(photojmethod) == 'fast-jx' .and. chemistry > 0)  then
!       return
!       !lfr
!       !interpolating to obtain opt. depth aerosol
!       !naer(1)=300nm - nwave=f(3)=296/f(4)=319    - nprob(3)/nprob(4)
!       !naer(2)=400nm - nwave=f(6)=365/f(7)=420    - nprob(6)/nprob(7)
!       !naer(3)=600nm - nwave=f(13)=598/f(14)=660  - nprob(13)/nprob(14)
!       !naer(4)=999nm - nwave=f(21)=926/f(22)=1005 - nprob(28)/nprob(32)
!       nwl(1)=3;nwu(1)=4;wl(1)=.300
!       nwl(2)=6;nwu(2)=7;wl(2)=.400
!       nwl(3)=13;nwu(3)=14;wl(3)=.600
!       nwl(4)=21;nwu(4)=22;wl(4)=.999
!       npl(1)=3;npu(1)=4
!       npl(2)=6;npu(2)=7
!       npl(3)=13;npu(3)=14
!       npl(4)=28;npu(4)=32
!       do k=1,m1
!         do ij=1,iend
!           do l=1,4
!             cm=wl(l)/(wave(nwl(l))+wave(nwu(l)))
!             daer_(l,ij,k)=(taucld(ij,npl(l),k)+taucld(ij,npu(l),k))*cm
!           end do
!         end do
!       end do
!       call c_3d_4d(daer_,daer(:,:,ia:iz,ja:jz),m1,ia,iz,ja,jz,iend,4) !lfr
!     endif
!
!   end subroutine oppr

  ! subroutine prerad(m1,dztr,fmapt,ia,iz,ja,jz,nzpmax,m2,m3,rhoa,gc)
  !   !# Prepare the aerad vars to rad
  !   !#
  !   !# @note
  !   !# ![](http://brams.cptec.inpe.br/wp-content/uploads/2015/11/logo-brams.navigation.png "")
  !   !#
  !   !# **Brief**: Prepare the aerad vars to rad
  !   !#
  !   !# **Documentation**: <http://brams.cptec.inpe.br/documentation/>
  !   !#
  !   !# **Author**: Rodrigues, L.F. **&#9993;**<mailto:luiz.rodrigues@inpe.br>
  !   !#
  !   !# **Date**: 2018Aug
  !   !#
  !   !#
  !   !#  note that vertical index in radiative transfer model domain is reversed
  !   !#  for cartesian coordinates.
  !   !#
  !   !#  indices <ix> and <iy> are passed through global common block.
  !   !#
  !   !# @endnote
  !   !#
  !   !# @changes
  !   !#
  !   !# +
  !   !# @endchanges
  !   !# @bug
  !   !# No active bugs reported now
  !   !# @endbug
  !   !#
  !   !# @todo
  !   !#  &#9744; <br/>
  !   !# @endtodo
  !   !#
  !   !# @warning
  !   !# Now is under CC-GPL License, please see
  !   !# &copy; <https://creativecommons.org/licenses/GPL/2.0/legalcode.pt>
  !   !# @endwarning
  !   !#
  !   !#--- ----------------------------------------------------------------------------------------
  !
  !   integer, intent(in) :: m1,m2,m3,ia,iz,ja,jz,nzpmax
  !   real, intent(in)    :: dztr(:,:)  ! ((iz-ia+1)*(jz-ja+1),nzpmax)
  !   real, intent(in)    :: fmapt(:) ! ((iz-ia+1)*(jz-ja+1))
  !   real, intent(in)    :: rhoa(:,:) !((iz-ia+1)*(jz-ja+1),m1)
  !   real, intent(in)    :: gc((iz-ia+1)*(jz-ja+1),m1,ngas)
  !
  !   integer :: ibin
  !   integer :: i,j,ij,iend
  !   integer :: iep
  !   integer :: igas
  !   integer :: igroup
  !   integer :: k,kk
  !   integer :: nzz
  !   real    :: xymet
  !
  !   iend=(iz-ia+1)*(jz-ja+1)
  !
  !   !  load profiles of temperature [k], water vapor [g/g], and
  !   !  aerosol particle concentrations [#/cm^2]
  !   !
  !   igas = 1
  !   !srf
  !   nzz = m1 - 1
  !
  !   do k = 1,nzz
  !      !  reverse the vertical index when in cartesian coordinates
  !      kk = nzz + 1 - k
  !      ! for radiation code: qv-aerad have g[h20]/g[ar] unit
  !      do ij=1,iend
  !         qv_aerad(ij,kk) = gc(ij,k,igas) / rhoa(ij,k)
  !      end do
  !      do igroup = 1,ngroup
  !     iep = ienconc(igroup)
  !     do ibin = 1,nbin
  !       do ij=1,iend
  !              xymet = fmapt(ij)*fmapt(ij)
  !          pc_aerad(ij,kk,ibin,igroup) = pc(ij,k,ibin,iep) *  &
  !                (1./dztr(ij,k)) / xymet
  !       end do
  !     end do
  !   end do
  !   end do
  !   !
  !
  ! end subroutine prerad

  subroutine setupraddata()
    !# Fill the rdqext array values and other variables
    !#
    !# @note
    !# ![](http://brams.cptec.inpe.br/wp-content/uploads/2015/11/logo-brams.navigation.png "")
    !#
    !# **Brief**: Fill the rdqext array values and other variables
    !#
    !# **Documentation**: <http://brams.cptec.inpe.br/documentation/>
    !#
    !# **Author**: Rodrigues, L.F. **&#9993;**<mailto:luiz.rodrigues@inpe.br>
    !#
    !# **Date**: 2018Aug
    !#
    !# @endnote
    !#
    !# @changes
    !#
    !# +
    !# @endchanges
    !# @bug
    !# No active bugs reported now
    !# @endbug
    !#
    !# @todo
    !#  &#9744; <br/>
    !# @endtodo
    !#
    !# @warning
    !# Now is under CC-GPL License, please see
    !# &copy; <https://creativecommons.org/licenses/GPL/2.0/legalcode.pt>
    !# @endwarning
    !#
    !#--- ----------------------------------------------------------------------------------------

        integer :: i,n,w
        real :: sq3,jdble,jn,tpi,cpcon,fdegday

        rdqext(1:30,1:1,1:6)=reshape( (/                                 &
         0.1238996e-05, 0.9044850e-03, 0.1058328e-02, 0.1310231e-02, &
         0.1623058e-02, 0.2011839e-02, 0.2492962e-02, 0.3089402e-02, &
         0.3829632e-02, 0.4747202e-02, 0.5886665e-02, 0.7303554e-02, &
         0.9068833e-02, 0.1127584e-01, 0.1405085e-01, 0.1757244e-01, &
         0.2210986e-01, 0.2809986e-01, 0.3631130e-01, 0.4820201e-01, &
         0.6671599e-01, 0.9805208e-01, 0.1554057e+00, 0.2653104e+00, &
         0.4692753e+00, 0.7888026e+00, 0.1204756e+01, 0.1752262e+01, &
         0.2199486e+01, 0.2739768e+01, 0.2356571e-05, 0.8107370e-03, &
         0.9485792e-03, 0.1175354e-02, 0.1457698e-02, 0.1806319e-02, &
         0.2237227e-02, 0.2773230e-02, 0.3436975e-02, 0.4259836e-02, &
         0.5281698e-02, 0.6551373e-02, 0.8131125e-02, 0.1010238e-01, &
         0.1257303e-01, 0.1569186e-01, 0.1967621e-01, 0.2486489e-01, &
         0.3183057e-01, 0.4162014e-01, 0.5628673e-01, 0.8007501e-01, &
         0.1220185e+00, 0.2010520e+00, 0.3521606e+00, 0.6149174e+00, &
         0.9806738e+00, 0.1471468e+01, 0.1975219e+01, 0.2499236e+01, &
         0.4482199e-05, 0.6878105e-03, 0.8052588e-03, 0.9958311e-03, &
         0.1234672e-02, 0.1529180e-02, 0.1895103e-02, 0.2348306e-02, &
         0.2910469e-02, 0.3606677e-02, 0.4470860e-02, 0.5543770e-02, &
         0.6876979e-02, 0.8536757e-02, 0.1060989e-01, 0.1321133e-01, &
         0.1650270e-01, 0.2072231e-01, 0.2624893e-01, 0.3373386e-01, &
         0.4438854e-01, 0.6062073e-01, 0.8745370e-01, 0.1355955e+00, &
         0.2271680e+00, 0.4006258e+00, 0.6897491e+00, 0.1075740e+01, &
         0.1600417e+01, 0.2066453e+01, 0.8525142e-05, 0.5920068e-03, &
         0.6939178e-03, 0.8606558e-03, 0.1066172e-02, 0.1321362e-02, &
         0.1637412e-02, 0.2028739e-02, 0.2514813e-02, 0.3116922e-02, &
         0.3863455e-02, 0.4789023e-02, 0.5938904e-02, 0.7368284e-02, &
         0.9149896e-02, 0.1137746e-01, 0.1417908e-01, 0.1773632e-01, &
         0.2232307e-01, 0.2838602e-01, 0.3671290e-01, 0.4880296e-01, &
         0.6768916e-01, 0.9976597e-01, 0.1586369e+00, 0.2715197e+00, &
         0.4801895e+00, 0.8039033e+00, 0.1225015e+01, 0.1773117e+01, &
         0.1621482e-04, 0.5126877e-03, 0.5990996e-03, 0.7428692e-03, &
         0.9220786e-03, 0.1141519e-02, 0.1414444e-02, 0.1753329e-02, &
         0.2172156e-02, 0.2691440e-02, 0.3335611e-02, 0.4134388e-02, &
         0.5125767e-02, 0.6357079e-02, 0.7889097e-02, 0.9800074e-02, &
         0.1219303e-01, 0.1521039e-01, 0.1905701e-01, 0.2405036e-01, &
         0.3071964e-01, 0.4002312e-01, 0.5382273e-01, 0.7594469e-01, &
         0.1145147e+00, 0.1866735e+00, 0.3250482e+00, 0.5710782e+00, &
         0.9245852e+00, 0.1392398e+01, 0.3084059e-04, 0.4278156e-03, &
         0.4990920e-03, 0.6195636e-03, 0.7684358e-03, 0.9515302e-03, &
         0.1178236e-02, 0.1461325e-02, 0.1809853e-02, 0.2242170e-02, &
         0.2778809e-02, 0.3443943e-02, 0.4269208e-02, 0.5292946e-02, &
         0.6565067e-02, 0.8148359e-02, 0.1012359e-01, 0.1259963e-01, &
         0.1572569e-01, 0.1971983e-01, 0.2492236e-01, 0.3190922e-01, &
         0.4173374e-01, 0.5646301e-01, 0.8037232e-01, 0.1225615e+00, &
         0.2020946e+00, 0.3541155e+00, 0.6180223e+00, 0.9846209e+00/),&
         (/30,1,6/))

    rdqext(1:30,1:1,7:12)=reshape( (/&
         0.5865879e-04, 0.3386404e-03, 0.3943375e-03, 0.4902230e-03, &
         0.6069456e-03, 0.7545453e-03, 0.9325470e-03, 0.1156643e-02, &
         0.1432099e-02, 0.1774964e-02, 0.2199463e-02, 0.2725592e-02, &
         0.3378012e-02, 0.4187341e-02, 0.5191328e-02, 0.6438720e-02, &
         0.7990645e-02, 0.9926975e-02, 0.1235256e-01, 0.1541248e-01, &
         0.1931667e-01, 0.2439146e-01, 0.3118407e-01, 0.4068905e-01, &
         0.5484687e-01, 0.7765548e-01, 0.1176138e+00, 0.1926048e+00, &
         0.3362654e+00, 0.5893933e+00, 0.1115690e-03, 0.2516592e-03, &
         0.2934801e-03, 0.3651093e-03, 0.4550150e-03, 0.5632643e-03, &
         0.6970990e-03, 0.8643277e-03, 0.1070622e-02, 0.1324547e-02, &
         0.1642295e-02, 0.2034783e-02, 0.2521489e-02, 0.3124935e-02, &
         0.3872860e-02, 0.4801260e-02, 0.5953945e-02, 0.7387332e-02, &
         0.9173343e-02, 0.1140706e-01, 0.1421631e-01, 0.1778384e-01, &
         0.2238511e-01, 0.2846927e-01, 0.3683005e-01, 0.4897850e-01, &
         0.6797409e-01, 0.1002687e+00, 0.1595861e+00, 0.2733425e+00, &
         0.2122042e-03, 0.1571589e-03, 0.1840258e-03, 0.2313218e-03, &
         0.2901404e-03, 0.3512961e-03, 0.4390195e-03, 0.5448884e-03, &
         0.6724349e-03, 0.8375491e-03, 0.1036835e-02, 0.1285574e-02, &
         0.1592136e-02, 0.1972729e-02, 0.2444628e-02, 0.3029664e-02, &
         0.3754555e-02, 0.4654815e-02, 0.5771693e-02, 0.7160614e-02, &
         0.8890757e-02, 0.1105248e-01, 0.1376909e-01, 0.1721303e-01, &
         0.2164283e-01, 0.2747480e-01, 0.3543673e-01, 0.4689975e-01, &
         0.6461873e-01, 0.9437821e-01, 0.4036125e-03, 0.4093335e-01, &
         0.5522372e-01, 0.7828710e-01, 0.1187611e+00, 0.1948035e+00, &
         0.3404121e+00, 0.5961001e+00, 0.9566938e+00, 0.1437752e+01, &
         0.1951394e+01, 0.2464083e+01, 0.2949664e+01, 0.3406605e+01, &
         0.3629449e+01, 0.3356592e+01, 0.2565616e+01, 0.1934376e+01, &
         0.2454620e+01, 0.2495687e+01, 0.2086732e+01, 0.2298010e+01, &
         0.2203053e+01, 0.2162956e+01, 0.2135190e+01, 0.2162642e+01, &
         0.2130731e+01, 0.2120723e+01, 0.2111154e+01, 0.2073417e+01, &
         0.3014119e-01, 0.3649874e-01, 0.4848227e-01, 0.6716944e-01, &
         0.9884995e-01, 0.1569090e+00, 0.2682001e+00, 0.4743640e+00, &
         0.7958630e+00, 0.1214206e+01, 0.1762091e+01, 0.2209994e+01, &
         0.2747663e+01, 0.3190050e+01, 0.3513948e+01, 0.3566287e+01, &
         0.2919923e+01, 0.2062144e+01, 0.2168095e+01, 0.2654343e+01, &
         0.2093068e+01, 0.2346398e+01, 0.2183643e+01, 0.2237450e+01, &
         0.2201363e+01, 0.2169828e+01, 0.2122523e+01, 0.2131855e+01, &
         0.2098403e+01, 0.2083777e+01, 0.2825850e-01, 0.3407567e-01, &
         0.4489003e-01, 0.6141419e-01, 0.8881936e-01, 0.1381295e+00, &
         0.2320493e+00, 0.4095595e+00, 0.7030742e+00, 0.1092780e+01, &
         0.1622257e+01, 0.2083007e+01, 0.2637337e+01, 0.3099045e+01, &
         0.3466475e+01, 0.3576084e+01, 0.3138925e+01, 0.2289058e+01, &
         0.2024652e+01, 0.2619606e+01, 0.2271519e+01, 0.2340897e+01, &
         0.2215942e+01, 0.2211942e+01, 0.2123458e+01, 0.2147658e+01, &
         0.2145206e+01, 0.2089916e+01, 0.2105316e+01, 0.2094704e+01/),&
         (/30,1,6/))

    rdqext(1:30,1:1,13:18)=reshape( (/&
         0.2595173e-01, 0.3114211e-01, 0.4062882e-01, 0.5475407e-01, &
         0.7750003e-01, 0.1173315e+00, 0.1920643e+00, 0.3352450e+00, &
         0.5877376e+00, 0.9459963e+00, 0.1422653e+01, 0.1940509e+01, &
         0.2447865e+01, 0.2935348e+01, 0.3393993e+01, 0.3634520e+01, &
         0.3377689e+01, 0.2585614e+01, 0.1928602e+01, 0.2453908e+01, &
         0.2537595e+01, 0.2120584e+01, 0.2355100e+01, 0.2236805e+01, &
         0.2206528e+01, 0.2137048e+01, 0.2110582e+01, 0.2129888e+01, &
         0.2112728e+01, 0.2111520e+01, 0.2456848e-01, 0.2940105e-01, &
         0.3814531e-01, 0.5096052e-01, 0.7121034e-01, 0.1060126e+00, &
         0.1704660e+00, 0.2941915e+00, 0.5193896e+00, 0.8569373e+00, &
         0.1297529e+01, 0.1841395e+01, 0.2305595e+01, 0.2818348e+01, &
         0.3263292e+01, 0.3597307e+01, 0.3494461e+01, 0.2745187e+01, &
         0.2013707e+01, 0.2287942e+01, 0.2655969e+01, 0.2131976e+01, &
         0.2384275e+01, 0.2205176e+01, 0.2206340e+01, 0.2215608e+01, &
         0.2134920e+01, 0.2115479e+01, 0.2095288e+01, 0.2098952e+01, &
         0.2235298e-01, 0.2663950e-01, 0.3427472e-01, 0.4518270e-01, &
         0.6187829e-01, 0.8962012e-01, 0.1396183e+00, 0.2349173e+00, &
         0.4147888e+00, 0.7108101e+00, 0.1102706e+01, 0.1634750e+01, &
         0.2092761e+01, 0.2647524e+01, 0.3107261e+01, 0.3467156e+01, &
         0.3579113e+01, 0.3125170e+01, 0.2247379e+01, 0.2026859e+01, &
         0.2657282e+01, 0.2265156e+01, 0.2332558e+01, 0.2186288e+01, &
         0.2227778e+01, 0.2146957e+01, 0.2169617e+01, 0.2140218e+01, &
         0.2092722e+01, 0.2088475e+01, 0.1921333e-01, 0.2277928e-01, &
         0.2899945e-01, 0.3757729e-01, 0.5010217e-01, 0.6980440e-01, &
         0.1035102e+00, 0.1657181e+00, 0.2851043e+00, 0.5038010e+00, &
         0.8360577e+00, 0.1268737e+01, 0.1815424e+01, 0.2272147e+01, &
         0.2793396e+01, 0.3235432e+01, 0.3570348e+01, 0.3524326e+01, &
         0.2809205e+01, 0.2151345e+01, 0.2281744e+01, 0.2666198e+01, &
         0.2126539e+01, 0.2388358e+01, 0.2219515e+01, 0.2215051e+01, &
         0.2223578e+01, 0.2160036e+01, 0.2126196e+01, 0.2106740e+01, &
         0.1828648e-01, 0.2165078e-01, 0.2748548e-01, 0.3545161e-01, &
         0.4692184e-01, 0.6465408e-01, 0.9444001e-01, 0.1486191e+00, &
         0.2522533e+00, 0.4460807e+00, 0.7561581e+00, 0.1161547e+01, &
         0.1705030e+01, 0.2152755e+01, 0.2702824e+01, 0.3151649e+01, &
         0.3478999e+01, 0.3582580e+01, 0.3043263e+01, 0.2252439e+01, &
         0.2120131e+01, 0.2719469e+01, 0.2260474e+01, 0.2383116e+01, &
         0.2185176e+01, 0.2205244e+01, 0.2167593e+01, 0.2190734e+01, &
         0.2141221e+01, 0.2117435e+01, 0.1705720e-01, 0.2016090e-01, &
         0.2550504e-01, 0.3270831e-01, 0.4289185e-01, 0.5826783e-01, &
         0.8343055e-01, 0.1281675e+00, 0.2128689e+00, 0.3742239e+00, &
         0.6495301e+00, 0.1024606e+01, 0.1532292e+01, 0.2017566e+01, &
         0.2558653e+01, 0.3033171e+01, 0.3455450e+01, 0.3578496e+01, &
         0.3254357e+01, 0.2367896e+01, 0.1937363e+01, 0.2580973e+01, &
         0.2382489e+01, 0.2224710e+01, 0.2240751e+01, 0.2242849e+01, &
         0.2187788e+01, 0.2117765e+01, 0.2155853e+01, 0.2127444e+01/),&
         (/30,1,6/))

    rdqext(1:30,1:1,19:24)=reshape( (/&
         0.1661168e-01, 0.1962307e-01, 0.2479483e-01, 0.3173472e-01, &
         0.4148188e-01, 0.5607228e-01, 0.7971362e-01, 0.1213590e+00, &
         0.1997862e+00, 0.3497850e+00, 0.6111342e+00, 0.9758610e+00, &
         0.1464721e+01, 0.1970491e+01, 0.2492324e+01, 0.2974746e+01, &
         0.3425845e+01, 0.3616955e+01, 0.3324818e+01, 0.2534858e+01, &
         0.2057498e+01, 0.2525669e+01, 0.2476210e+01, 0.2140753e+01, &
         0.2303478e+01, 0.2236078e+01, 0.2172018e+01, 0.2146353e+01, &
         0.2130648e+01, 0.2142246e+01, 0.1598668e-01, 0.1886983e-01, &
         0.2380455e-01, 0.3038575e-01, 0.3954578e-01, 0.5309152e-01, &
         0.7472860e-01, 0.1123202e+00, 0.1824803e+00, 0.3170920e+00, &
         0.5579348e+00, 0.9075783e+00, 0.1368394e+01, 0.1899847e+01, &
         0.2387506e+01, 0.2883463e+01, 0.3339896e+01, 0.3636674e+01, &
         0.3422941e+01, 0.2665677e+01, 0.2101645e+01, 0.2395268e+01, &
         0.2593362e+01, 0.2087997e+01, 0.2348550e+01, 0.2197394e+01, &
         0.2205405e+01, 0.2186869e+01, 0.2118561e+01, 0.2120739e+01, &
         0.1447862e-01, 0.1705971e-01, 0.2144386e-01, 0.2720915e-01, &
         0.3506626e-01, 0.4635068e-01, 0.6373934e-01, 0.9284600e-01, &
         0.1456350e+00, 0.2465072e+00, 0.4357705e+00, 0.7413899e+00, &
         0.1142247e+01, 0.1682719e+01, 0.2132625e+01, 0.2685511e+01, &
         0.3137660e+01, 0.3472792e+01, 0.3583071e+01, 0.3077484e+01, &
         0.2215237e+01, 0.2114642e+01, 0.2624185e+01, 0.2220664e+01, &
         0.2292433e+01, 0.2218276e+01, 0.2214263e+01, 0.2160881e+01, &
         0.2179611e+01, 0.2127588e+01, 0.1328048e-01, 0.1562869e-01, &
         0.1959484e-01, 0.2475764e-01, 0.3168387e-01, 0.4140854e-01, &
         0.5595865e-01, 0.7952227e-01, 0.1210101e+00, 0.1991165e+00, &
         0.3485271e+00, 0.6091268e+00, 0.9733056e+00, 0.1461134e+01, &
         0.1967971e+01, 0.2488624e+01, 0.2971455e+01, 0.3423538e+01, &
         0.3618834e+01, 0.3328263e+01, 0.2539237e+01, 0.2039369e+01, &
         0.2503954e+01, 0.2453909e+01, 0.2115271e+01, 0.2290610e+01, &
         0.2230195e+01, 0.2159497e+01, 0.2120082e+01, 0.2116015e+01, &
         0.1200182e-01, 0.1410724e-01, 0.1764445e-01, 0.2220361e-01, &
         0.2822565e-01, 0.3648769e-01, 0.4846574e-01, 0.6714261e-01, &
         0.9880274e-01, 0.1568201e+00, 0.2680290e+00, 0.4740632e+00, &
         0.7954467e+00, 0.1213648e+01, 0.1761516e+01, 0.2209371e+01, &
         0.2747197e+01, 0.3189625e+01, 0.3513435e+01, 0.3566643e+01, &
         0.2920861e+01, 0.2063354e+01, 0.2168825e+01, 0.2651909e+01, &
         0.2093758e+01, 0.2346431e+01, 0.2182174e+01, 0.2229954e+01, &
         0.2199746e+01, 0.2167265e+01, 0.1181864e-01, 0.1388969e-01, &
         0.1736677e-01, 0.2184242e-01, 0.2774175e-01, 0.3580971e-01, &
         0.4745414e-01, 0.6550952e-01, 0.9593535e-01, 0.1514246e+00, &
         0.2576532e+00, 0.4557129e+00, 0.7698092e+00, 0.1179520e+01, &
         0.1725135e+01, 0.2171919e+01, 0.2718426e+01, 0.3164545e+01, &
         0.3487779e+01, 0.3580781e+01, 0.2997886e+01, 0.2219576e+01, &
         0.2137670e+01, 0.2651932e+01, 0.2223345e+01, 0.2425306e+01, &
         0.2227556e+01, 0.2204142e+01, 0.2194215e+01, 0.2185364e+01/),&
         (/30,1,6/))

    rdqext(1:30,1:1,25:30)=reshape( (/&
         0.1146874e-01, 0.1347448e-01, 0.1683753e-01, 0.2115573e-01, &
         0.2682521e-01, 0.3453235e-01, 0.4556208e-01, 0.6248134e-01, &
         0.9066294e-01, 0.1415598e+00, 0.2386575e+00, 0.4215866e+00, &
         0.7207971e+00, 0.1115563e+01, 0.1650671e+01, 0.2105534e+01, &
         0.2660306e+01, 0.3117507e+01, 0.3468289e+01, 0.3581810e+01, &
         0.3107697e+01, 0.2204360e+01, 0.2083869e+01, 0.2692604e+01, &
         0.2246163e+01, 0.2310285e+01, 0.2205017e+01, 0.2212293e+01, &
         0.2154623e+01, 0.2157726e+01, 0.1036541e-01, 0.1216810e-01, &
         0.1517874e-01, 0.1901641e-01, 0.2399688e-01, 0.3064706e-01, &
         0.3991923e-01, 0.5366334e-01, 0.7567919e-01, 0.1140351e+00, &
         0.1857564e+00, 0.3233098e+00, 0.5682176e+00, 0.9208928e+00, &
         0.1387183e+01, 0.1914258e+01, 0.2408734e+01, 0.2901393e+01, &
         0.3359867e+01, 0.3639189e+01, 0.3413174e+01, 0.2648792e+01, &
         0.2071824e+01, 0.2370969e+01, 0.2552859e+01, 0.2186358e+01, &
         0.2357531e+01, 0.2219193e+01, 0.2180621e+01, 0.2176074e+01, &
         0.1099698e-01, 0.1291546e-01, 0.1612651e-01, 0.2023638e-01, &
         0.2560486e-01, 0.3284562e-01, 0.4309161e-01, 0.5858051e-01, &
         0.8396278e-01, 0.1291468e+00, 0.2147529e+00, 0.3777217e+00, &
         0.6549321e+00, 0.1031459e+01, 0.1541618e+01, 0.2024099e+01, &
         0.2567300e+01, 0.3040632e+01, 0.3457704e+01, 0.3574554e+01, &
         0.3238655e+01, 0.2353804e+01, 0.1952011e+01, 0.2580399e+01, &
         0.2390371e+01, 0.2200474e+01, 0.2244411e+01, 0.2220684e+01, &
         0.2163651e+01, 0.2148918e+01, 0.9705304e-02, 0.1138785e-01, &
         0.1419229e-01, 0.1775317e-01, 0.2234502e-01, 0.2841550e-01, &
         0.3675437e-01, 0.4886510e-01, 0.6778988e-01, 0.9994365e-01, &
         0.1589722e+00, 0.2721637e+00, 0.4813170e+00, 0.8054546e+00, &
         0.1227107e+01, 0.1775224e+01, 0.2224483e+01, 0.2758411e+01, &
         0.3200041e+01, 0.3526455e+01, 0.3557312e+01, 0.2899145e+01, &
         0.2049803e+01, 0.2180376e+01, 0.2742225e+01, 0.2126863e+01, &
         0.2353465e+01, 0.2187709e+01, 0.2224508e+01, 0.2222815e+01, &
         0.9146382e-02, 0.1072809e-01, 0.1336029e-01, 0.1669216e-01, &
         0.2096744e-01, 0.2657457e-01, 0.3418472e-01, 0.4505027e-01, &
         0.6166824e-01, 0.8925748e-01, 0.1389438e+00, 0.2336179e+00, &
         0.4124213e+00, 0.7073137e+00, 0.1098216e+01, 0.1629121e+01, &
         0.2088338e+01, 0.2642952e+01, 0.3103581e+01, 0.3466837e+01, &
         0.3577819e+01, 0.3131474e+01, 0.2266839e+01, 0.2022272e+01, &
         0.2655272e+01, 0.2286961e+01, 0.2383289e+01, 0.2214466e+01, &
         0.2205394e+01, 0.2140238e+01, 0.8506889e-02, 0.9974504e-02, &
         0.1241214e-01, 0.1548789e-01, 0.1941376e-01, 0.2451917e-01, &
         0.3135827e-01, 0.4093947e-01, 0.5523325e-01, 0.7830309e-01, &
         0.1187902e+00, 0.1948592e+00, 0.3405170e+00, 0.5962692e+00, &
         0.9569100e+00, 0.1438057e+01, 0.1951612e+01, 0.2464408e+01, &
         0.2949951e+01, 0.3406850e+01, 0.3629331e+01, 0.3356170e+01, &
         0.2565260e+01, 0.1935007e+01, 0.2454862e+01, 0.2496692e+01, &
         0.2086216e+01, 0.2314208e+01, 0.2219513e+01, 0.2162504e+01/),&
         (/30,1,6/))

    rdqext(1:30,1:1,31:36)=reshape( (/&
         0.7831769e-02, 0.9179345e-02, 0.1141457e-01, 0.1422596e-01, &
         0.1779609e-01, 0.2240106e-01, 0.2849078e-01, 0.3686021e-01, &
         0.4902377e-01, 0.6804754e-01, 0.1003985e+00, 0.1598312e+00, &
         0.2738129e+00, 0.4842010e+00, 0.8094147e+00, 0.1232453e+01, &
         0.1780571e+01, 0.2230534e+01, 0.2762867e+01, 0.3204298e+01, &
         0.3531933e+01, 0.3553329e+01, 0.2890255e+01, 0.2053158e+01, &
         0.2194848e+01, 0.2679376e+01, 0.2134710e+01, 0.2353978e+01, &
         0.2197762e+01, 0.2211876e+01, 0.7078757e-02, 0.8293798e-02, &
         0.1030588e-01, 0.1282875e-01, 0.1601650e-01, 0.2009444e-01, &
         0.2541709e-01, 0.3258747e-01, 0.4271624e-01, 0.5799330e-01, &
         0.8296370e-01, 0.1273095e+00, 0.2112185e+00, 0.3711553e+00, &
         0.6447718e+00, 0.1018571e+01, 0.1524037e+01, 0.2011806e+01, &
         0.2550897e+01, 0.3026439e+01, 0.3453130e+01, 0.3582561e+01, &
         0.3267390e+01, 0.2385726e+01, 0.1939468e+01, 0.2592190e+01, &
         0.2432119e+01, 0.2266550e+01, 0.2245551e+01, 0.2216399e+01, &
         0.5893333e-02, 0.6901805e-02, 0.8567906e-02, 0.1064833e-01, &
         0.1325984e-01, 0.1656435e-01, 0.2080209e-01, 0.2635475e-01, &
         0.3388031e-01, 0.4460318e-01, 0.6096004e-01, 0.8803720e-01, &
         0.1366774e+00, 0.2292520e+00, 0.4044449e+00, 0.6954624e+00, &
         0.1083038e+01, 0.1609830e+01, 0.2073517e+01, 0.2627049e+01, &
         0.3090688e+01, 0.3465830e+01, 0.3572824e+01, 0.3151445e+01, &
         0.2316983e+01, 0.2038591e+01, 0.2610828e+01, 0.2285346e+01, &
         0.2321426e+01, 0.2202826e+01, 0.5025937e-02, 0.5884357e-02, &
         0.7300625e-02, 0.9065186e-02, 0.1127127e-01, 0.1404497e-01, &
         0.1756500e-01, 0.2210026e-01, 0.2808706e-01, 0.3629318e-01, &
         0.4817502e-01, 0.6667232e-01, 0.9797529e-01, 0.1552611e+00, &
         0.2650325e+00, 0.4687851e+00, 0.7881204e+00, 0.1203845e+01, &
         0.1751305e+01, 0.2198479e+01, 0.2739005e+01, 0.3182265e+01, &
         0.3504886e+01, 0.3572367e+01, 0.2939144e+01, 0.2093931e+01, &
         0.2184590e+01, 0.2653599e+01, 0.2120970e+01, 0.2362548e+01, &
         0.4433422e-02, 0.5189727e-02, 0.6437071e-02, 0.7988810e-02, &
         0.9924201e-02, 0.1234918e-01, 0.1540817e-01, 0.1931125e-01, &
         0.2438432e-01, 0.3117430e-01, 0.4067514e-01, 0.5482540e-01, &
         0.7761944e-01, 0.1175483e+00, 0.1924794e+00, 0.3360287e+00, &
         0.5890094e+00, 0.9476252e+00, 0.1424953e+01, 0.1942187e+01, &
         0.2450353e+01, 0.2937539e+01, 0.3395996e+01, 0.3633851e+01, &
         0.3374526e+01, 0.2582175e+01, 0.1926354e+01, 0.2447965e+01, &
         0.2536666e+01, 0.2114620e+01, 0.3825215e-02, 0.4476689e-02, &
         0.5550712e-02, 0.6885700e-02, 0.8548012e-02, 0.1062361e-01, &
         0.1322856e-01, 0.1652461e-01, 0.2075062e-01, 0.2628650e-01, &
         0.3378580e-01, 0.4446473e-01, 0.6074107e-01, 0.8766055e-01, &
         0.1359789e+00, 0.2279065e+00, 0.4019802e+00, 0.6917781e+00, &
         0.1078330e+01, 0.1603768e+01, 0.2068956e+01, 0.2621971e+01, &
         0.3086536e+01, 0.3465486e+01, 0.3571346e+01, 0.3157148e+01, &
         0.2325003e+01, 0.2047926e+01, 0.2618621e+01, 0.2334253e+01/),&
         (/30,1,6/))

    rdqext(1:30,1:1,37:42)=reshape( (/&
         0.3549440e-02, 0.4153976e-02, 0.5149985e-02, 0.6387233e-02, &
         0.7926833e-02, 0.9847006e-02, 0.1225196e-01, 0.1528491e-01, &
         0.1915287e-01, 0.2417619e-01, 0.3089087e-01, 0.4026837e-01, &
         0.5419936e-01, 0.7657288e-01, 0.1156511e+00, 0.1888472e+00, &
         0.3291641e+00, 0.5778280e+00, 0.9332777e+00, 0.1404682e+01, &
         0.1927353e+01, 0.2428208e+01, 0.2918166e+01, 0.3377411e+01, &
         0.3638303e+01, 0.3399381e+01, 0.2617698e+01, 0.1981718e+01, &
         0.2419561e+01, 0.2593332e+01, 0.3034851e-02, 0.3551537e-02, &
         0.4402093e-02, 0.5458310e-02, 0.6770817e-02, 0.8404701e-02, &
         0.1044440e-01, 0.1300307e-01, 0.1623797e-01, 0.2038019e-01, &
         0.2579521e-01, 0.3310764e-01, 0.4347322e-01, 0.5917915e-01, &
         0.8498399e-01, 0.1310288e+00, 0.2183748e+00, 0.3844299e+00, &
         0.6652285e+00, 0.1044529e+01, 0.1559250e+01, 0.2036551e+01, &
         0.2583322e+01, 0.3054318e+01, 0.3461036e+01, 0.3569443e+01, &
         0.3209359e+01, 0.2341575e+01, 0.2014180e+01, 0.2588365e+01, &
         0.2387623e-02, 0.2794191e-02, 0.3462401e-02, 0.4291734e-02, &
         0.5320973e-02, 0.6599757e-02, 0.8191557e-02, 0.1017778e-01, &
         0.1266782e-01, 0.1581217e-01, 0.1983106e-01, 0.2506921e-01, &
         0.3211028e-01, 0.4202449e-01, 0.5691476e-01, 0.8113549e-01, &
         0.1239570e+00, 0.2047747e+00, 0.3591337e+00, 0.6259582e+00, &
         0.9947014e+00, 0.1491055e+01, 0.1988876e+01, 0.2518942e+01, &
         0.2998372e+01, 0.3440423e+01, 0.3601781e+01, 0.3304095e+01, &
         0.2485376e+01, 0.2048215e+01, 0.2057811e-02, 0.2408417e-02, &
         0.2984623e-02, 0.3698977e-02, 0.4585142e-02, 0.5685347e-02, &
         0.7053476e-02, 0.8756823e-02, 0.1088493e-01, 0.1355791e-01, &
         0.1694377e-01, 0.2129343e-01, 0.2700860e-01, 0.3478724e-01, &
         0.4593816e-01, 0.6308043e-01, 0.9170132e-01, 0.1434965e+00, &
         0.2423881e+00, 0.4283417e+00, 0.7306451e+00, 0.1128293e+01, &
         0.1666130e+01, 0.2118356e+01, 0.2672537e+01, 0.3127278e+01, &
         0.3469966e+01, 0.3582943e+01, 0.3093040e+01, 0.2194497e+01, &
         0.1826921e-02, 0.2137052e-02, 0.2648383e-02, 0.3281895e-02, &
         0.4067792e-02, 0.5043134e-02, 0.6254602e-02, 0.7761572e-02, &
         0.9640561e-02, 0.1199300e-01, 0.1495711e-01, 0.1873215e-01, &
         0.2362413e-01, 0.3014096e-01, 0.3919665e-01, 0.5255830e-01, &
         0.7384456e-01, 0.1107295e+00, 0.1794446e+00, 0.3113199e+00, &
         0.5483193e+00, 0.8950609e+00, 0.1350772e+01, 0.1885957e+01, &
         0.2367349e+01, 0.2866825e+01, 0.3320561e+01, 0.3631001e+01, &
         0.3433996e+01, 0.2677806e+01, 0.1663156e-02, 0.1945878e-02, &
         0.2411773e-02, 0.2989146e-02, 0.3704706e-02, 0.4592277e-02, &
         0.5694410e-02, 0.7064310e-02, 0.8770572e-02, 0.1090218e-01, &
         0.1357968e-01, 0.1697154e-01, 0.2132941e-01, 0.2705653e-01, &
         0.3485396e-01, 0.4603667e-01, 0.6323758e-01, 0.9197413e-01, &
         0.1440057e+00, 0.2433691e+00, 0.4301138e+00, 0.7332162e+00, &
         0.1131626e+01, 0.1670127e+01, 0.2121743e+01, 0.2675674e+01, &
         0.3129785e+01, 0.3470570e+01, 0.3583046e+01, 0.3089532e+01/),&
         (/30,1,6/))

    rdqext(1:30,1:1,43:48)=reshape( (/&
         0.1567061e-02, 0.1833278e-02, 0.2271720e-02, 0.2815047e-02, &
         0.3488794e-02, 0.4324771e-02, 0.5361873e-02, 0.6650588e-02, &
         0.8254934e-02, 0.1025699e-01, 0.1276743e-01, 0.1593864e-01, &
         0.1999407e-01, 0.2528441e-01, 0.3240535e-01, 0.4245194e-01, &
         0.5758059e-01, 0.8226308e-01, 0.1260231e+00, 0.2087452e+00, &
         0.3665486e+00, 0.6375951e+00, 0.1009468e+01, 0.1511518e+01, &
         0.2003096e+01, 0.2538950e+01, 0.3016001e+01, 0.3448988e+01, &
         0.3589492e+01, 0.3284389e+01, 0.1488439e-02, 0.1741797e-02, &
         0.2157624e-02, 0.2673772e-02, 0.3314134e-02, 0.4107578e-02, &
         0.5092440e-02, 0.6315934e-02, 0.7837981e-02, 0.9735893e-02, &
         0.1211273e-01, 0.1510853e-01, 0.1892637e-01, 0.2387865e-01, &
         0.3048642e-01, 0.3968957e-01, 0.5331150e-01, 0.7509397e-01, &
         0.1129788e+00, 0.1837380e+00, 0.3194806e+00, 0.5618942e+00, &
         0.9127132e+00, 0.1375636e+01, 0.1905448e+01, 0.2395723e+01, &
         0.2890356e+01, 0.3347702e+01, 0.3638081e+01, 0.3419334e+01, &
         0.1409372e-02, 0.1649454e-02, 0.2044782e-02, 0.2532639e-02, &
         0.3138920e-02, 0.3890525e-02, 0.4823158e-02, 0.5981331e-02, &
         0.7421338e-02, 0.9215627e-02, 0.1146004e-01, 0.1428342e-01, &
         0.1786957e-01, 0.2249676e-01, 0.2861935e-01, 0.3704118e-01, &
         0.4929540e-01, 0.6848914e-01, 0.1011789e+00, 0.1613057e+00, &
         0.2766435e+00, 0.4891382e+00, 0.8161697e+00, 0.1241602e+01, &
         0.1789592e+01, 0.2240942e+01, 0.2770504e+01, 0.3211761e+01, &
         0.3541587e+01, 0.3546342e+01, 0.1330705e-02, 0.1558003e-02, &
         0.1930830e-02, 0.2391852e-02, 0.2964355e-02, 0.3673612e-02, &
         0.4553958e-02, 0.5646588e-02, 0.7004765e-02, 0.8696110e-02, &
         0.1080916e-01, 0.1346239e-01, 0.1682217e-01, 0.2113581e-01, &
         0.2679852e-01, 0.3449527e-01, 0.4550745e-01, 0.6239447e-01, &
         0.9051254e-01, 0.1412796e+00, 0.2381176e+00, 0.4206070e+00, &
         0.7193625e+00, 0.1113713e+01, 0.1648399e+01, 0.2103685e+01, &
         0.2658494e+01, 0.3116058e+01, 0.3468100e+01, 0.3581523e+01, &
         0.1251933e-02, 0.1466046e-02, 0.1816172e-02, 0.2251175e-02, &
         0.2788802e-02, 0.3456600e-02, 0.4284468e-02, 0.5312049e-02, &
         0.6588972e-02, 0.8177907e-02, 0.1016081e-01, 0.1264632e-01, &
         0.1578495e-01, 0.1979593e-01, 0.2502291e-01, 0.3204691e-01, &
         0.4193268e-01, 0.5677203e-01, 0.8089415e-01, 0.1235155e+00, &
         0.2039266e+00, 0.3575470e+00, 0.6234539e+00, 0.9915216e+00, &
         0.1486626e+01, 0.1985795e+01, 0.2514534e+01, 0.2994468e+01, &
         0.3438259e+01, 0.3604452e+01, 0.1174132e-02, 0.1373839e-02, &
         0.1703073e-02, 0.2110218e-02, 0.2614515e-02, 0.3239858e-02, &
         0.4015790e-02, 0.4978259e-02, 0.6173875e-02, 0.7660876e-02, &
         0.9514965e-02, 0.1183530e-01, 0.1475762e-01, 0.1847636e-01, &
         0.2328914e-01, 0.2968762e-01, 0.3855173e-01, 0.5157678e-01, &
         0.7222367e-01, 0.1078229e+00, 0.1739076e+00, 0.3007662e+00, &
         0.5305643e+00, 0.8717529e+00, 0.1318129e+01, 0.1859136e+01, &
         0.2329573e+01, 0.2836743e+01, 0.3284825e+01, 0.3613000e+01/),&
         (/30,1,6/))

    rdqext(1:30,1:1,49:50)=reshape( (/&
         0.1095844e-02, 0.1281458e-02, 0.1588486e-02, 0.1968793e-02, &
         0.2439237e-02, 0.3023065e-02, 0.3747140e-02, 0.4644954e-02, &
         0.5759797e-02, 0.7145516e-02, 0.8871601e-02, 0.1102877e-01, &
         0.1373922e-01, 0.1717497e-01, 0.2159321e-01, 0.2740864e-01, &
         0.3534436e-01, 0.4676276e-01, 0.6439901e-01, 0.9399497e-01, &
         0.1477852e+00, 0.2506478e+00, 0.4432062e+00, 0.7520572e+00, &
         0.1156173e+01, 0.1698893e+01, 0.2147104e+01, 0.2698069e+01, &
         0.3147782e+01, 0.3476962e+01, 0.9974022e-03, 0.1165836e-02, &
         0.1443403e-02, 0.1788278e-02, 0.2216780e-02, 0.2746567e-02, &
         0.3404155e-02, 0.4219844e-02, 0.5231818e-02, 0.6489299e-02, &
         0.8053846e-02, 0.1000574e-01, 0.1245149e-01, 0.1553765e-01, &
         0.1947781e-01, 0.2460348e-01, 0.3147328e-01, 0.4110512e-01, &
         0.5548911e-01, 0.7873266e-01, 0.1195716e+00, 0.1963573e+00, &
         0.3433384e+00, 0.6008126e+00, 0.9627100e+00, 0.1446232e+01, &
         0.1957438e+01, 0.2473066e+01, 0.2957629e+01, 0.3413126e+01/),&
         (/30,1,2/))

    qscat(1:30,1:1,1:6)=reshape( (/&
         0.1535112e-05, 0.8998886e-09, 0.1729664e-08, 0.4070829e-08, &
         0.9589681e-08, 0.2261375e-07, 0.5328723e-07, 0.1255802e-06, &
         0.2960315e-06, 0.6976022e-06, 0.1644110e-05, 0.3875383e-05, &
         0.9135672e-05, 0.2154004e-04, 0.5080069e-04, 0.1198578e-03, &
         0.2829565e-03, 0.6685623e-03, 0.1581542e-02, 0.3746952e-02, &
         0.8891535e-02, 0.2111296e-01, 0.4993847e-01, 0.1159231e+00, &
         0.2536170e+00, 0.4850497e+00, 0.7988749e+00, 0.1241824e+01, &
         0.1650353e+01, 0.2180239e+01, 0.2919784e-05, 0.5829020e-09, &
         0.1120681e-08, 0.2640323e-08, 0.6234001e-08, 0.1468913e-07, &
         0.3459422e-07, 0.8157038e-07, 0.1922175e-06, 0.4529455e-06, &
         0.1067609e-05, 0.2516517e-05, 0.5931983e-05, 0.1398515e-04, &
         0.3297836e-04, 0.7779063e-04, 0.1835863e-03, 0.4335718e-03, &
         0.1024991e-02, 0.2426435e-02, 0.5753419e-02, 0.1366061e-01, &
         0.3240611e-01, 0.7616633e-01, 0.1729649e+00, 0.3576158e+00, &
         0.6282287e+00, 0.1009327e+01, 0.1438690e+01, 0.1939538e+01, &
         0.5553430e-05, 0.3008072e-09, 0.5785822e-09, 0.1359781e-08, &
         0.3207448e-08, 0.7555678e-08, 0.1780665e-07, 0.4196275e-07, &
         0.9891541e-07, 0.2330545e-06, 0.5492852e-06, 0.1294664e-05, &
         0.3051624e-05, 0.7193325e-05, 0.1696003e-04, 0.3999544e-04, &
         0.9435236e-04, 0.2227023e-03, 0.5260573e-03, 0.1243972e-02, &
         0.2945860e-02, 0.6987582e-02, 0.1659290e-01, 0.3932323e-01, &
         0.9199918e-01, 0.2059242e+00, 0.4121182e+00, 0.7000768e+00, &
         0.1114519e+01, 0.1523778e+01, 0.1056262e-04, 0.1667335e-09, &
         0.3215872e-09, 0.7584442e-09, 0.1788107e-08, 0.4214065e-08, &
         0.9933288e-08, 0.2340783e-07, 0.5519188e-07, 0.1300859e-06, &
         0.3066088e-06, 0.7225132e-06, 0.1702958e-05, 0.4013857e-05, &
         0.9462706e-05, 0.2231118e-04, 0.5262020e-04, 0.1241540e-03, &
         0.2931054e-03, 0.6925743e-03, 0.1638427e-02, 0.3881993e-02, &
         0.9212516e-02, 0.2187444e-01, 0.5172065e-01, 0.1198937e+00, &
         0.2612906e+00, 0.4962334e+00, 0.8145505e+00, 0.1259660e+01, &
         0.2009011e-04, 0.9312614e-10, 0.1788817e-09, 0.4216057e-09, &
         0.9970077e-09, 0.2345042e-08, 0.5529286e-08, 0.1303760e-07, &
         0.3071025e-07, 0.7236788e-07, 0.1705414e-06, 0.4019309e-06, &
         0.9472516e-06, 0.2232579e-05, 0.5262621e-05, 0.1240705e-04, &
         0.2925507e-04, 0.6900538e-04, 0.1628382e-03, 0.3845306e-03, &
         0.9089075e-03, 0.2151185e-02, 0.5099573e-02, 0.1210664e-01, &
         0.2873163e-01, 0.6768199e-01, 0.1548432e+00, 0.3259844e+00, &
         0.5861368e+00, 0.9459786e+00, 0.3821139e-04, 0.4500313e-10, &
         0.8613462e-10, 0.2036637e-09, 0.4809451e-09, 0.1131263e-08, &
         0.2664907e-08, 0.6288710e-08, 0.1480967e-07, 0.3488467e-07, &
         0.8223534e-07, 0.1937976e-06, 0.4568121e-06, 0.1076551e-05, &
         0.2537450e-05, 0.5981515e-05, 0.1410117e-04, 0.3325193e-04, &
         0.7843683e-04, 0.1851134e-03, 0.4371810e-03, 0.1033536e-02, &
         0.2446699e-02, 0.5801552e-02, 0.1377500e-01, 0.3267637e-01, &
         0.7678847e-01, 0.1742809e+00, 0.3598650e+00, 0.6311989e+00/),&
         (/30,1,6/))

    qscat(1:30,1:1,7:12)=reshape( (/&
         0.7267806e-04, 0.1765787e-10, 0.3378414e-10, 0.7984936e-10, &
         0.1884185e-09, 0.4455544e-09, 0.1046103e-08, 0.2469826e-08, &
         0.5812942e-08, 0.1370527e-07, 0.3230256e-07, 0.7611979e-07, &
         0.1794039e-06, 0.4228477e-06, 0.9965112e-06, 0.2348792e-05, &
         0.5536366e-05, 0.1305259e-04, 0.3077852e-04, 0.7260025e-04, &
         0.1713272e-03, 0.4045929e-03, 0.9563949e-03, 0.2263770e-02, &
         0.5366995e-02, 0.1274226e-01, 0.3023528e-01, 0.7115984e-01, &
         0.1623103e+00, 0.3391695e+00, 0.1382336e-03, 0.5461978e-11, &
         0.1045634e-10, 0.2480865e-10, 0.5877250e-10, 0.1386328e-09, &
         0.3260322e-09, 0.7689554e-09, 0.1809900e-08, 0.4257048e-08, &
         0.1004131e-07, 0.2366112e-07, 0.5576113e-07, 0.1314203e-06, &
         0.3096752e-06, 0.7299032e-06, 0.1720274e-05, 0.4054981e-05, &
         0.9559084e-05, 0.2253919e-04, 0.5315715e-04, 0.1254218e-03, &
         0.2961037e-03, 0.6996637e-03, 0.1655233e-02, 0.3921884e-02, &
         0.9307324e-02, 0.2209932e-01, 0.5224660e-01, 0.1210630e+00, &
         0.2629204e-03, 0.8523221e-12, 0.1655710e-11, 0.3974937e-11, &
         0.9475196e-11, 0.2160493e-10, 0.5173064e-10, 0.1217817e-09, &
         0.2857532e-09, 0.6785524e-09, 0.1597406e-08, 0.3767277e-08, &
         0.8871726e-08, 0.2090630e-07, 0.4927249e-07, 0.1161264e-06, &
         0.2736318e-06, 0.6449878e-06, 0.1520047e-05, 0.3583024e-05, &
         0.8446612e-05, 0.1991419e-04, 0.4696518e-04, 0.1108043e-03, &
         0.2615690e-03, 0.6179706e-03, 0.1461678e-02, 0.3462458e-02, &
         0.8215324e-02, 0.1950821e-01, 0.5000746e-03, 0.2305856e-02, &
         0.5466960e-02, 0.1297981e-01, 0.3079700e-01, 0.7245710e-01, &
         0.1650817e+00, 0.3440087e+00, 0.6102081e+00, 0.9822114e+00, &
         0.1416896e+01, 0.1905245e+01, 0.2396360e+01, 0.2865409e+01, &
         0.3078600e+01, 0.2821023e+01, 0.2079308e+01, 0.1451826e+01, &
         0.2001947e+01, 0.2059541e+01, 0.1667592e+01, 0.1893693e+01, &
         0.1813272e+01, 0.1784861e+01, 0.1766699e+01, 0.1802252e+01, &
         0.1777354e+01, 0.1773361e+01, 0.1769136e+01, 0.1735761e+01, &
         0.8336438e-03, 0.1607949e-02, 0.3809640e-02, 0.9040533e-02, &
         0.2146645e-01, 0.5076601e-01, 0.1177685e+00, 0.2571919e+00, &
         0.4902766e+00, 0.8061812e+00, 0.1250220e+01, 0.1660402e+01, &
         0.2188359e+01, 0.2646668e+01, 0.2970078e+01, 0.3017053e+01, &
         0.2416580e+01, 0.1578704e+01, 0.1703342e+01, 0.2210760e+01, &
         0.1668151e+01, 0.1938131e+01, 0.1788607e+01, 0.1854164e+01, &
         0.1828766e+01, 0.1805977e+01, 0.1766851e+01, 0.1782201e+01, &
         0.1754907e+01, 0.1744322e+01, 0.6669299e-03, 0.1286015e-02, &
         0.3045615e-02, 0.7224638e-02, 0.1715594e-01, 0.4064883e-01, &
         0.9501187e-01, 0.2120690e+00, 0.4218818e+00, 0.7130325e+00, &
         0.1132582e+01, 0.1539426e+01, 0.2076132e+01, 0.2551370e+01, &
         0.2925028e+01, 0.3023993e+01, 0.2621071e+01, 0.1807222e+01, &
         0.1550507e+01, 0.2171909e+01, 0.1841270e+01, 0.1927457e+01, &
         0.1817936e+01, 0.1826131e+01, 0.1748519e+01, 0.1781892e+01, &
         0.1787518e+01, 0.1738967e+01, 0.1760810e+01, 0.1754529e+01/),&
         (/30,1,6/))

    qscat(1:30,1:1,13:18)=reshape( (/&
         0.4939059e-03, 0.9520435e-03, 0.2253458e-02, 0.5342496e-02, &
         0.1268401e-01, 0.3009751e-01, 0.7084160e-01, 0.1616292e+00, &
         0.3379757e+00, 0.6021811e+00, 0.9701180e+00, 0.1407054e+01, &
         0.1889471e+01, 0.2381627e+01, 0.2852786e+01, 0.3083985e+01, &
         0.2840933e+01, 0.2098689e+01, 0.1445787e+01, 0.2000638e+01, &
         0.2100897e+01, 0.1701578e+01, 0.1950433e+01, 0.1846692e+01, &
         0.1828247e+01, 0.1768301e+01, 0.1749795e+01, 0.1776199e+01, &
         0.1766105e+01, 0.1769362e+01, 0.4058536e-03, 0.7821483e-03, &
         0.1850704e-02, 0.4385977e-02, 0.1041047e-01, 0.2471486e-01, &
         0.5835165e-01, 0.1345477e+00, 0.2890299e+00, 0.5356299e+00, &
         0.8710567e+00, 0.1318777e+01, 0.1752060e+01, 0.2261234e+01, &
         0.2721528e+01, 0.3050559e+01, 0.2949300e+01, 0.2250151e+01, &
         0.1530140e+01, 0.1829173e+01, 0.2215410e+01, 0.1710420e+01, &
         0.1978602e+01, 0.1811628e+01, 0.1824998e+01, 0.1844743e+01, &
         0.1773051e+01, 0.1760529e+01, 0.1747295e+01, 0.1756304e+01, &
         0.2877247e-03, 0.5543064e-03, 0.1310881e-02, 0.3104613e-02, &
         0.7364837e-02, 0.1748890e-01, 0.4143228e-01, 0.9678915e-01, &
         0.2156751e+00, 0.4275572e+00, 0.7205915e+00, 0.1142951e+01, &
         0.1548665e+01, 0.2086365e+01, 0.2559992e+01, 0.2925613e+01, &
         0.3027108e+01, 0.2608634e+01, 0.1765504e+01, 0.1553489e+01, &
         0.2209895e+01, 0.1835341e+01, 0.1919490e+01, 0.1788675e+01, &
         0.1842055e+01, 0.1772597e+01, 0.1804116e+01, 0.1782525e+01, &
         0.1742764e+01, 0.1744599e+01, 0.1639157e-03, 0.3156405e-03, &
         0.7458854e-03, 0.1764762e-02, 0.4181915e-02, 0.9925411e-02, &
         0.2356508e-01, 0.5567071e-01, 0.1286458e+00, 0.2779670e+00, &
         0.5200978e+00, 0.8485439e+00, 0.1296149e+01, 0.1719943e+01, &
         0.2235500e+01, 0.2693254e+01, 0.3024626e+01, 0.2977588e+01, &
         0.2311464e+01, 0.1667730e+01, 0.1821186e+01, 0.2224497e+01, &
         0.1704216e+01, 0.1982319e+01, 0.1825384e+01, 0.1832962e+01, &
         0.1851820e+01, 0.1797334e+01, 0.1770782e+01, 0.1758385e+01, &
         0.1360369e-03, 0.2619239e-03, 0.6188128e-03, 0.1463670e-02, &
         0.3467186e-02, 0.8226552e-02, 0.1953486e-01, 0.4623934e-01, &
         0.1076391e+00, 0.2373873e+00, 0.4609284e+00, 0.7655962e+00, &
         0.1201772e+01, 0.1605729e+01, 0.2142393e+01, 0.2606626e+01, &
         0.2936520e+01, 0.3031624e+01, 0.2534039e+01, 0.1769609e+01, &
         0.1651198e+01, 0.2273975e+01, 0.1833097e+01, 0.1972575e+01, &
         0.1789315e+01, 0.1821071e+01, 0.1794554e+01, 0.1826451e+01, &
         0.1784653e+01, 0.1767444e+01, 0.1044451e-03, 0.2010619e-03, &
         0.4748902e-03, 0.1122807e-02, 0.2658433e-02, 0.6304602e-02, &
         0.1497037e-01, 0.3549854e-01, 0.8326766e-01, 0.1878861e+00, &
         0.3827521e+00, 0.6613459e+00, 0.1058644e+01, 0.1477897e+01, &
         0.1997881e+01, 0.2482627e+01, 0.2914294e+01, 0.3026468e+01, &
         0.2727181e+01, 0.1884788e+01, 0.1458210e+01, 0.2130828e+01, &
         0.1948996e+01, 0.1807852e+01, 0.1839294e+01, 0.1855008e+01, &
         0.1810482e+01, 0.1750667e+01, 0.1797086e+01, 0.1775596e+01/),&
         (/30,1,6/))

    qscat(1:30,1:1,19:24)=reshape( (/&
         0.9441071e-04, 0.1817370e-03, 0.4291996e-03, 0.1014639e-02, &
         0.2401892e-02, 0.5695102e-02, 0.1352202e-01, 0.3207864e-01, &
         0.7541227e-01, 0.1713672e+00, 0.3548766e+00, 0.6246089e+00, &
         0.1003889e+01, 0.1434349e+01, 0.1932783e+01, 0.2422200e+01, &
         0.2884667e+01, 0.3065633e+01, 0.2791466e+01, 0.2049569e+01, &
         0.1575594e+01, 0.2073899e+01, 0.2041062e+01, 0.1722158e+01, &
         0.1899726e+01, 0.1846775e+01, 0.1793869e+01, 0.1778048e+01, &
         0.1770816e+01, 0.1789440e+01, 0.8151573e-04, 0.1569007e-03, &
         0.3704934e-03, 0.8756850e-03, 0.2072421e-02, 0.4912498e-02, &
         0.1166200e-01, 0.2767925e-01, 0.6524293e-01, 0.1495746e+00, &
         0.3165503e+00, 0.5734129e+00, 0.9269114e+00, 0.1370421e+01, &
         0.1830986e+01, 0.2328271e+01, 0.2798606e+01, 0.3087561e+01, &
         0.2882272e+01, 0.2176012e+01, 0.1618124e+01, 0.1939663e+01, &
         0.2155108e+01, 0.1668577e+01, 0.1943499e+01, 0.1805721e+01, &
         0.1826187e+01, 0.1817263e+01, 0.1757304e+01, 0.1767201e+01, &
         0.5565022e-04, 0.1070932e-03, 0.2528019e-03, 0.5972348e-03, &
         0.1412555e-02, 0.3345877e-02, 0.7938247e-02, 0.1885045e-01, &
         0.4463263e-01, 0.1040232e+00, 0.2302090e+00, 0.4500423e+00, &
         0.7507968e+00, 0.1183003e+01, 0.1586547e+01, 0.2124766e+01, &
         0.2591938e+01, 0.2930691e+01, 0.3031670e+01, 0.2565965e+01, &
         0.1732758e+01, 0.1644230e+01, 0.2178035e+01, 0.1792361e+01, &
         0.1880221e+01, 0.1821903e+01, 0.1829716e+01, 0.1787137e+01, &
         0.1815011e+01, 0.1770356e+01, 0.3980661e-04, 0.7659413e-04, &
         0.1807602e-03, 0.4268896e-03, 0.1009173e-02, 0.2388930e-02, &
         0.5664303e-02, 0.1344884e-01, 0.3190569e-01, 0.7501386e-01, &
         0.1705220e+00, 0.3534238e+00, 0.6226878e+00, 0.1001000e+01, &
         0.1432039e+01, 0.1929170e+01, 0.2418807e+01, 0.2882358e+01, &
         0.3067569e+01, 0.2794610e+01, 0.2053816e+01, 0.1557362e+01, &
         0.2052078e+01, 0.2018643e+01, 0.1696542e+01, 0.1886784e+01, &
         0.1840827e+01, 0.1781331e+01, 0.1751741e+01, 0.1756093e+01, &
         0.2681827e-04, 0.5159516e-04, 0.1217334e-03, 0.2873892e-03, &
         0.6790494e-03, 0.1606384e-02, 0.3805922e-02, 0.9031690e-02, &
         0.2144548e-01, 0.5071694e-01, 0.1176591e+00, 0.2569804e+00, &
         0.4899683e+00, 0.8057491e+00, 0.1249726e+01, 0.1659806e+01, &
         0.2187880e+01, 0.2646229e+01, 0.2969582e+01, 0.3017386e+01, &
         0.2417457e+01, 0.1579919e+01, 0.1704027e+01, 0.2208308e+01, &
         0.1668819e+01, 0.1938163e+01, 0.1787129e+01, 0.1846662e+01, &
         0.1827163e+01, 0.1803410e+01, 0.2525160e-04, 0.4857985e-04, &
         0.1146153e-03, 0.2705707e-03, 0.6392652e-03, 0.1512128e-02, &
         0.3582197e-02, 0.8499914e-02, 0.2018366e-01, 0.4776109e-01, &
         0.1110539e+00, 0.2441137e+00, 0.4710058e+00, 0.7794141e+00, &
         0.1218766e+01, 0.1624015e+01, 0.2158341e+01, 0.2620128e+01, &
         0.2944883e+01, 0.3030327e+01, 0.2490729e+01, 0.1736488e+01, &
         0.1670145e+01, 0.2207124e+01, 0.1796909e+01, 0.2016083e+01, &
         0.1831616e+01, 0.1820178e+01, 0.1820695e+01, 0.1821183e+01/),&
         (/30,1,6/))

    qscat(1:30,1:1,25:30)=reshape( (/&
         0.2244690e-04, 0.4318224e-04, 0.1018737e-03, 0.2404698e-03, &
         0.5680734e-03, 0.1343483e-02, 0.3181970e-02, 0.7548691e-02, &
         0.1792551e-01, 0.4245912e-01, 0.9911473e-01, 0.2203728e+00, &
         0.4348921e+00, 0.7303964e+00, 0.1156200e+01, 0.1560783e+01, &
         0.2099243e+01, 0.2570755e+01, 0.2926596e+01, 0.3029954e+01, &
         0.2592838e+01, 0.1722351e+01, 0.1611468e+01, 0.2245607e+01, &
         0.1816836e+01, 0.1897303e+01, 0.1807774e+01, 0.1827002e+01, &
         0.1780776e+01, 0.1792670e+01, 0.1508714e-04, 0.2902153e-04, &
         0.6845398e-04, 0.1615359e-03, 0.3814493e-03, 0.9016190e-03, &
         0.2133902e-02, 0.5058516e-02, 0.1200905e-01, 0.2850071e-01, &
         0.6714708e-01, 0.1536901e+00, 0.3239290e+00, 0.5833729e+00, &
         0.9418291e+00, 0.1383321e+01, 0.1851519e+01, 0.2346708e+01, &
         0.2818617e+01, 0.3089535e+01, 0.2873804e+01, 0.2160190e+01, &
         0.1588495e+01, 0.1916183e+01, 0.2115090e+01, 0.1767522e+01, &
         0.1952622e+01, 0.1827966e+01, 0.1801739e+01, 0.1806674e+01, &
         0.1903640e-04, 0.3661983e-04, 0.8638486e-04, 0.2038823e-03, &
         0.4815583e-03, 0.1138599e-02, 0.2695894e-02, 0.6393607e-02, &
         0.1518182e-01, 0.3599739e-01, 0.8440971e-01, 0.1902647e+00, &
         0.3866871e+00, 0.6665245e+00, 0.1066252e+01, 0.1483992e+01, &
         0.2006419e+01, 0.2490376e+01, 0.2916542e+01, 0.3022475e+01, &
         0.2712411e+01, 0.1870931e+01, 0.1473295e+01, 0.2130482e+01, &
         0.1957074e+01, 0.1783501e+01, 0.1843272e+01, 0.1833044e+01, &
         0.1786526e+01, 0.1781775e+01, 0.1164130e-04, 0.2239145e-04, &
         0.5281003e-04, 0.1246017e-03, 0.2941640e-03, 0.6950784e-03, &
         0.1644360e-02, 0.3896082e-02, 0.9245978e-02, 0.2195383e-01, &
         0.5190634e-01, 0.1203065e+00, 0.2620847e+00, 0.4973831e+00, &
         0.8161713e+00, 0.1261464e+01, 0.1674268e+01, 0.2199425e+01, &
         0.2656993e+01, 0.2982174e+01, 0.3008611e+01, 0.2397170e+01, &
         0.1566271e+01, 0.1716662e+01, 0.2299051e+01, 0.1702490e+01, &
         0.1945370e+01, 0.1792890e+01, 0.1841396e+01, 0.1849848e+01, &
         0.9210702e-05, 0.1771512e-04, 0.4177722e-04, 0.9855754e-04, &
         0.2326356e-03, 0.5495443e-03, 0.1299600e-02, 0.3077847e-02, &
         0.7301237e-02, 0.1733785e-01, 0.4107690e-01, 0.9598324e-01, &
         0.2140418e+00, 0.4249914e+00, 0.7171717e+00, 0.1138275e+01, &
         0.1544475e+01, 0.2081769e+01, 0.2556129e+01, 0.2925340e+01, &
         0.3025771e+01, 0.2614344e+01, 0.1784989e+01, 0.1548554e+01, &
         0.2207918e+01, 0.1856935e+01, 0.1970010e+01, 0.1816692e+01, &
         0.1819728e+01, 0.1765595e+01, 0.6915328e-05, 0.1330029e-04, &
         0.3136207e-04, 0.7397675e-04, 0.1745795e-03, 0.4122800e-03, &
         0.9745910e-03, 0.2306919e-02, 0.5469492e-02, 0.1298585e-01, &
         0.3081126e-01, 0.7249000e-01, 0.1651520e+00, 0.3441308e+00, &
         0.6103705e+00, 0.9824560e+00, 0.1417094e+01, 0.1905561e+01, &
         0.2396655e+01, 0.2865654e+01, 0.3078475e+01, 0.2820626e+01, &
         0.2078964e+01, 0.1452464e+01, 0.2002201e+01, 0.2060558e+01, &
         0.1667076e+01, 0.1909250e+01, 0.1829093e+01, 0.1784407e+01/),&
         (/30,1,6/))

    qscat(1:30,1:1,31:36)=reshape( (/&
         0.4983547e-05, 0.9583965e-05, 0.2259792e-04, 0.5329637e-04, &
         0.1257496e-03, 0.2968784e-03, 0.7014993e-03, 0.1659575e-02, &
         0.3932194e-02, 0.9331833e-02, 0.2215747e-01, 0.5238259e-01, &
         0.1213650e+00, 0.2641168e+00, 0.5003189e+00, 0.8203176e+00, &
         0.1266057e+01, 0.1680060e+01, 0.2204016e+01, 0.2661379e+01, &
         0.2987473e+01, 0.3004859e+01, 0.2388839e+01, 0.1569596e+01, &
         0.1731562e+01, 0.2236392e+01, 0.1710583e+01, 0.1946052e+01, &
         0.1803016e+01, 0.1828850e+01, 0.3336525e-05, 0.6416181e-05, &
         0.1512720e-04, 0.3567183e-04, 0.8414762e-04, 0.1985995e-03, &
         0.4690658e-03, 0.1109019e-02, 0.2625725e-02, 0.6226893e-02, &
         0.1478571e-01, 0.3506286e-01, 0.8226940e-01, 0.1858021e+00, &
         0.3792889e+00, 0.6567879e+00, 0.1051920e+01, 0.1472531e+01, &
         0.1990234e+01, 0.2475642e+01, 0.2911975e+01, 0.3030586e+01, &
         0.2739404e+01, 0.1902393e+01, 0.1459937e+01, 0.2141851e+01, &
         0.1998446e+01, 0.1849737e+01, 0.1843773e+01, 0.1828411e+01, &
         0.1609733e-05, 0.3095464e-05, 0.7296939e-05, 0.1720301e-04, &
         0.4056917e-04, 0.9570643e-04, 0.2259008e-03, 0.5336171e-03, &
         0.1261881e-02, 0.2988344e-02, 0.7088546e-02, 0.1683272e-01, &
         0.3988795e-01, 0.9328347e-01, 0.2085486e+00, 0.4163024e+00, &
         0.7056225e+00, 0.1122294e+01, 0.1530449e+01, 0.2065823e+01, &
         0.2542608e+01, 0.2924463e+01, 0.3020677e+01, 0.2632277e+01, &
         0.1835126e+01, 0.1563670e+01, 0.2162810e+01, 0.1854607e+01, &
         0.1907334e+01, 0.1804355e+01, 0.8536749e-06, 0.1641531e-05, &
         0.3869212e-05, 0.9121159e-05, 0.2150566e-04, 0.5071913e-04, &
         0.1196660e-03, 0.2825053e-03, 0.6674958e-03, 0.1579006e-02, &
         0.3740940e-02, 0.8877230e-02, 0.2107904e-01, 0.4985903e-01, &
         0.1157458e+00, 0.2532728e+00, 0.4845448e+00, 0.7981708e+00, &
         0.1241009e+01, 0.1649390e+01, 0.2179454e+01, 0.2638595e+01, &
         0.2961328e+01, 0.3022736e+01, 0.2434684e+01, 0.1610582e+01, &
         0.1719000e+01, 0.2209665e+01, 0.1695634e+01, 0.1954261e+01, &
         0.5176838e-06, 0.9954088e-06, 0.2346362e-05, 0.5530849e-05, &
         0.1303859e-04, 0.3074600e-04, 0.7252304e-04, 0.1711466e-03, &
         0.4041660e-03, 0.9553821e-03, 0.2261380e-02, 0.5361313e-02, &
         0.1272874e-01, 0.3020329e-01, 0.7108603e-01, 0.1621523e+00, &
         0.3388926e+00, 0.6034029e+00, 0.9719585e+00, 0.1408524e+01, &
         0.1891890e+01, 0.2383881e+01, 0.2854789e+01, 0.3083266e+01, &
         0.2837948e+01, 0.2095346e+01, 0.1443578e+01, 0.1994793e+01, &
         0.2100053e+01, 0.1695583e+01, 0.2871854e-06, 0.5520937e-06, &
         0.1301202e-05, 0.3067068e-05, 0.7230215e-05, 0.1704622e-04, &
         0.4019847e-04, 0.9483170e-04, 0.2238343e-03, 0.5287331e-03, &
         0.1250309e-02, 0.2960899e-02, 0.7023325e-02, 0.1667778e-01, &
         0.3952316e-01, 0.9245398e-01, 0.2068544e+00, 0.4136037e+00, &
         0.7020445e+00, 0.1117285e+01, 0.1526141e+01, 0.2060742e+01, &
         0.2538260e+01, 0.2924151e+01, 0.3019181e+01, 0.2637346e+01, &
         0.1843114e+01, 0.1572628e+01, 0.2170437e+01, 0.1903235e+01/),&
         (/30,1,6/))

    qscat(1:30,1:1,37:42)=reshape( (/&
         0.2130258e-06, 0.4095630e-06, 0.9652463e-06, 0.2275018e-05, &
         0.5362792e-05, 0.1264280e-04, 0.2981126e-04, 0.7031697e-04, &
         0.1659372e-03, 0.3918542e-03, 0.9262424e-03, 0.2192283e-02, &
         0.5197192e-02, 0.1233868e-01, 0.2928065e-01, 0.6895282e-01, &
         0.1575780e+00, 0.3308383e+00, 0.5926470e+00, 0.9557645e+00, &
         0.1395105e+01, 0.1870389e+01, 0.2363955e+01, 0.2836186e+01, &
         0.3088190e+01, 0.2861268e+01, 0.2129971e+01, 0.1498628e+01, &
         0.1965536e+01, 0.2156023e+01, 0.1139642e-06, 0.2191180e-06, &
         0.5163641e-06, 0.1217040e-05, 0.2868713e-05, 0.6762461e-05, &
         0.1594324e-04, 0.3759716e-04, 0.8869241e-04, 0.2093319e-03, &
         0.4944418e-03, 0.1169102e-02, 0.2768247e-02, 0.6565538e-02, &
         0.1559031e-01, 0.3696075e-01, 0.8661259e-01, 0.1948363e+00, &
         0.3941952e+00, 0.6764117e+00, 0.1080671e+01, 0.1495647e+01, &
         0.2022275e+01, 0.2504614e+01, 0.2919853e+01, 0.3017293e+01, &
         0.2684913e+01, 0.1859088e+01, 0.1536330e+01, 0.2138908e+01, &
         0.4368971e-07, 0.8401571e-07, 0.1979443e-06, 0.4665041e-06, &
         0.1099423e-05, 0.2591267e-05, 0.6108387e-05, 0.1440077e-04, &
         0.3395877e-04, 0.8010484e-04, 0.1890504e-03, 0.4464910e-03, &
         0.1055577e-02, 0.2498972e-02, 0.5925733e-02, 0.1407011e-01, &
         0.3337342e-01, 0.7839159e-01, 0.1776647e+00, 0.3656191e+00, &
         0.6387883e+00, 0.1025151e+01, 0.1451272e+01, 0.1958829e+01, &
         0.2446588e+01, 0.2899261e+01, 0.3050108e+01, 0.2773031e+01, &
         0.2001010e+01, 0.1567242e+01, 0.2413002e-07, 0.4641096e-07, &
         0.1093743e-06, 0.2577491e-06, 0.6074086e-06, 0.1431569e-05, &
         0.3374563e-05, 0.7954735e-05, 0.1875470e-04, 0.4422979e-04, &
         0.1043463e-03, 0.2463109e-03, 0.5818865e-03, 0.1376202e-02, &
         0.3259612e-02, 0.7733199e-02, 0.1836365e-01, 0.4348901e-01, &
         0.1014429e+00, 0.2250517e+00, 0.4421329e+00, 0.7401199e+00, &
         0.1169107e+01, 0.1572968e+01, 0.2111608e+01, 0.2581027e+01, &
         0.2928094e+01, 0.3031281e+01, 0.2579799e+01, 0.1712282e+01, &
         0.1497814e-07, 0.2878358e-07, 0.6784342e-07, 0.1598607e-06, &
         0.3767407e-06, 0.8879427e-06, 0.2092868e-05, 0.4933270e-05, &
         0.1163001e-04, 0.2742337e-04, 0.6468265e-04, 0.1526333e-03, &
         0.3604091e-03, 0.8518165e-03, 0.2015840e-02, 0.4778121e-02, &
         0.1134260e-01, 0.2692300e-01, 0.6348782e-01, 0.1457673e+00, &
         0.3096642e+00, 0.5640594e+00, 0.9129618e+00, 0.1358052e+01, &
         0.1811523e+01, 0.2311157e+01, 0.2779208e+01, 0.3082438e+01, &
         0.2892156e+01, 0.2187006e+01, 0.1030113e-07, 0.1980520e-07, &
         0.4668839e-07, 0.1100485e-06, 0.2593487e-06, 0.6112119e-06, &
         0.1440549e-05, 0.3395485e-05, 0.8004393e-05, 0.1887181e-04, &
         0.4450613e-04, 0.1049992e-03, 0.2478533e-03, 0.5855319e-03, &
         0.1384842e-02, 0.3280106e-02, 0.7781909e-02, 0.1847929e-01, &
         0.4376074e-01, 0.1020566e+00, 0.2262809e+00, 0.4440247e+00, &
         0.7426682e+00, 0.1172451e+01, 0.1576190e+01, 0.2114785e+01, &
         0.2583662e+01, 0.2928657e+01, 0.3031442e+01, 0.2576708e+01/),&
         (/30,1,6/))

    qscat(1:30,1:1,43:48)=reshape( (/&
         0.8111209e-08, 0.1559276e-07, 0.3674471e-07, 0.8658950e-07, &
         0.2040660e-06, 0.4809785e-06, 0.1133488e-05, 0.2671544e-05, &
         0.6297743e-05, 0.1484709e-04, 0.3501190e-04, 0.8259039e-04, &
         0.1949215e-03, 0.4603695e-03, 0.1088433e-02, 0.2576899e-02, &
         0.6110877e-02, 0.1451004e-01, 0.3441217e-01, 0.8077721e-01, &
         0.1826787e+00, 0.3740697e+00, 0.6499188e+00, 0.1041741e+01, &
         0.1464437e+01, 0.1978474e+01, 0.2464825e+01, 0.2907833e+01, &
         0.3037615e+01, 0.2755203e+01, 0.6604543e-08, 0.1270030e-07, &
         0.2991808e-07, 0.7050917e-07, 0.1661954e-06, 0.3916392e-06, &
         0.9230437e-06, 0.2175630e-05, 0.5128434e-05, 0.1209001e-04, &
         0.2850855e-04, 0.6724245e-04, 0.1586770e-03, 0.3746897e-03, &
         0.8856207e-03, 0.2095975e-02, 0.4968437e-02, 0.1179495e-01, &
         0.2799396e-01, 0.6597276e-01, 0.1511538e+00, 0.3193898e+00, &
         0.5772527e+00, 0.9326557e+00, 0.1375426e+01, 0.1838931e+01, &
         0.2335359e+01, 0.2806430e+01, 0.3088753e+01, 0.2879159e+01, &
         0.5314572e-08, 0.1022444e-07, 0.2410669e-07, 0.5677523e-07, &
         0.1338119e-06, 0.3153663e-06, 0.7432756e-06, 0.1751925e-05, &
         0.4129471e-05, 0.9734469e-05, 0.2295285e-04, 0.5413433e-04, &
         0.1277282e-03, 0.3015520e-03, 0.7125568e-03, 0.1685775e-02, &
         0.3994397e-02, 0.9479688e-02, 0.2250814e-01, 0.5320236e-01, &
         0.1231845e+00, 0.2675992e+00, 0.5053291e+00, 0.8274198e+00, &
         0.1273818e+01, 0.1690028e+01, 0.2211889e+01, 0.2669053e+01, &
         0.2996819e+01, 0.2998283e+01, 0.4225172e-08, 0.8133153e-08, &
         0.1916917e-07, 0.4515045e-07, 0.1064213e-06, 0.2507710e-06, &
         0.5910236e-06, 0.1392893e-05, 0.3283100e-05, 0.7739130e-05, &
         0.1824681e-04, 0.4303140e-04, 0.1015181e-03, 0.2396298e-03, &
         0.5660809e-03, 0.1338762e-02, 0.3170775e-02, 0.7522085e-02, &
         0.1786233e-01, 0.4231057e-01, 0.9877851e-01, 0.2196952e+00, &
         0.4338378e+00, 0.7289849e+00, 0.1154307e+01, 0.1559031e+01, &
         0.2097416e+01, 0.2569233e+01, 0.2926430e+01, 0.3029643e+01, &
         0.3311696e-08, 0.6376909e-08, 0.1502053e-07, 0.3541976e-07, &
         0.8341913e-07, 0.1966386e-06, 0.4633977e-06, 0.1092155e-05, &
         0.2574293e-05, 0.6068137e-05, 0.1430610e-04, 0.3373469e-04, &
         0.7957646e-04, 0.1878019e-03, 0.4435409e-03, 0.1048594e-02, &
         0.2482403e-02, 0.5886369e-02, 0.1397656e-01, 0.3315251e-01, &
         0.7788367e-01, 0.1765939e+00, 0.3638026e+00, 0.6363935e+00, &
         0.1021569e+01, 0.1448428e+01, 0.1954509e+01, 0.2442554e+01, &
         0.2897095e+01, 0.3052830e+01, 0.2558936e-08, 0.4921040e-08, &
         0.1160372e-07, 0.2734706e-07, 0.6442724e-07, 0.1518185e-06, &
         0.3578024e-06, 0.8431982e-06, 0.1987328e-05, 0.4684338e-05, &
         0.1104337e-04, 0.2603939e-04, 0.6141687e-04, 0.1449215e-03, &
         0.3421815e-03, 0.8086877e-03, 0.1913604e-02, 0.4535340e-02, &
         0.1076550e-01, 0.2555612e-01, 0.6031040e-01, 0.1388400e+00, &
         0.2969850e+00, 0.5466675e+00, 0.8872278e+00, 0.1334345e+01, &
         0.1775121e+01, 0.2280190e+01, 0.2743268e+01, 0.3065531e+01/),&
         (/30,1,6/))

    qscat(1:30,1:1,49:50)=reshape( (/&
         0.1941416e-08, 0.3729086e-08, 0.8796222e-08, 0.2073586e-07, &
         0.4884795e-07, 0.1151409e-06, 0.2713934e-06, 0.6395728e-06, &
         0.1507417e-05, 0.3552973e-05, 0.8375334e-05, 0.1974695e-04, &
         0.4657068e-04, 0.1098731e-03, 0.2593660e-03, 0.6127629e-03, &
         0.1449341e-02, 0.3433179e-02, 0.8145735e-02, 0.1934303e-01, &
         0.4578911e-01, 0.1066269e+00, 0.2353837e+00, 0.4579038e+00, &
         0.7614718e+00, 0.1196600e+01, 0.1600340e+01, 0.2137544e+01, &
         0.2602569e+01, 0.2934595e+01, 0.1327624e-08, 0.2548038e-08, &
         0.5997629e-08, 0.1413148e-07, 0.3332353e-07, 0.7850970e-07, &
         0.1850432e-06, 0.4361592e-06, 0.1027941e-05, 0.2423005e-05, &
         0.5711535e-05, 0.1346519e-04, 0.3175162e-04, 0.7489492e-04, &
         0.1767484e-03, 0.4174066e-03, 0.9867256e-03, 0.2335701e-02, &
         0.5537855e-02, 0.1314833e-01, 0.3119540e-01, 0.7337637e-01, &
         0.1670410e+00, 0.3474123e+00, 0.6147258e+00, 0.9890165e+00, &
         0.1422405e+01, 0.1913994e+01, 0.2404561e+01, 0.2871936e+01/),&
         (/30,1,2/))

    qbrqs(1:30,1:1,1:6)=reshape( (/&
         0.1901999e-05, 0.4716377e-12, 0.7370087e-12, 0.3116265e-12, &
         -0.2044577e-11, 0.1364757e-11, 0.2750097e-11, 0.1206867e-10, &
         0.7804748e-10, 0.2244802e-09, 0.7640885e-09, 0.2786042e-08, &
         0.1003014e-07, 0.3626222e-07, 0.1312045e-06, 0.4747086e-06, &
         0.1717792e-05, 0.6216306e-05, 0.2250494e-04, 0.8146375e-04, &
         0.2948020e-03, 0.1065178e-02, 0.3828139e-02, 0.1352974e-01, &
         0.4564070e-01, 0.1391708e+00, 0.3677734e+00, 0.7288581e+00, &
         0.1024653e+01, 0.1386942e+01, 0.3617603e-05, 0.2991549e-12, &
         -0.5700891e-12, 0.1586302e-11, 0.9922366e-12, 0.1795592e-11, &
         -0.3836203e-11, 0.1123241e-10, 0.2974527e-10, 0.8357062e-10, &
         0.3698330e-09, 0.1441129e-08, 0.5219182e-08, 0.1896117e-07, &
         0.6871815e-07, 0.2483693e-06, 0.8989706e-06, 0.3253124e-05, &
         0.1177489e-04, 0.4262225e-04, 0.1542823e-03, 0.5580288e-03, &
         0.2012467e-02, 0.7188866e-02, 0.2496183e-01, 0.8068376e-01, &
         0.2299847e+00, 0.5466217e+00, 0.8696541e+00, 0.1232296e+01, &
         0.6880682e-05, 0.1916953e-12, 0.4707819e-12, 0.3414841e-12, &
         0.4034255e-12, 0.1418572e-12, 0.3161054e-12, 0.1475615e-11, &
         0.1669863e-10, 0.3005890e-10, 0.1336998e-09, 0.5276825e-09, &
         0.1935742e-08, 0.6909711e-08, 0.2535454e-07, 0.9163686e-07, &
         0.3316923e-06, 0.1200273e-05, 0.4344654e-05, 0.1572453e-04, &
         0.5692060e-04, 0.2060147e-03, 0.7448617e-03, 0.2682767e-02, &
         0.9544524e-02, 0.3276870e-01, 0.1033715e+00, 0.2852662e+00, &
         0.6315490e+00, 0.9307482e+00, 0.1308705e-04, 0.3488645e-12, &
         -0.2643488e-12, 0.3843259e-12, 0.5917927e-12, 0.1269802e-11, &
         -0.1910060e-11, 0.4475100e-11, 0.2189990e-11, 0.1775183e-10, &
         0.7611258e-10, 0.2165611e-09, 0.8112211e-09, 0.2849104e-08, &
         0.1058281e-07, 0.3819969e-07, 0.1382846e-06, 0.5006514e-06, &
         0.1810602e-05, 0.6554102e-05, 0.2372228e-04, 0.8587552e-04, &
         0.3107595e-03, 0.1122709e-02, 0.4033369e-02, 0.1423876e-01, &
         0.4789057e-01, 0.1452604e+00, 0.3812047e+00, 0.7420156e+00, &
         0.2489158e-04, 0.4701444e-13, 0.1535593e-12, 0.3381911e-12, &
         0.5717167e-12, 0.4095736e-12, 0.2617854e-12, 0.3672750e-11, &
         0.3647954e-11, 0.8980333e-11, 0.2740179e-10, 0.1001760e-09, &
         0.3395771e-09, 0.1195295e-08, 0.4350772e-08, 0.1593059e-07, &
         0.5727330e-07, 0.2076147e-06, 0.7506055e-06, 0.2718510e-05, &
         0.9838780e-05, 0.3561608e-04, 0.1289225e-03, 0.4663866e-03, &
         0.1683064e-02, 0.6024596e-02, 0.2104279e-01, 0.6894487e-01, &
         0.2004059e+00, 0.4936804e+00, 0.4734378e-04, 0.1351300e-13, &
         -0.1268058e-12, 0.2985778e-13, 0.3237699e-12, 0.6953172e-13, &
         -0.5454742e-12, 0.2173147e-11, 0.1272297e-11, 0.2537878e-11, &
         0.6615754e-11, 0.2416349e-10, 0.1269510e-09, 0.4061239e-09, &
         0.1475770e-08, 0.5395762e-08, 0.1915448e-07, 0.6942866e-07, &
         0.2513579e-06, 0.9103257e-06, 0.3293702e-05, 0.1192200e-04, &
         0.4315420e-04, 0.1562069e-03, 0.5649814e-03, 0.2037445e-02, &
         0.7276999e-02, 0.2525676e-01, 0.8155691e-01, 0.2321569e+00/),&
         (/30,1,6/))

    qbrqs(1:30,1:1,7:12)=reshape( (/&
         0.9004787e-04, 0.2465319e-14, 0.6956497e-13, 0.7135020e-13, &
         -0.7006778e-13, 0.4401354e-12, 0.4894050e-12, 0.6742820e-12, &
         -0.1286480e-11, 0.2726573e-12, 0.1833192e-11, 0.4639781e-11, &
         0.2582518e-10, 0.1105154e-09, 0.3573153e-09, 0.1309629e-08, &
         0.4652057e-08, 0.1709559e-07, 0.6191839e-07, 0.2242348e-06, &
         0.8105023e-06, 0.2933317e-05, 0.1061740e-04, 0.3843134e-04, &
         0.1391129e-03, 0.5032207e-03, 0.1815523e-02, 0.6493290e-02, &
         0.2262557e-01, 0.7371690e-01, 0.1712710e-03, 0.1129701e-13, &
         -0.3926947e-13, 0.2638894e-13, 0.5981981e-13, 0.1928346e-12, &
         0.1923591e-12, 0.7062969e-12, 0.8035807e-12, 0.1282500e-11, &
         0.7179135e-12, 0.1216088e-11, 0.4708282e-11, 0.2094201e-10, &
         0.5307086e-10, 0.2274950e-09, 0.8008884e-09, 0.2959651e-08, &
         0.1066964e-07, 0.3889165e-07, 0.1402413e-06, 0.5080287e-06, &
         0.1838745e-05, 0.6654002e-05, 0.2408813e-04, 0.8719498e-04, &
         0.3155253e-03, 0.1139886e-02, 0.4094611e-02, 0.1445007e-01, &
         0.3257575e-03, 0.8448067e-14, 0.9466076e-14, 0.6598221e-14, &
         0.5819855e-13, 0.1483693e-12, 0.2723606e-13, 0.1134216e-12, &
         -0.7536919e-12, 0.2585718e-12, 0.3713785e-13, 0.1066277e-11, &
         0.1599707e-12, 0.3999822e-12, 0.3919878e-11, 0.1730125e-10, &
         0.4113725e-10, 0.1999410e-09, 0.6582198e-09, 0.2469145e-08, &
         0.8993994e-08, 0.3211079e-07, 0.1164509e-06, 0.4218663e-06, &
         0.1527693e-05, 0.5526857e-05, 0.2000606e-04, 0.7242112e-04, &
         0.2620910e-03, 0.9472192e-03, 0.3353513e-01, 0.3950379e-04, &
         0.1429898e-03, 0.5172197e-03, 0.1865839e-02, 0.6671186e-02, &
         0.2322445e-01, 0.7551139e-01, 0.2170407e+00, 0.5241100e+00, &
         0.8542780e+00, 0.1208818e+01, 0.1538107e+01, 0.1891108e+01, &
         0.2099686e+01, 0.1850941e+01, 0.1153786e+01, 0.7584199e+00, &
         0.1408991e+01, 0.1537233e+01, 0.1201549e+01, 0.1408215e+01, &
         0.1379510e+01, 0.1359040e+01, 0.1355852e+01, 0.1382048e+01, &
         0.1385486e+01, 0.1393122e+01, 0.1399668e+01, 0.1368301e+01, &
         0.8438055e-05, 0.2306761e-04, 0.8350273e-04, 0.3021766e-03, &
         0.1091762e-02, 0.3922987e-02, 0.1385760e-01, 0.4668255e-01, &
         0.1419962e+00, 0.3740283e+00, 0.7350657e+00, 0.1032138e+01, &
         0.1392090e+01, 0.1722258e+01, 0.1978830e+01, 0.2038919e+01, &
         0.1466292e+01, 0.8217813e+00, 0.1068222e+01, 0.1678929e+01, &
         0.1179439e+01, 0.1465021e+01, 0.1322734e+01, 0.1388140e+01, &
         0.1398959e+01, 0.1402826e+01, 0.1374126e+01, 0.1401267e+01, &
         0.1384118e+01, 0.1380084e+01, 0.6045238e-05, 0.1652407e-04, &
         0.5981760e-04, 0.2164966e-03, 0.7826978e-03, 0.2818324e-02, &
         0.1001874e-01, 0.3432159e-01, 0.1077876e+00, 0.2957576e+00, &
         0.6457256e+00, 0.9422171e+00, 0.1321832e+01, 0.1654439e+01, &
         0.1947359e+01, 0.2037713e+01, 0.1652677e+01, 0.9366399e+00, &
         0.8927257e+00, 0.1617906e+01, 0.1334269e+01, 0.1372657e+01, &
         0.1323403e+01, 0.1372922e+01, 0.1323401e+01, 0.1366046e+01, &
         0.1377430e+01, 0.1352949e+01, 0.1383479e+01, 0.1394892e+01/),&
         (/30,1,6/))

    qbrqs(1:30,1:1,13:18)=reshape( (/&
         0.3857907e-05, 0.1054521e-04, 0.3817201e-04, 0.1381718e-03, &
         0.4998088e-03, 0.1803249e-02, 0.6449907e-02, 0.2247932e-01, &
         0.7327781e-01, 0.2114086e+00, 0.5139958e+00, 0.8473598e+00, &
         0.1197895e+01, 0.1527089e+01, 0.1880320e+01, 0.2101947e+01, &
         0.1867903e+01, 0.1170279e+01, 0.7712408e+00, 0.1393570e+01, &
         0.1562685e+01, 0.1217631e+01, 0.1419034e+01, 0.1369893e+01, &
         0.1385373e+01, 0.1366546e+01, 0.1356363e+01, 0.1383033e+01, &
         0.1387707e+01, 0.1393639e+01, 0.2875744e-05, 0.7860280e-05, &
         0.2845365e-04, 0.1030003e-03, 0.3726821e-03, 0.1345828e-02, &
         0.4827840e-02, 0.1696944e-01, 0.5645249e-01, 0.1680575e+00, &
         0.4297750e+00, 0.7848734e+00, 0.1099823e+01, 0.1440201e+01, &
         0.1776021e+01, 0.2050056e+01, 0.1986646e+01, 0.1322586e+01, &
         0.7947940e+00, 0.1190645e+01, 0.1659948e+01, 0.1198948e+01, &
         0.1479658e+01, 0.1347349e+01, 0.1393301e+01, 0.1416820e+01, &
         0.1365937e+01, 0.1370853e+01, 0.1368950e+01, 0.1383935e+01, &
         0.1718626e-05, 0.4697151e-05, 0.1700411e-04, 0.6155527e-04, &
         0.2227757e-03, 0.8053585e-03, 0.2899469e-02, 0.1030230e-01, &
         0.3524727e-01, 0.1104061e+00, 0.3019377e+00, 0.6538075e+00, &
         0.9490179e+00, 0.1328286e+01, 0.1660758e+01, 0.1947923e+01, &
         0.2039193e+01, 0.1638955e+01, 0.9184783e+00, 0.8948992e+00, &
         0.1630617e+01, 0.1313450e+01, 0.1395140e+01, 0.1328123e+01, &
         0.1377631e+01, 0.1357848e+01, 0.1388879e+01, 0.1391962e+01, &
         0.1361340e+01, 0.1369491e+01, 0.7404256e-06, 0.2023248e-05, &
         0.7321936e-05, 0.2650530e-04, 0.9594424e-04, 0.3471650e-03, &
         0.1253919e-02, 0.4500857e-02, 0.1584823e-01, 0.5295629e-01, &
         0.1588167e+00, 0.4104221e+00, 0.7685978e+00, 0.1076261e+01, &
         0.1422771e+01, 0.1755290e+01, 0.2024872e+01, 0.2010120e+01, &
         0.1377721e+01, 0.8087896e+00, 0.1148468e+01, 0.1692759e+01, &
         0.1168727e+01, 0.1450756e+01, 0.1345646e+01, 0.1390995e+01, &
         0.1421059e+01, 0.1388422e+01, 0.1380926e+01, 0.1383566e+01, &
         0.5598029e-06, 0.1530578e-05, 0.5538715e-05, 0.2004728e-04, &
         0.7256944e-04, 0.2626222e-03, 0.9491406e-03, 0.3413769e-02, &
         0.1209360e-01, 0.4104747e-01, 0.1265943e+00, 0.3394618e+00, &
         0.6988942e+00, 0.9913729e+00, 0.1363254e+01, 0.1694228e+01, &
         0.1954880e+01, 0.2044281e+01, 0.1563253e+01, 0.8779636e+00, &
         0.9851181e+00, 0.1671286e+01, 0.1245029e+01, 0.1393614e+01, &
         0.1312447e+01, 0.1383697e+01, 0.1368982e+01, 0.1410644e+01, &
         0.1386400e+01, 0.1377081e+01, 0.3768961e-06, 0.1029766e-05, &
         0.3728270e-05, 0.1349196e-04, 0.4884070e-04, 0.1767822e-03, &
         0.6393095e-03, 0.2304285e-02, 0.8216626e-02, 0.2838739e-01, &
         0.9074473e-01, 0.2547902e+00, 0.5869321e+00, 0.8975460e+00, &
         0.1271339e+01, 0.1603128e+01, 0.1935510e+01, 0.2057565e+01, &
         0.1761917e+01, 0.1026894e+01, 0.8074358e+00, 0.1538698e+01, &
         0.1437847e+01, 0.1310425e+01, 0.1362680e+01, 0.1381976e+01, &
         0.1356858e+01, 0.1333898e+01, 0.1381830e+01, 0.1388422e+01/),&
         (/30,1,6/))

    qbrqs(1:30,1:1,19:24)=reshape( (/&
         0.3236136e-06, 0.8855458e-06, 0.3204303e-05, 0.1159740e-04, &
         0.4198187e-04, 0.1519609e-03, 0.5496361e-03, 0.1982331e-02, &
         0.7082549e-02, 0.2460569e-01, 0.7962730e-01, 0.2273520e+00, &
         0.5421261e+00, 0.8665863e+00, 0.1227701e+01, 0.1557529e+01, &
         0.1907946e+01, 0.2091759e+01, 0.1820815e+01, 0.1128173e+01, &
         0.7792270e+00, 0.1441805e+01, 0.1500771e+01, 0.1177203e+01, &
         0.1397181e+01, 0.1378007e+01, 0.1360600e+01, 0.1342718e+01, &
         0.1381574e+01, 0.1394904e+01, 0.2598837e-06, 0.7105884e-06, &
         0.2571407e-05, 0.9306488e-05, 0.3368797e-04, 0.1219437e-03, &
         0.4411651e-03, 0.1592339e-02, 0.5703115e-02, 0.1995317e-01, &
         0.6563381e-01, 0.1919264e+00, 0.4775247e+00, 0.8215518e+00, &
         0.1156758e+01, 0.1487721e+01, 0.1835645e+01, 0.2094382e+01, &
         0.1909802e+01, 0.1222000e+01, 0.7729526e+00, 0.1325953e+01, &
         0.1618831e+01, 0.1129606e+01, 0.1441841e+01, 0.1368787e+01, &
         0.1390773e+01, 0.1396055e+01, 0.1353166e+01, 0.1377240e+01, &
         0.1468962e-06, 0.4008892e-06, 0.1451859e-05, 0.5252375e-05, &
         0.1901069e-04, 0.6881658e-04, 0.2490552e-03, 0.9001884e-03, &
         0.3238772e-02, 0.1148521e-01, 0.3908658e-01, 0.1211622e+00, &
         0.3270030e+00, 0.6846489e+00, 0.9770864e+00, 0.1352290e+01, &
         0.1683818e+01, 0.1951200e+01, 0.2043353e+01, 0.1586950e+01, &
         0.8647165e+00, 0.9542888e+00, 0.1668071e+01, 0.1251961e+01, &
         0.1387784e+01, 0.1315097e+01, 0.1377233e+01, 0.1364871e+01, &
         0.1410496e+01, 0.1380941e+01, 0.8878270e-07, 0.2427971e-06, &
         0.8784166e-06, 0.3178303e-05, 0.1150401e-04, 0.4164483e-04, &
         0.1507380e-03, 0.5452277e-03, 0.1966480e-02, 0.7026603e-02, &
         0.2441811e-01, 0.7907017e-01, 0.2259613e+00, 0.5397338e+00, &
         0.8649546e+00, 0.1225237e+01, 0.1554972e+01, 0.1905898e+01, &
         0.2093079e+01, 0.1824446e+01, 0.1131443e+01, 0.7718529e+00, &
         0.1433445e+01, 0.1505972e+01, 0.1179354e+01, 0.1397423e+01, &
         0.1375723e+01, 0.1354594e+01, 0.1339999e+01, 0.1368543e+01, &
         0.4909048e-07, 0.1342343e-06, 0.4857623e-06, 0.1758437e-05, &
         0.6363671e-05, 0.2303496e-04, 0.8338189e-04, 0.3017350e-03, &
         0.1090177e-02, 0.3917343e-02, 0.1383809e-01, 0.4662061e-01, &
         0.1418286e+00, 0.3736582e+00, 0.7347008e+00, 0.1031695e+01, &
         0.1391785e+01, 0.1721950e+01, 0.1978449e+01, 0.2039113e+01, &
         0.1467247e+01, 0.8228753e+00, 0.1067643e+01, 0.1679000e+01, &
         0.1180979e+01, 0.1465477e+01, 0.1323149e+01, 0.1388206e+01, &
         0.1398749e+01, 0.1403252e+01, 0.4491110e-07, 0.1226495e-06, &
         0.4438543e-06, 0.1606513e-05, 0.5813939e-05, 0.2104629e-04, &
         0.7618610e-04, 0.2757049e-03, 0.9963178e-03, 0.3582313e-02, &
         0.1267850e-01, 0.4292409e-01, 0.1317570e+00, 0.3511752e+00, &
         0.7116793e+00, 0.1005011e+01, 0.1373192e+01, 0.1703715e+01, &
         0.1960462e+01, 0.2044763e+01, 0.1533077e+01, 0.8851303e+00, &
         0.1014989e+01, 0.1669717e+01, 0.1245077e+01, 0.1428272e+01, &
         0.1312307e+01, 0.1375859e+01, 0.1382066e+01, 0.1413976e+01/),&
         (/30,1,6/))

    qbrqs(1:30,1:1,25:30)=reshape( (/&
         0.3771462e-07, 0.1028282e-06, 0.3719264e-06, 0.1346401e-05, &
         0.4873332e-05, 0.1763927e-04, 0.6385347e-04, 0.2310964e-03, &
         0.8353932e-03, 0.3006974e-02, 0.1067756e-01, 0.3646917e-01, &
         0.1138473e+00, 0.3100131e+00, 0.6640764e+00, 0.9579683e+00, &
         0.1336368e+01, 0.1668591e+01, 0.1948658e+01, 0.2041109e+01, &
         0.1620370e+01, 0.8906084e+00, 0.9080491e+00, 0.1650102e+01, &
         0.1287251e+01, 0.1404940e+01, 0.1312431e+01, 0.1369794e+01, &
         0.1347291e+01, 0.1383594e+01, 0.2066275e-07, 0.5672286e-07, &
         0.2053315e-06, 0.7420299e-06, 0.2685598e-05, 0.9722087e-05, &
         0.3519086e-04, 0.1273800e-03, 0.4608098e-03, 0.1663016e-02, &
         0.5953574e-02, 0.2080238e-01, 0.6821626e-01, 0.1985453e+00, &
         0.4901721e+00, 0.8306453e+00, 0.1171311e+01, 0.1501209e+01, &
         0.1851867e+01, 0.2099972e+01, 0.1896659e+01, 0.1208096e+01, &
         0.8045296e+00, 0.1344672e+01, 0.1607270e+01, 0.1197700e+01, &
         0.1446757e+01, 0.1368162e+01, 0.1369020e+01, 0.1384346e+01, &
         0.2943546e-07, 0.8037221e-07, 0.2906546e-06, 0.1051397e-05, &
         0.3806524e-05, 0.1377661e-04, 0.4987255e-04, 0.1805111e-03, &
         0.6527653e-03, 0.2352573e-02, 0.8386328e-02, 0.2895008e-01, &
         0.9238102e-01, 0.2587797e+00, 0.5930740e+00, 0.9019188e+00, &
         0.1276954e+01, 0.1608966e+01, 0.1937796e+01, 0.2052794e+01, &
         0.1750075e+01, 0.1011107e+01, 0.8058437e+00, 0.1549847e+01, &
         0.1425072e+01, 0.1307907e+01, 0.1361917e+01, 0.1381650e+01, &
         0.1353116e+01, 0.1336342e+01, 0.1401500e-07, 0.3835394e-07, &
         0.1390823e-06, 0.5033391e-06, 0.1820643e-05, 0.6589878e-05, &
         0.2385175e-04, 0.8634376e-04, 0.3124391e-03, 0.1128766e-02, &
         0.4054963e-02, 0.1431323e-01, 0.4812629e-01, 0.1458958e+00, &
         0.3825957e+00, 0.7433397e+00, 0.1042454e+01, 0.1399161e+01, &
         0.1729491e+01, 0.1988325e+01, 0.2033609e+01, 0.1445695e+01, &
         0.8003992e+00, 0.1082630e+01, 0.1693802e+01, 0.1152685e+01, &
         0.1457584e+01, 0.1329207e+01, 0.1388109e+01, 0.1413331e+01, &
         0.9938845e-08, 0.2700967e-07, 0.9789189e-07, 0.3541029e-06, &
         0.1281403e-05, 0.4637087e-05, 0.1678571e-04, 0.6076441e-04, &
         0.2199208e-03, 0.7950518e-03, 0.2862566e-02, 0.1017336e-01, &
         0.3482668e-01, 0.1092176e+00, 0.2991365e+00, 0.6501684e+00, &
         0.9459308e+00, 0.1325391e+01, 0.1657931e+01, 0.1947675e+01, &
         0.2038492e+01, 0.1645274e+01, 0.9275196e+00, 0.8941895e+00, &
         0.1624768e+01, 0.1322001e+01, 0.1387358e+01, 0.1331443e+01, &
         0.1374707e+01, 0.1341997e+01, 0.6412746e-08, 0.1769780e-07, &
         0.6366894e-07, 0.2303611e-06, 0.8336778e-06, 0.3016920e-05, &
         0.1092051e-04, 0.3952926e-04, 0.1430871e-03, 0.5175775e-03, &
         0.1867124e-02, 0.6675717e-02, 0.2323973e-01, 0.7555697e-01, &
         0.2171555e+00, 0.5243142e+00, 0.8544180e+00, 0.1209036e+01, &
         0.1538329e+01, 0.1891320e+01, 0.2099623e+01, 0.1850582e+01, &
         0.1153475e+01, 0.7582853e+00, 0.1409339e+01, 0.1536727e+01, &
         0.1201235e+01, 0.1408159e+01, 0.1379772e+01, 0.1358475e+01/),&
         (/30,1,6/))

    qbrqs(1:30,1:1,31:36)=reshape( (/&
         0.3923926e-08, 0.1067798e-07, 0.3897862e-07, 0.1409177e-06, &
         0.5099691e-06, 0.1845879e-05, 0.6680979e-05, 0.2418180e-04, &
         0.8753642e-04, 0.3167605e-03, 0.1144346e-02, 0.4110507e-02, &
         0.1450484e-01, 0.4873199e-01, 0.1475266e+00, 0.3861555e+00, &
         0.7467078e+00, 0.1046755e+01, 0.1402116e+01, 0.1732574e+01, &
         0.1992608e+01, 0.2031040e+01, 0.1437798e+01, 0.7941419e+00, &
         0.1091892e+01, 0.1695350e+01, 0.1146589e+01, 0.1452048e+01, &
         0.1327341e+01, 0.1386919e+01, 0.2159660e-08, 0.5853871e-08, &
         0.2139184e-07, 0.7725549e-07, 0.2794547e-06, 0.1011294e-05, &
         0.3659937e-05, 0.1324533e-04, 0.4794678e-04, 0.1735502e-03, &
         0.6276303e-03, 0.2262394e-02, 0.8069280e-02, 0.2789819e-01, &
         0.8931855e-01, 0.2513025e+00, 0.5814862e+00, 0.8937050e+00, &
         0.1266289e+01, 0.1597861e+01, 0.1933207e+01, 0.2062025e+01, &
         0.1771295e+01, 0.1042512e+01, 0.8124586e+00, 0.1524010e+01, &
         0.1449887e+01, 0.1308147e+01, 0.1367361e+01, 0.1380090e+01, &
         0.7260568e-09, 0.2020068e-08, 0.7262182e-08, 0.2583743e-07, &
         0.9367815e-07, 0.3389539e-06, 0.1226493e-05, 0.4437584e-05, &
         0.1606419e-04, 0.5814762e-04, 0.2104592e-03, 0.7609050e-03, &
         0.2740245e-02, 0.9745683e-02, 0.3342814e-01, 0.1052505e+00, &
         0.2897407e+00, 0.6376665e+00, 0.9356289e+00, 0.1315299e+01, &
         0.1647984e+01, 0.1946704e+01, 0.2036762e+01, 0.1665433e+01, &
         0.9483317e+00, 0.8870602e+00, 0.1611704e+01, 0.1349355e+01, &
         0.1338266e+01, 0.1332024e+01, 0.2729792e-09, 0.7788261e-09, &
         0.2781656e-08, 0.1002817e-07, 0.3617201e-07, 0.1306958e-06, &
         0.4733878e-06, 0.1714013e-05, 0.6202667e-05, 0.2245028e-04, &
         0.8127029e-04, 0.2940956e-03, 0.1062638e-02, 0.3819079e-02, &
         0.1349840e-01, 0.4554095e-01, 0.1388998e+00, 0.3671713e+00, &
         0.7282541e+00, 0.1023936e+01, 0.1386446e+01, 0.1716618e+01, &
         0.1972223e+01, 0.2042006e+01, 0.1485218e+01, 0.8435971e+00, &
         0.1052239e+01, 0.1682687e+01, 0.1208819e+01, 0.1468988e+01, &
         0.1138483e-09, 0.3352924e-09, 0.1323467e-08, 0.4765051e-08, &
         0.1696672e-07, 0.6176972e-07, 0.2235700e-06, 0.8091876e-06, &
         0.2928622e-05, 0.1059961e-04, 0.3837255e-04, 0.1388960e-03, &
         0.5024281e-03, 0.1812669e-02, 0.6483227e-02, 0.2259160e-01, &
         0.7361497e-01, 0.2122605e+00, 0.5155379e+00, 0.8483830e+00, &
         0.1199574e+01, 0.1528771e+01, 0.1882021e+01, 0.2101714e+01, &
         0.1865457e+01, 0.1167623e+01, 0.7686450e+00, 0.1395737e+01, &
         0.1559300e+01, 0.1215185e+01, 0.6871950e-10, 0.1435970e-09, &
         0.5198381e-09, 0.1935983e-08, 0.7118665e-08, 0.2559486e-07, &
         0.9233558e-07, 0.3342500e-06, 0.1209471e-05, 0.4377391e-05, &
         0.1584374e-04, 0.5735440e-04, 0.2075856e-03, 0.7505263e-03, &
         0.2703073e-02, 0.9615588e-02, 0.3300178e-01, 0.1040363e+00, &
         0.2868509e+00, 0.6337283e+00, 0.9324754e+00, 0.1312067e+01, &
         0.1644767e+01, 0.1946319e+01, 0.2036625e+01, 0.1671548e+01, &
         0.9529492e+00, 0.8828400e+00, 0.1606683e+01, 0.1356629e+01/),&
         (/30,1,6/))

    qbrqs(1:30,1:1,37:42)=reshape( (/&
         0.4279296e-10, 0.1019830e-09, 0.3495865e-09, 0.1240186e-08, &
         0.4548052e-08, 0.1637553e-07, 0.5894112e-07, 0.2132708e-06, &
         0.7721861e-06, 0.2795869e-05, 0.1012025e-04, 0.3663430e-04, &
         0.1326112e-03, 0.4797276e-03, 0.1731045e-02, 0.6194473e-02, &
         0.2161726e-01, 0.7068184e-01, 0.2048290e+00, 0.5019357e+00, &
         0.8389409e+00, 0.1184580e+01, 0.1513944e+01, 0.1866371e+01, &
         0.2102310e+01, 0.1884416e+01, 0.1191447e+01, 0.7944575e+00, &
         0.1358286e+01, 0.1585731e+01, 0.1320506e-10, 0.3754272e-10, &
         0.1172757e-09, 0.4610952e-09, 0.1747573e-08, 0.6396914e-08, &
         0.2307197e-07, 0.8355009e-07, 0.3025998e-06, 0.1094051e-05, &
         0.3960026e-05, 0.1433091e-04, 0.5187821e-04, 0.1877786e-03, &
         0.6790176e-03, 0.2446735e-02, 0.8717073e-02, 0.3004434e-01, &
         0.9555029e-01, 0.2664733e+00, 0.6046580e+00, 0.9103130e+00, &
         0.1287310e+01, 0.1619669e+01, 0.1941297e+01, 0.2045045e+01, &
         0.1725672e+01, 0.9880461e+00, 0.8178003e+00, 0.1567536e+01, &
         0.5899305e-11, 0.2042627e-10, 0.3764592e-10, 0.1280842e-09, &
         0.4243975e-09, 0.1496634e-08, 0.5502207e-08, 0.1978670e-07, &
         0.7175582e-07, 0.2595964e-06, 0.9389952e-06, 0.3399184e-05, &
         0.1230400e-04, 0.4453738e-04, 0.1612071e-03, 0.5830508e-03, &
         0.2102342e-02, 0.7505785e-02, 0.2602144e-01, 0.8381479e-01, &
         0.2377558e+00, 0.5596477e+00, 0.8785649e+00, 0.1245335e+01, &
         0.1575926e+01, 0.1921143e+01, 0.2079966e+01, 0.1798919e+01, &
         0.1101398e+01, 0.8241425e+00, 0.1203991e-11, 0.5900688e-11, &
         0.1824277e-10, 0.5389456e-10, 0.1671786e-09, 0.5994772e-09, &
         0.2299403e-08, 0.8175137e-08, 0.2941320e-07, 0.1066256e-06, &
         0.3856113e-06, 0.1395808e-05, 0.5051475e-05, 0.1828509e-04, &
         0.6619136e-04, 0.2395459e-03, 0.8658932e-03, 0.3116101e-02, &
         0.1105803e-01, 0.3770426e-01, 0.1173082e+00, 0.3180817e+00, &
         0.6740167e+00, 0.9669973e+00, 0.1344094e+01, 0.1676006e+01, &
         0.1949607e+01, 0.2042465e+01, 0.1603115e+01, 0.8707204e+00, &
         0.3155373e-11, 0.1247106e-11, 0.9425561e-11, 0.2065244e-10, &
         0.7427725e-10, 0.2928719e-09, 0.1081926e-08, 0.3947230e-08, &
         0.1429128e-07, 0.5198186e-07, 0.1883016e-06, 0.6817618e-06, &
         0.2467739e-05, 0.8929537e-05, 0.3232451e-04, 0.1170097e-03, &
         0.4233310e-03, 0.1528165e-02, 0.5475521e-02, 0.1917978e-01, &
         0.6327049e-01, 0.1858341e+00, 0.4656517e+00, 0.8128141e+00, &
         0.1142859e+01, 0.1475338e+01, 0.1820214e+01, 0.2086410e+01, &
         0.1925889e+01, 0.1240782e+01, 0.1531637e-11, 0.2570500e-11, &
         0.1066159e-11, 0.1496788e-10, 0.5043575e-10, 0.1727316e-09, &
         0.6206500e-09, 0.2259410e-08, 0.8238917e-08, 0.2965713e-07, &
         0.1075832e-06, 0.3893423e-06, 0.1409170e-05, 0.5098709e-05, &
         0.1845820e-04, 0.6681245e-04, 0.2417935e-03, 0.8740014e-03, &
         0.3145104e-02, 0.1115909e-01, 0.3803171e-01, 0.1182230e+00, &
         0.3202052e+00, 0.6765814e+00, 0.9693897e+00, 0.1346075e+01, &
         0.1677899e+01, 0.1949971e+01, 0.2042721e+01, 0.1599026e+01/),&
         (/30,1,6/))

    qbrqs(1:30,1:1,43:48)=reshape( (/&
         0.9917659e-12, 0.1381379e-11, 0.3523106e-11, 0.9577097e-11, &
         0.3459986e-10, 0.1411091e-09, 0.4419279e-09, 0.1553517e-08, &
         0.5765216e-08, 0.2065135e-07, 0.7509527e-07, 0.2717656e-06, &
         0.9833307e-06, 0.3558670e-05, 0.1288058e-04, 0.4662503e-04, &
         0.1687613e-03, 0.6103279e-03, 0.2200291e-02, 0.7850773e-02, &
         0.2717153e-01, 0.8719374e-01, 0.2460894e+00, 0.5732117e+00, &
         0.8879268e+00, 0.1258482e+01, 0.1589697e+01, 0.1929189e+01, &
         0.2068972e+01, 0.1783306e+01, 0.3368454e-12, 0.1753294e-11, &
         0.1704585e-12, 0.3925133e-11, 0.2875746e-10, 0.8277545e-10, &
         0.3136528e-09, 0.1162799e-08, 0.4243529e-08, 0.1521952e-07, &
         0.5533235e-07, 0.1997696e-06, 0.7227579e-06, 0.2614614e-05, &
         0.9465122e-05, 0.3426161e-04, 0.1240174e-03, 0.4486567e-03, &
         0.1619298e-02, 0.5798678e-02, 0.2027742e-01, 0.6662144e-01, &
         0.1944622e+00, 0.4824011e+00, 0.8250818e+00, 0.1162402e+01, &
         0.1492890e+01, 0.1841950e+01, 0.2096908e+01, 0.1904410e+01, &
         -0.1283701e-11, 0.2700828e-12, 0.5694661e-11, 0.1144088e-11, &
         0.1337860e-10, 0.5726562e-10, 0.2234361e-09, 0.8505050e-09, &
         0.3057868e-08, 0.1090547e-07, 0.3982935e-07, 0.1442852e-06, &
         0.5222154e-06, 0.1889433e-05, 0.6839028e-05, 0.2475330e-04, &
         0.8960722e-04, 0.3242505e-03, 0.1171335e-02, 0.4206709e-02, &
         0.1483641e-01, 0.4977831e-01, 0.1503363e+00, 0.3922572e+00, &
         0.7523814e+00, 0.1054152e+01, 0.1407214e+01, 0.1737990e+01, &
         0.2000353e+01, 0.2026293e+01, 0.1193975e-11, 0.1670941e-11, &
         0.5259805e-11, 0.4074115e-11, 0.2072773e-10, 0.5170789e-10, &
         0.1838727e-09, 0.6078937e-09, 0.2173212e-08, 0.7777988e-08, &
         0.2828418e-07, 0.1023848e-06, 0.3703800e-06, 0.1340259e-05, &
         0.4847633e-05, 0.1754610e-04, 0.6351911e-04, 0.2298873e-03, &
         0.8310285e-03, 0.2991342e-02, 0.1062301e-01, 0.3629179e-01, &
         0.1133487e+00, 0.3088466e+00, 0.6626130e+00, 0.9566727e+00, &
         0.1335223e+01, 0.1667487e+01, 0.1948545e+01, 0.2040859e+01, &
         -0.1518614e-11, 0.1025309e-11, 0.2377287e-12, 0.8970014e-11, &
         0.3335786e-11, 0.3682587e-10, 0.1141844e-09, 0.4113278e-09, &
         0.1523402e-08, 0.5454212e-08, 0.1970126e-07, 0.7100462e-07, &
         0.2571013e-06, 0.9296294e-06, 0.3366042e-05, 0.1218411e-04, &
         0.4409765e-04, 0.1596165e-03, 0.5773018e-03, 0.2081709e-02, &
         0.7433035e-02, 0.2577847e-01, 0.8309824e-01, 0.2359817e+00, &
         0.5567063e+00, 0.8765479e+00, 0.1242425e+01, 0.1572880e+01, &
         0.1919152e+01, 0.2082179e+01, 0.3449489e-12, 0.1323594e-12, &
         0.2524434e-11, 0.7129730e-11, 0.1099827e-10, 0.2720793e-10, &
         0.9374397e-10, 0.2881573e-09, 0.1014721e-08, 0.3635337e-08, &
         0.1330892e-07, 0.4817363e-07, 0.1743618e-06, 0.6308808e-06, &
         0.2282745e-05, 0.8262761e-05, 0.2990859e-04, 0.1082676e-03, &
         0.3917242e-03, 0.1414399e-02, 0.5071576e-02, 0.1780289e-01, &
         0.5903469e-01, 0.1748266e+00, 0.4436502e+00, 0.7959911e+00, &
         0.1116611e+01, 0.1453352e+01, 0.1792378e+01, 0.2065975e+01/),&
         (/30,1,6/))

    qbrqs(1:30,1:1,49:50)=reshape( (/&
         -0.9806707e-13, 0.1529088e-11, 0.2389664e-12, 0.2416812e-11, &
         -0.3707256e-12, 0.1301249e-10, 0.6405868e-10, 0.1982637e-09, &
         0.7039382e-09, 0.2456551e-08, 0.8758652e-08, 0.3174901e-07, &
         0.1151133e-06, 0.4168501e-06, 0.1507989e-05, 0.5457498e-05, &
         0.1975455e-04, 0.7150998e-04, 0.2587977e-03, 0.9353363e-03, &
         0.3364424e-02, 0.1192217e-01, 0.4049582e-01, 0.1250701e+00, &
         0.3359799e+00, 0.6949816e+00, 0.9873564e+00, 0.1360237e+01, &
         0.1691364e+01, 0.1953644e+01, 0.1310020e-11, 0.7826037e-12, &
         -0.8811858e-12, 0.2869265e-11, 0.2279223e-11, 0.5720607e-12, &
         0.1676336e-10, 0.9887384e-10, 0.3575822e-09, 0.1381762e-08, &
         0.4984234e-08, 0.1798714e-07, 0.6499247e-07, 0.2345360e-06, &
         0.8492439e-06, 0.3073006e-05, 0.1112344e-04, 0.4026809e-04, &
         0.1457573e-03, 0.5272286e-03, 0.1901805e-02, 0.6798243e-02, &
         0.2365158e-01, 0.7678756e-01, 0.2202472e+00, 0.5297817e+00, &
         0.8581598e+00, 0.1214843e+01, 0.1544259e+01, 0.1896759e+01/),&
         (/30,1,2/))

    ako3(1,:)=  (/12103.15537, 12402.63363,  12815.75276, 13293.58735, 12792.67732,  5263.93060/)
    ako3(2,:)=  (/ 4073.95522,  3824.69235,   3430.8699 ,  2683.45418,  1705.58659,   294.00564/)
    ako3(3,:)=  (/  256.02950,   246.52991,    229.39518,   189.37447,   123.25784,    19.16440/)
    ako3(4,:)=  (/    0.02424,     0.02417,     0.02402 ,     0.02357,     0.02223,     0.00373/)

    akco2(1,:)= (/  21.55629,    19.04724,    15.82831,    11.24202,      6.87763,     1.93565/)
    akco2(2,:)= (/   1.41477,     1.19470,     0.94231,     0.63257,      0.38015,     0.06712/)
    akco2(3,:)= (/   0.02091,     0.01822,     0.01471,     0.00950,      0.00443,     0.00784/)
    akco2(4,:)= (/4285.18491,  4190.41876,  3956.55409,  3319.57528,   2275.73601,   527.26813/)
    akco2(5,:)= (/ 490.02297,   420.10700,   331.10330,   211.02566,    109.50739,    18.39552/)
    akco2(6,:)= (/  52.24044,    42.14958,    31.06085,    18.17948,      8.80769,     1.40437/)

    akh2o(01,:)=(/   0.02080,     0.01550,     0.01040,     0.00510,      0.00200,     0.00062/)
    akh2o(02,:)=(/   0.17070,     0.12910,     0.08630,     0.04300,      0.01710,     0.00043/)
    akh2o(03,:)=(/   3.69350,     3.03920,     2.18270,     1.18620,      0.48190,     0.00012/)
    akh2o(04,:)=(/   0.09550,     0.07170,     0.04775,     0.02390,      0.00970,     0.00096/)
    akh2o(05,:)=(/   0.56670,     0.42490,     0.28390,     0.14190,      0.05620,     0.00565/)
    akh2o(06,:)=(/   4.42380,     3.37860,     2.27150,     1.14360,      0.45980,     0.04582/)
    akh2o(07,:)=(/  52.2782 ,    46.7368 ,    37.4543 ,    22.1443 ,      9.48870,     0.96239/)
    akh2o(08,:)=(/   0.74700,     0.55860,     0.37410,     0.18640,      0.07480,     0.00757/)
    akh2o(09,:)=(/   6.21910,     4.71170,     3.17460,     1.57760,      0.63070,     0.06400/)
    akh2o(10,:)=(/ 108.8498 ,    93.64720,    70.7601 ,    40.4311 ,     16.7743 ,     1.64448/)
    akh2o(11,:)=(/   5.32480,     4.02140,     2.69230,     1.33950,      0.53460,     0.05812/)
    akh2o(12,:)=(/  37.4911 ,    28.7735 ,    19.3551 ,     9.8670 ,      3.93660,     0.48536/)
    akh2o(13,:)=(/ 399.4563 ,   372.4726 ,   317.0208 ,   199.0702 ,     91.07190,     9.51138/)
    akh2o(14,:)=(/  19.5458 ,    14.6823 ,     9.89610,     4.88830,      1.96260,     0.19703/)
    akh2o(15,:)=(/ 129.8270 ,   100.9301 ,    68.2980 ,    34.9858 ,     13.9141 ,     1.39263/)
    akh2o(16,:)=(/1289.8795 ,  1225.3730 ,  1077.8933 ,   713.0905 ,    338.1374 ,    34.83768/)
    akh2o(17,:)=(/  10.1830 ,     7.64440,     5.09240,     2.54900,      1.04120,     0.10272/)
    akh2o(18,:)=(/  61.2038 ,    46.4646 ,    31.2440 ,    15.7152 ,      6.24550,     0.62696/)
    akh2o(19,:)=(/ 363.8096 ,   297.8458 ,   214.1972 ,   112.6198 ,     45.6958 ,     4.55650/)
    akh2o(20,:)=(/2005.95   ,  2053.91   ,  1991.06   ,  1561.74   ,    825.730  ,    89.7681 /)
    akh2o(21,:)=(/  15.3380 ,    11.5228 ,     8.5296 ,     3.8592 ,      1.54910,     0.15449/)
    akh2o(22,:)=(/  72.4480 ,    54.6480 ,    32.2028 ,    18.2840 ,      7.31640,     0.73264/)
    akh2o(23,:)=(/ 440.2140 ,   352.1620 ,   512.833  ,   128.293  ,     51.6640 ,     5.14812/)
    akh2o(24,:)=(/3343.51   ,  3347.37   ,   512.833  ,   233.720  ,   1182.99   ,   124.7161 /)
    akh2o(25,:)=(/   3.76110,     2.82540,     1.88200,     0.94320,      0.37750,     0.03777/)
    akh2o(26,:)=(/  29.5500 ,    22.4360 ,    15.1239 ,     7.57360,      3.02760,     0.30409/)
    akh2o(27,:)=(/ 424.620  ,   379.480  ,   304.175  ,   180.650  ,     78.0440 ,     7.94202/)
    akh2o(28,:)=(/   1.25230,     0.93840,     0.62750,     0.31300,      0.12450,     0.01251/)
    akh2o(29,:)=(/  27.4300 ,    22.1200 ,    15.7100 ,     8.2700 ,      3.37560,     0.34112/)
    akh2o(30,:)=(/   0.14650,     0.11010,     0.07330,     0.03670,      0.01480,     0.01476/)
    akh2o(31,:)=(/   4.95e-8,     4.95e-8,     4.95e-8,     4.95e-8,      4.95e-8,     4.95e-8/)
    akh2o(32,:)=(/   6.06e-8,     6.06e-8,     6.06e-8,     6.06e-8,      6.06e-8,     6.06e-8/)
    akh2o(33,:)=(/   1.12e-7,     1.12e-7,     1.12e-7,     1.12e-7,      1.12e-7,     1.12e-7/)
    akh2o(34,:)=(/   1.32700,     1.38470,     0.97515,     0.50750,      0.20570,     0.02090/)
    akh2o(35,:)=(/   0.07038,     0.05380,     0.03595,     0.01790,      0.00720,     0.00068/)
    akh2o(36,:)=(/   0.00782,     0.00590,     0.00393,     0.00190,      0.00078,     0.00002/)
    akh2o(37,:)=(/   0.00128,     0.00096,     0.00642,     0.00032,      0.00013,     0.00010/)
    akh2o(38,:)=(/   3.00300,     2.91330,     1.97760,     1.00080,      0.40170,     0.0402 /)
    akh2o(39,:)=(/   0.13090,     0.09870,     0.06580,     0.03290,      0.00131,     0.00131/)
    akh2o(40,:)=(/   0.01379,     0.01350,     0.00690,     0.00350,      0.00014,     0.00014/)
    akh2o(41,:)=(/  29.2600 ,    27.7800 ,    19.0800 ,     9.7400 ,      3.92000,     0.39310/)
    akh2o(42,:)=(/   1.27900,     0.96860,     0.64580,     0.32290,      0.12910,     0.01290/)
    akh2o(43,:)=(/   0.13480,     0.10150,     0.06770,     0.03380,      0.01350,     0.00135/)
    akh2o(44,:)=(/ 716.500  ,   666.670  ,   491.250  ,   266.300  ,    109.670  ,    11.2050 /)
    akh2o(45,:)=(/  39.7800 ,    29.7300 ,    19.8800 ,     9.9500 ,      3.98000,     0.37610/)
    akh2o(46,:)=(/   4.47600,     3.35300,     2.23640,     1.11900,      0.44740,     0.00996/)
    akh2o(47,:)=(/   0.73960,     0.55410,     0.36940,     0.18460,      0.07390,     0.06010/)
    akh2o(48,:)=(/5631.00   ,  6090.99   ,  4475.54   ,  2422.79   ,    998.340  ,   100.540  /)
    akh2o(49,:)=(/ 414.800  ,   309.620  ,   206.840  ,   103.450  ,     41.3600 ,     4.13620/)
    akh2o(50,:)=(/  53.4600 ,    40.2600 ,    26.8600 ,    13.4300 ,      5.37520,     0.53760/)
    akh2o(51,:)=(/   9.56300,     7.17130,     4.78020,     2.38970,      0.95580,     0.09560/)
    akh2o(52,:)=(/2779.00   ,  2844.55   ,  1937.23   ,   982.710  ,    394.850  ,    39.5000 /)
    akh2o(53,:)=(/ 134.600  ,   108.410  ,    72.2400 ,    36.1100 ,     14.4000 ,     1.44600/)
    akh2o(54,:)=(/   0.008995,    8.62480,     5.75790,     2.8800 ,      1.15300,     0.10640/)

    treal(1,:)=   (/1.38    , 1.37    , 1.365   , 1.36    , 1.357   ,&
                    1.353   , 1.342   , 1.3401  , 1.3372  , 1.336   ,&
                    1.3355  , 1.3342  , 1.333   , 1.3323  , 1.3322  ,&
                    1.332   , 1.332   , 1.332   , 1.331   , 1.330   ,&
                    1.329   , 1.328   , 1.327   , 1.323   , 1.319   ,&
                    1.313   , 1.305   , 1.295   , 1.252   , 1.455   ,&
                    1.362   , 1.334   , 1.326   , 1.320   , 1.308   ,&
                    1.283   , 1.278   , 1.313   , 1.326   , 1.310   ,&
                    1.293   , 1.270   , 1.227   , 1.164   , 1.173   ,&
                    1.287   , 1.415   , 1.508   , 1.541   , 1.669/)

    treal(2,:)=     (/1.3480e+00,  1.3407e+00,  1.3353e+00,  1.3307e+00,  1.3275e+00,&
                    1.3231e+00,  1.3177e+00,  1.3165e+00,  1.3147e+00,  1.3140e+00,&
                    1.3133e+00,  1.3115e+00,  1.3098e+00,  1.3080e+00,  1.3077e+00,&
                    1.3071e+00,  1.3065e+00,  1.3056e+00,  1.3047e+00,  1.3038e+00,&
                    1.3028e+00,  1.3014e+00,  1.2997e+00,  1.2955e+00,  1.2906e+00,&
                    1.2837e+00,  1.2717e+00,  1.2629e+00,  1.1815e+00,  1.4310e+00,&
                    1.3874e+00,  1.3473e+00,  1.3427e+00,  1.3288e+00,  1.3140e+00,&
                    1.2972e+00,  1.2908e+00,  1.3155e+00,  1.3186e+00,  1.3189e+00,&
                    1.3125e+00,  1.2848e+00,  1.2268e+00,  1.1921e+00,  1.4906e+00,&
                    1.5588e+00,  1.5143e+00,  1.4427e+00,  1.3047e+00,  1.4859e+00/)

    ttmag(1,:)=     (/1.e-15  , 1.e-15  , 1.e-15  , 1.e-15  , 1.e-15  ,&
                    1.e-15  , 1.e-15  , 1.e-15  , 1.e-15  , 1.e-15  ,&
                    1.e-15  , 1.e-15  , 1.e-15  , 2.70e-08, 1.82e-8 ,&
                    2.11e-8,  1.68e-07, 6.66e-08, 1.24e-07, 2.94e-07,&
                    8.77e-07, 3.06e-06, 2.05e-06, 2.39e-05, 1.20e-04,&
                    1.18e-04, 6.79e-04, 3.51e-04, 2.39e-03, 0.0442  ,&
                    0.00339 , 0.00833 , 0.0139  , 0.0125  , 0.011   ,&
                    0.015   , 0.075   , 0.086   , 0.039   , 0.035   ,&
                    0.035   , 0.038   , 0.051   , 0.161   , 0.308   ,&
                    0.39    , 0.42    , 0.395   , 0.373   , 0.5/)

    ttmag(2,:)=     (/8.0822e-09,  6.7516e-09,  5.7565e-09,  4.8780e-09,  4.2693e-09,&
                    3.4207e-09,  2.2612e-09,  2.09e-09  ,  1.84e-09  ,  1.7429e-09,&
                    7.5115e-09,  2.64e-09  ,  5.12e-09  ,  2.4450e-08,  1.89e-08  ,&
                    2.08e-08  ,  3.8345e-08,  7.5497e-08,  1.3765e-07,  2.3306e-07,&
                    5.2670e-07,  1.7241e-06,  2.2288e-06,  7.3358e-05,  4.8415e-04,&
                    2.6270e-04,  1.2123e-03,  3.0644e-04,  2.7738e-02,  2.7197e-01,&
                    7.5589e-03,  1.6397e-02,  1.9980e-02,  1.2469e-02,  1.4622e-02,&
                    2.5275e-02,  5.4226e-02,  6.5501e-02,  5.6727e-02,  5.4499e-02,&
                    4.6806e-02,  4.0056e-02,  4.9961e-02,  3.0257e-01,  3.6528e-01,&
                    1.7601e-01,  8.6618e-02,  3.4573e-02,  8.5167e-02,  4.4443e-01/)

    !14 - rrtm_sw bands
    ! rrtmg_sw 14 spectral intervals (microns):
    !  3.846 -  3.077
    !  3.077 -  2.500
    !  2.500 -  2.150
    !  2.150 -  1.942
    !  1.942 -  1.626
    !  1.626 -  1.299
    !  1.299 -  1.242
    !  1.242 -  0.7782
    !  0.7782-  0.6250
    !  0.6250-  0.4415
    !  0.4415-  0.3448
    !  0.3448-  0.2632
    !  0.2632-  0.2000
    ! 12.195 -  3.846

    casee(1,:,1)=(/0.067605, 0.083813, 0.095147, 0.108877, 0.125733, 0.183275, 0.260600, 0.419425 &
    , 0.766550, 1.364483, 2.039000, 2.710200, 3.038000, 0.044794/)
    casee(1,:,2)=(/0.105700, 0.120133, 0.130200, 0.142367, 0.156767, 0.205800, 0.271967, 0.406000 &
    , 0.694517, 1.184967, 1.778000, 2.453400, 2.819000, 0.076972/)
    casee(1,:,3)=(/0.077145, 0.095350, 0.107637, 0.121200, 0.137267, 0.190025, 0.260733, 0.407925 &
    , 0.729217, 1.279900, 1.915500, 2.579200, 2.917000, 0.048857/)
    casee(1,:,4)=(/0.089590, 0.105820, 0.116967, 0.130000, 0.145267, 0.196150, 0.264333, 0.402575 &
    , 0.699233, 1.213633, 1.831750, 2.518200, 2.886000, 0.061656/)
    casee(1,:,5)=(/0.208850, 0.228067, 0.240467, 0.254733, 0.269500, 0.318325, 0.383900, 0.517662 &
    , 0.799017, 1.271833, 1.832750, 2.453200, 2.784000, 0.158391/)
    casee(1,:,6)=(/0.199050, 0.229233, 0.247867, 0.266200, 0.283367, 0.328825, 0.384200, 0.477125 &
    , 0.665450, 1.000850, 1.452000, 2.045400, 2.402000, 0.138197/)
    casee(1,:,7)=(/0.282000, 0.306167, 0.321200, 0.334300, 0.344567, 0.371800, 0.408900, 0.483887 &
    , 0.650083, 0.944833, 1.357250, 1.932600, 2.283000, 0.211903/)
    casee(1,:,8)=(/0.074305, 0.088370, 0.098230, 0.110390, 0.125700, 0.178750, 0.250367, 0.404125 &
    , 0.739767, 1.324250, 1.974500, 2.625800, 2.950000, 0.050663/)
    casee(1,:,9)=(/0.111750, 0.132267, 0.146100, 0.160500, 0.177033, 0.226225, 0.289433, 0.406663 &
    , 0.635017, 1.015317, 1.491750, 2.082000, 2.422000, 0.075764/)
    casee(1,:,10)=(/0.197650, 0.244600, 0.272867, 0.297167, 0.321467, 0.378375, 0.444433, 0.553400 &
    , 0.766833, 1.123350, 1.585000, 2.173400, 2.507000, 0.113190/)
    casee(1,:,11)=(/0.077840, 0.094697, 0.106310, 0.119733, 0.135833, 0.189625, 0.261867, 0.410200 &
    , 0.732133, 1.286433, 1.930000, 2.604800, 2.950000, 0.051535/)
    casee(1,:,12)=(/0.077840, 0.094697, 0.106310, 0.119733, 0.135833, 0.189625, 0.261867, 0.410200 &
    , 0.732133, 1.286433, 1.930000, 2.604800, 2.950000, 0.051535/)

    casew(1,:,1)=(/ 0.831850, 0.817933, 0.813100, 0.813600, 0.816700, 0.833625, 0.854900, 0.881525 &
    , 0.908383, 0.924267, 0.931950, 0.933200, 0.931600, 0.856627/)
    casew(1,:,2)=(/ 0.835150, 0.810867, 0.799733, 0.793967, 0.790900, 0.798675, 0.814367, 0.838187 &
    , 0.863067, 0.883717, 0.897575, 0.904800, 0.905400, 0.871067/)
    casew(1,:,3)=(/ 0.845550, 0.831900, 0.825700, 0.823533, 0.823633, 0.834400, 0.850533, 0.872075 &
    , 0.895383, 0.913900, 0.923775, 0.926500, 0.925500, 0.861173/)
    casew(1,:,4)=(/ 0.868650, 0.852367, 0.845167, 0.842033, 0.840700, 0.848325, 0.861333, 0.881663 &
    , 0.905367, 0.917850, 0.925325, 0.929800, 0.929800, 0.892560/)
    casew(1,:,5)=(/ 0.895850, 0.877533, 0.868400, 0.861367, 0.854767, 0.849975, 0.851200, 0.860438 &
    , 0.876617, 0.890533, 0.901600, 0.909120, 0.910200, 0.922853/)
    casew(1,:,6)=(/ 0.881800, 0.866867, 0.859233, 0.853533, 0.847567, 0.840575, 0.837233, 0.840862 &
    , 0.851483, 0.856567, 0.867725, 0.883760, 0.888800, 0.903793/)
    casew(1,:,7)=(/ 0.926500, 0.912400, 0.904333, 0.897767, 0.890233, 0.878450, 0.869633, 0.864238 &
    , 0.865633, 0.870083, 0.883050, 0.900900, 0.907100, 0.945307/)
    casew(1,:,8)=(/ 0.738650, 0.709900, 0.698100, 0.695567, 0.698700, 0.723450, 0.756733, 0.792437 &
    , 0.825550, 0.848350, 0.861300, 0.864980, 0.863300, 0.778507/)
    casew(1,:,9)=(/ 0.827700, 0.807633, 0.798100, 0.793200, 0.790900, 0.795350, 0.805633, 0.822563 &
    , 0.844867, 0.857567, 0.868825, 0.880140, 0.882700, 0.853527/)
    casew(1,:,10)=(/ 0.911500, 0.903067, 0.898100, 0.894167, 0.890467, 0.884675, 0.880500, 0.879000 &
    , 0.879533, 0.886917, 0.897550, 0.907280, 0.909700, 0.911600/)
    casew(1,:,11)=(/ 0.849800, 0.834833, 0.828500, 0.826767, 0.827200, 0.838825, 0.855600, 0.878563 &
    , 0.903217, 0.918750, 0.927225, 0.929940, 0.929000, 0.871673/)
    casew(1,:,12)=(/ 0.849800, 0.834833, 0.828500, 0.826767, 0.827200, 0.838825, 0.855600, 0.878563 &
    , 0.903217, 0.918750, 0.927225, 0.929940, 0.929000, 0.871673/)

    caseg(1,:,1)=(/0.675000, 0.640900, 0.617633, 0.590567, 0.560033, 0.515075, 0.488967, 0.508400 &
    , 0.575650, 0.651383, 0.702425, 0.734180, 0.748300, 0.684293/)
    caseg(1,:,2)=(/0.695700, 0.678200, 0.666400, 0.648033, 0.623167, 0.573350, 0.532967, 0.526863 &
    , 0.571833, 0.645600, 0.701325, 0.740420, 0.758600, 0.697947/)
    caseg(1,:,3)=(/0.690100, 0.664300, 0.645333, 0.621700, 0.593533, 0.546550, 0.514500, 0.524000 &
    , 0.581850, 0.654400, 0.705625, 0.739160, 0.754100, 0.686380/)
    caseg(1,:,4)=(/0.684800, 0.664167, 0.650533, 0.631667, 0.606933, 0.560900, 0.525367, 0.524025 &
    , 0.572283, 0.645533, 0.699675, 0.736560, 0.753700, 0.688613/)
    caseg(1,:,5)=(/0.696100, 0.694333, 0.693900, 0.686767, 0.672767, 0.637625, 0.604567, 0.587137 &
    , 0.608683, 0.664400, 0.709800, 0.741320, 0.756500, 0.695767/)
    caseg(1,:,6)=(/0.697600, 0.699967, 0.702300, 0.700933, 0.693900, 0.670700, 0.643867, 0.616875 &
    , 0.615300, 0.655700, 0.702475, 0.743380, 0.764000, 0.691247/)
    caseg(1,:,7)=(/0.703850, 0.710700, 0.715767, 0.716867, 0.711033, 0.690925, 0.665967, 0.635413 &
    , 0.627867, 0.662417, 0.705475, 0.744560, 0.764100, 0.699920/)
    caseg(1,:,8)=(/0.697550, 0.659633, 0.632933, 0.601400, 0.566900, 0.514425, 0.482867, 0.498663 &
    , 0.562783, 0.640817, 0.695150, 0.732500, 0.750100, 0.696553/)
    caseg(1,:,9)=(/0.713200, 0.697267, 0.686300, 0.672900, 0.656667, 0.623875, 0.595767, 0.586200 &
    , 0.610167, 0.663083, 0.711775, 0.751240, 0.770100, 0.710000/)
    caseg(1,:,10)=(/0.710700, 0.710533, 0.709433, 0.706600, 0.701267, 0.684850, 0.665300, 0.643262 &
    , 0.641733, 0.679233, 0.721300, 0.757620, 0.774500, 0.681393/)
    caseg(1,:,11)=(/0.683900, 0.657267, 0.638700, 0.615500, 0.587467, 0.540975, 0.509267, 0.518550 &
    , 0.576717, 0.650700, 0.702800, 0.736780, 0.752100, 0.686933/)
    caseg(1,:,12)=(/0.683900, 0.657267, 0.638700, 0.615500, 0.587467, 0.540975, 0.509267, 0.518550 &
    , 0.576717, 0.650700, 0.702800, 0.736780, 0.752100, 0.686933/)

    caser(1,1)= 0.207100
    caser(1,2)= 0.239200
    caser(1,3)= 0.212100
    caser(1,4)= 0.224100
    caser(1,5)= 0.355900
    caser(1,6)= 0.323200
    caser(1,7)= 0.422600
    caser(1,8)= 0.201400
    caser(1,9)= 0.242900
    caser(1,10)= 0.293500
    caser(1,11)= 0.214200
    caser(1,12)= 0.214200

    cased(1,1)= 1.30500
    cased(1,2)= 1.24600
    cased(1,3)= 1.25500
    cased(1,4)= 1.23200
    cased(1,5)= 1.38000
    cased(1,6)= 1.18200
    cased(1,7)= 1.15400
    cased(1,8)= 1.27200
    cased(1,9)= 1.14200
    cased(1,10)= 1.48500
    cased(1,11)= 1.26400
    cased(1,12)= 1.26400

    ! derived parameters
    !
    sq3  =sqrt(3.)
    jdble   =   2*nlayer
    jn    =   jdble-1
    tpi   =   2.*pi
    cpcon   =   1.006
    fdegday =   1.0e-4*g*scday/cpcon

!--(kml2)----------------------------------------------------------
    casew_tuv=0.0
    !print *,'ii. nWave=',nWave
  do n = 1, nsitesaot
    do i = 1, nmodelsaot
      do w=1, nbndsw
        casew_tuv(w,n) = casew_tuv(w,n) + casew(i,w,n)
      end do
    end do

    do w = 1, nwave
      casew_tuv(w,n) = casew_tuv(w,n) / (nmodelsaot)
    end do
  end do

  end subroutine setupraddata


end module optical
